
#pragma once

#include "stdafx.h"
#include "sample.h"
#include "sampleDlg.h"

#include <mtype.h>
#include <math.h>
#include <time.h>
#include <usgstrrf.h>
#include <afxdlgs.h>

#include "DIBSectn.h"
#include "OptionsDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#define MAX_DATE 20
#endif

#define new DEBUG_NEW
#define MAX_DATE 20

#ifdef _WIN64
#define MAX_CINE_BUFFER_SIZE_MB 4000
#else
#define MAX_CINE_BUFFER_SIZE_MB 1200
#endif

volatile LONG g_thread_finished;
volatile LONG g_app_closing;
volatile int freeze_after_scanning_frames_num;
volatile bool using_rf_stream = true;

// create single ultrasound control
void CsampleDlg::CreateUsgControl(IUsgDataView* data_view, const IID& type_id, ULONG scan_mode, ULONG stream_id, void** ctrl)
{
	IUsgControl* ctrl2;
	ctrl2 = NULL;

	if (data_view == NULL) return;

	data_view->GetControlObj(&type_id, scan_mode, stream_id, &ctrl2);

	if (ctrl2 != NULL)
	{
		HRESULT hr;
		hr = ctrl2->QueryInterface(type_id, (void**)ctrl);
		if (hr != S_OK)
			*ctrl = NULL;
		SAFE_RELEASE(ctrl2);
	}
}

// create all required ultrasound controls
void CsampleDlg::CreateUsgControls()
{
	HRESULT hr = S_OK;
	IUnknown* tmp_obj = NULL;

	ShowWaitCursor();

	do
	{
		using_regs_line_mode = false;

		// create main Usgfw2 library object
		if (usgfw2 == NULL)
			hr = CoCreateInstance(CLSID_Usgfw2, NULL, CLSCTX_INPROC_SERVER, IID_IUsgfw2, (LPVOID*)&usgfw2);
		if (hr != S_OK)
		{
			usgfw2 = NULL;
			break;
		}

		// attach to device change connection point

		if (m_usg_device_change_cpnt == NULL)
		{
			IConnectionPointContainer* cpc = NULL;
			hr = usgfw2->QueryInterface(IID_IConnectionPointContainer, (void**)&cpc);
			if (hr != S_OK)
				cpc = NULL;

			if (cpc != NULL)
				hr = cpc->FindConnectionPoint(IID_IUsgDeviceChangeSink, &m_usg_device_change_cpnt);

			if (hr != S_OK)
			{
				m_usg_device_change_cpnt = NULL;
				m_usg_device_change_cpnt_cookie = 0;
			}
			SAFE_RELEASE(cpc);

			if (m_usg_device_change_cpnt != NULL)
				hr = m_usg_device_change_cpnt->Advise((IUnknown*)(IUsgDeviceChangeSink*)this, &m_usg_device_change_cpnt_cookie);
		}


		IUsgCollection* probes_collection = NULL;

		// get collection of connected probes
		tmp_obj = NULL;
		usgfw2->get_ProbesCollection(&tmp_obj);
		if (tmp_obj == NULL)
		{

			probes_collection = NULL;
			break;

		}
		hr = tmp_obj->QueryInterface(IID_IUsgCollection, (void**)&probes_collection);
		tmp_obj->Release();
		if ((hr != S_OK) || (probes_collection == NULL))
		{
			probes_collection = NULL;
			break;
		}

		// get the number of connected probes
		LONG probes_count = 0;
		probes_collection->get_Count(&probes_count);
		if (probes_count == 0)
		{
			probes_collection->Release();
			probes_collection = NULL;
			break;
		}

		if (m_probe_idx >= probes_count)
			m_probe_idx = 0;
		if (m_probe_idx < 0)
			m_probe_idx = 0;

		tmp_obj = NULL;
		probes_collection->Item(m_probe_idx, &tmp_obj);
		probes_collection->Release();
		probes_collection = NULL;

		if (tmp_obj == NULL)
		{
			probe = NULL;
			break;
		}
		hr = tmp_obj->QueryInterface(IID_IProbe, (void**)&probe);
		tmp_obj->Release();

		if ((hr != S_OK) || (probe == NULL))
		{
			probe = NULL;
			break;
		}

		// get probe name
		CComBSTR bstr;
		probe->get_Name(&bstr);
		//bstr += L"    v";
		SetDlgItemText(IDC_PROBE_NAME_BUTTON, bstr);

		// create main ultrasound scanning object for selected probe
		usgfw2->CreateDataView(probe, &data_view);

		if (data_view == NULL)
		{
			break;
		}

		data_view->put_ScanState(SCAN_STATE_STOP);
		StopTimer();

		IUsgScanMode* mode = NULL;
		data_view->GetScanModeObj(SCAN_MODE_B, &mode);
		if (mode == NULL)
		{
			break;
		}

		// get B mixer control //
		mode->GetMixerControl(SCAN_MODE_B, 0, &mixer_control);
		mode->Release();
		mode = NULL;
		if (mixer_control == NULL)
		{
			break;
		}

		// set B scanning mode
		data_view->put_ScanMode(SCAN_MODE_B);

		// enable B RF data
		IUsgStreamEnable *pUsgStreamEnable = NULL;
		CreateUsgControl(data_view, IID_IUsgStreamEnable, SCAN_MODE_RFB, 0, (void**)&pUsgStreamEnable);
		if (pUsgStreamEnable)
		{
			pUsgStreamEnable->put_Enable(TRUE);
			pUsgStreamEnable->Release();
		}

		usg_wnd = (HWND)(*this);

		// set ultrasound output window (panel)
		mixer_control->SetOutputWindow((LONG)usg_wnd);

		tagRECT rect1;
		rect1.left = 20;
		rect1.top = 20;
		rect1.right = rect1.left + img_w;
		rect1.bottom = rect1.top + img_h;

		// set B ultrasound output rectangle
		mixer_control->SetOutputRect(&rect1);

		// set background color that surrounds B ultrasound image
		tagPALETTEENTRY clr1;
		clr1.peRed = 150;
		clr1.peGreen = 150;
		clr1.peBlue = 150;
		clr1.peFlags = 0;
		mixer_control->put_BkColor(clr1);

		// create scanning type control
		wideview_is_supported = false;
		compound_is_supported = false;
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgScanType, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
		{
			scan_type_ctrl = (IUsgScanType*)tmp_obj;

			// check what B scanning types are supported
			IUsgValues* usg_values = NULL;
			scan_type_ctrl->get_Values(&usg_values);
			if (usg_values != NULL)
			{
				LONG val_count;
				usg_values->get_Count(&val_count);
				for (LONG i1 = 0; i1 < val_count; i1++)
				{
					VARIANT item;
					LONG val;
					usg_values->Item(i1, &item);
					val = item.lVal;
					if (val == SCAN_TYPE_VIRTUALCONVEX)
						wideview_is_supported = true;
					else if (val == SCAN_TYPE_COMPOUND)
						compound_is_supported = true;
					VariantClear(&item);
				}
				SAFE_RELEASE(usg_values);
			}
		}
		else
		{
			scan_type_ctrl = NULL;
		}

		// create depth control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgDepth, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			depth_ctrl = (IUsgDepth*)tmp_obj;
		else
			depth_ctrl = NULL;

		// create view area control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgViewArea, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
		{
			view_area_ctrl = (IUsgViewArea*)tmp_obj;
			// set value to 100%
			view_area_ctrl->put_Current(100);
		}
		else
			view_area_ctrl = NULL;

		// create B mode gain control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgGain, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_gain_ctrl = (IUsgGain*)tmp_obj;
		else
			b_gain_ctrl = NULL;

		// create B mode power control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgPower, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_power_ctrl = (IUsgPower*)tmp_obj;
		else
			b_power_ctrl = NULL;

		// create B mode frequency control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgProbeFrequency3, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_frequency_ctrl = (IUsgProbeFrequency3*)tmp_obj;
		else
			b_frequency_ctrl = NULL;

		// create B mode lines density control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgLineDensity, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_lines_density_ctrl = (IUsgLineDensity*)tmp_obj;
		else
			b_lines_density_ctrl = NULL;

		// create B mode focus control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgFocus, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_focus_ctrl = (IUsgFocus*)tmp_obj;
		else
			b_focus_ctrl = NULL;

		// create B image orientation control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgImageOrientation, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_image_orientation_ctrl = (IUsgImageOrientation*)tmp_obj;
		else
			b_image_orientation_ctrl = NULL;

		// create steering angle control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgSteeringAngle, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			b_steering_angle_ctrl = (IUsgSteeringAngle*)tmp_obj;
		else
			b_steering_angle_ctrl = NULL;

		// create WideView angle control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgViewAngle, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			wideview_angle_ctrl = (IUsgViewAngle*)tmp_obj;
		else
			wideview_angle_ctrl = NULL;

		// create Compound angle control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgCompoundAngle, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			compound_angle_ctrl = (IUsgCompoundAngle*)tmp_obj;
		else
			compound_angle_ctrl = NULL;

		// create Compound frames number control
		compound_frames_enabled = false;
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgCompoundFrames, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
		{
			compound_frames_ctrl = (IUsgCompoundFrames*)tmp_obj;

			compound_frames_enabled = true;
			LONG val_count = 0;
			IUsgValues* usg_values = NULL;
			compound_frames_ctrl->get_Values(&usg_values);
			if (usg_values != NULL)
				usg_values->get_Count(&val_count);
			SAFE_RELEASE(usg_values);
			if (val_count <= 1)
				compound_frames_enabled = false;
		}
		else
			compound_frames_ctrl = NULL;

		// image properties for B stream
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgImageProperties, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			image_properties_b = (IUsgImageProperties*)tmp_obj;
		else
			image_properties_b = NULL;

		// scan line properties for B stream
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgScanLineProperties2, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			scan_line_properties_b = (IUsgScanLineProperties2*)tmp_obj;
		else
			scan_line_properties_b = NULL;

		// RF works only with single focus
		SetSingleStaticFocus();

		// control to set RF window
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgWindowRF, SCAN_MODE_RFB, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			m_pUsgWindowRF = (IUsgWindowRF*)tmp_obj;
		else
			m_pUsgWindowRF = NULL;

		// control to set RF data type
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgDataSourceRF, SCAN_MODE_RFB, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
		{
			m_pUsgDataSourceRF = (IUsgDataSourceRF*)tmp_obj;
			m_pUsgDataSourceRF->put_Current(0);
			rf_data_source_point_id = 4;
		}
		else
			m_pUsgDataSourceRF = NULL;

		// control to get B FPS
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgQualProp, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			m_pUsgQualProp = (IUsgQualProp*)tmp_obj;
		else
			m_pUsgQualProp = NULL;

		// cine control
		tmp_obj = NULL;
		m_pUsgCine = NULL;
		m_pUsgCineB = NULL;
		m_pUsgCineRF = NULL;
		if (m_pUsgWindowRF != NULL)
			using_rf_stream = true;
		else
			using_rf_stream = false;

		CreateUsgControl(data_view, IID_IUsgCineStream, SCAN_MODE_RFB, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
		{
			m_pUsgCineRF = (IUsgCineStream*)tmp_obj;
			m_pUsgCineRF->put_TimeFormat(CINE_FORMAT_FRAME);
		}

		CreateUsgControl(data_view, IID_IUsgCineStream, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
		{
			m_pUsgCineB = (IUsgCineStream*)tmp_obj;
			// for cine control we will use frames-based format
			m_pUsgCineB->put_TimeFormat(CINE_FORMAT_FRAME);
		}

		if (!using_rf_stream)
			m_pUsgCine = m_pUsgCineB;
		else
			m_pUsgCine = m_pUsgCineRF;

		// cine sink control
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgCineSink, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			m_pUsgCineSink = (IUsgCineSink*)tmp_obj;
		else
			m_pUsgCineSink = NULL;

		// create sample grabber callback COM object that can be implemented as class like in
		// this sample or can be implemented as a usual COM object that is created using CoCreateInstance
		m_sample_grabber_callback = new SampleGrabberCallback();
		m_sample_grabber_callback->AddRef();

		// set output window where callback will display received ultrasound data
		m_sample_grabber_callback->SetUsgWnd(this, 20, 20);

		// get ISampleGrabberFilter for desired ultrasound data *stream*
		IUnknown* tmp_obj = NULL;
		CreateUsgControl(data_view, IID_ISampleGrabberFilter, SCAN_MODE_RFB, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			sample_grabber_filter = (ISampleGrabberFilter*)tmp_obj;

		// get sample grabber interface
		IUnknown* tmp_unk = NULL;
		m_sample_grabber_plugin = NULL;
		if (sample_grabber_filter != NULL)
			sample_grabber_filter->get_FilterObj(0, &tmp_unk);

		// get plugin
		if (tmp_unk != NULL)
		{
			hr = tmp_unk->QueryInterface(IID_ISampleGrabber, (void**)&m_sample_grabber_plugin);
			if (hr != S_OK)
				m_sample_grabber_plugin = NULL;
			SAFE_RELEASE(tmp_unk);
		}

		// register COM callback object and specify method(s) to callback
		if ((m_sample_grabber_plugin != NULL) && (m_sample_grabber_callback != NULL))
		{
			hr = m_sample_grabber_plugin->SetCallback(m_sample_grabber_callback, 0);
			m_sample_grabber_callback->SetPlugin(m_sample_grabber_plugin);
		}


		// create scan-converter callback COM object that can be implemented as class like in
		// this sample or can be implemented as a usual COM object that is created using CoCreateInstance
		m_scan_converter_callback = new ScanConverterCallback();
		m_scan_converter_callback->AddRef();
		// set output window where callback will display received ultrasound frames
		m_scan_converter_callback->SetUsgWnd((HWND)(*this), 20, 20);

		// get IUsgScanConverterPlugin for desired ultrasound data *stream*:
		// SCAN_MODE_B, SCAN_MODE_M, SCAN_MODE_PW, SCAN_MODE_CFM, SCAN_MODE_PDI, SCAN_MODE_DPDI
		tmp_obj = NULL;
		CreateUsgControl(data_view, IID_IUsgScanConverterPlugin, SCAN_MODE_B, 0, (void**)&tmp_obj);
		if (tmp_obj != NULL)
			scp = (IUsgScanConverterPlugin*)tmp_obj;

		// get IUsgfwScanConverterPlugin interface
		tmp_unk = NULL;
		m_plugin = NULL;
		if (scp != NULL)
			scp->get_ScanConverter(&tmp_unk);
		if (tmp_unk != NULL)
		{
			hr = tmp_unk->QueryInterface(IID_IUsgfwScanConverterPlugin, (void**)&m_plugin);
			if (hr != S_OK)
				m_plugin = NULL;
			SAFE_RELEASE(tmp_unk);
		}

		// register COM callback object and specify method(s) to callback
		if ((m_plugin != NULL) && (m_scan_converter_callback != NULL))
		{
			hr = m_plugin->SetCallback(m_scan_converter_callback, USPC_BUFFER_INPUT_OUTPUT);
			m_scan_converter_callback->SetPlugin(m_plugin);
		}

		// attach to control value change connection point in order to be informed about changed values
		IConnectionPointContainer* cpc = NULL;
		hr = data_view->QueryInterface(IID_IConnectionPointContainer, (void**)&cpc);
		if (hr != S_OK)
			cpc = NULL;

		if (cpc != NULL)
			hr = cpc->FindConnectionPoint(IID_IUsgCtrlChangeCommon, &usg_control_change_cpnt);

		if (hr != S_OK)
		{
			usg_control_change_cpnt = NULL;
			usg_control_change_cpnt_cookie = 0;
		}
		SAFE_RELEASE(cpc);

		hr = usg_control_change_cpnt->Advise((IUnknown*)(IUsgCtrlChangeCommon*)this, &usg_control_change_cpnt_cookie);

		if (m_pUsgWindowRF)
		{
			// calculate and set some "default" RF window position and size
			int depth1, depth2;
			int line1, line2;
			VALUE_STEPPING LinesRange;
			VALUE_STEPPING DepthsRange;
			m_pUsgWindowRF->get_LinesRange(&LinesRange);
			m_pUsgWindowRF->get_DepthsRange(&DepthsRange);
			if (depth_ctrl)
			{
				depth_ctrl->get_Current(&DepthsRange.maximum);
			}
			line1 = LinesRange.minimum + (LinesRange.maximum - LinesRange.minimum) / 4;
			line2 = LinesRange.minimum + (LinesRange.maximum - LinesRange.minimum) * 3 / 4;
			depth1 = DepthsRange.minimum + (DepthsRange.maximum - DepthsRange.minimum) / 4;
			depth2 = DepthsRange.minimum + (DepthsRange.maximum - DepthsRange.minimum) * 3 / 4;

			// set RF window size
			m_pUsgWindowRF->SetWindow(line1, line2, depth1, depth2);
			m_nLineIndex = (line2 - line1) / 2;
			number_of_RF_rows = line2 - line1;

		}

		if (m_pUsgDataSourceRF)
			m_pUsgDataSourceRF->put_Current(0);

		RegsAttach();
		RegsReset();

		// start ultrasound scanning (Run)
		StartTimer();
		m_nFramesNumberRF = 0;
		m_nFramesDrawnB = 0;
		m_nFramesScannedB = 0;
		m_nFramesScannedRF = 0;

		data_view->put_ScanState(SCAN_STATE_RUN);

		ShowNormalCursor();

		return;
	} while (false);
	ShowNormalCursor();

	ReleaseUsgControls(false);

} // void CsampleDlg::CreateUsgControls

// convert hexadecimal string of form 0x0000 to integer
int CsampleDlg::GetIntFromHexStr(CString str)
{
	wchar_t* p;
	long n = wcstol(str, &p, 16);
	if (*p != 0)
		return 0;
	return (int)n;
}

void CsampleDlg::InitializeHid()
{
	if ((data_view == NULL) || (probe == NULL))
		return;

	m_vid = 0; //0xFFFF;
	m_pid = 0; //0x3417;
	m_usage_page = 0x0040;
	m_usage = 0x0001;
	CComBSTR bstr;
	probe->get_DispalyName(&bstr);
	// display name is something like this: @device:pnp:\\?\usb#vid_ffff&pid_3417&mi_00#6&1823548e&5&0000#{fe1d88a0-f32b-11d6-95e3-0020ed1a49ff}\{d7b8a242-ba77-409f-abdd-c2170849c30a}
	// need to extract VID and PID (or only PID if assume that VID for our scanners is always FFFF)
	CString str;
	CString str2;
	int idx;

	str = bstr;
	str = str.MakeLower();

	idx = str.Find(L"vid_");
	if (idx < 0)
		return;
	str2 = str.Mid(idx + 4, 4);
	str2 = L"0x" + str2;
	m_vid = GetIntFromHexStr(str2);

	idx = str.Find(L"pid_");
	if (idx < 0)
		return;
	str2 = str.Mid(idx + 4, 4);
	str2 = L"0x" + str2;
	m_pid = GetIntFromHexStr(str2);

	SetHidDevice();
}

// for debugging purposes output what parameter was changed
void CsampleDlg::DisplayChangedParam(BSTR ctrlGUID, LONG scanMode, LONG streamId, IUsgControl *pControlObject, LONG dispId, LONG flags)
{
	/*
		CString str = L"";
		CComBSTR tmp_guid;
		tmp_guid = CComBSTR(ctrlGUID);
		str = tmp_guid;

		if (tmp_guid==CComBSTR(IID_IUsgScanType)) str = "IUsgScanType";
		if (tmp_guid==CComBSTR(IID_IUsgPower)) str = "IUsgPower";
		if (tmp_guid==CComBSTR(IID_IUsgGain)) str = "IUsgGain";
		if (tmp_guid==CComBSTR(IID_IUsgDepth)) str = "IUsgDepth";
		if (tmp_guid==CComBSTR(IID_IUsgRejection2)) str = "IUsgRejection2";
		if (tmp_guid==CComBSTR(IID_IUsgFocus)) str = "IUsgFocus";
		if (tmp_guid==CComBSTR(IID_IUsgDynamicRange)) str = "IUsgDynamicRange";
		if (tmp_guid==CComBSTR(IID_IUsgProbeFrequency2)) str = "IUsgProbeFrequency2";
		if (tmp_guid==CComBSTR(IID_IUsgProbeFrequency3)) str = "IUsgProbeFrequency3";
		if (tmp_guid==CComBSTR(IID_IUsgViewArea)) str = "IUsgViewArea";
		if (tmp_guid==CComBSTR(IID_IUsgImageEnhancement)) str = "IUsgImageEnhancement";
		if (tmp_guid==CComBSTR(IID_IUsgClearView)) str = "IUsgClearView";
		if (tmp_guid==CComBSTR(IID_IUsgFrameAvg)) str = "IUsgFrameAvg";
		if (tmp_guid==CComBSTR(IID_IUsgImageOrientation)) str = "IUsgImageOrientation";
		if (tmp_guid==CComBSTR(IID_IUsgLineDensity)) str = "IUsgLineDensity";
		if (tmp_guid==CComBSTR(IID_IUsgSteeringAngle)) str = "IUsgSteeringAngle";
		if (tmp_guid==CComBSTR(IID_IUsgViewAngle)) str = "IUsgViewAngle";
		if (tmp_guid==CComBSTR(IID_IUsgCompoundAngle)) str = "IUsgCompoundAngle";
		if (tmp_guid==CComBSTR(IID_IUsgCompoundFrames)) str = "IUsgCompoundFrames";
		if (tmp_guid==CComBSTR(IID_IUsgPalette)) str = "IUsgPalette";
		if (tmp_guid==CComBSTR(IID_IUsgTgc)) str = "IUsgTgc";
		if (tmp_guid==CComBSTR(IID_IUsgImageProperties)) str = "IUsgImageProperties";
		if (tmp_guid==CComBSTR(IID_IUsgMixerControl)) str = "IUsgMixerControl";
		if (tmp_guid==CComBSTR(IID_IUsgScanLineSelector)) str = "IUsgScanLineSelector";
		if (tmp_guid==CComBSTR(IID_IUsgSweepSpeed)) str = "IUsgSweepSpeed";
		if (tmp_guid==CComBSTR(IID_IUsgSweepMode)) str = "IUsgSweepMode";
		if (tmp_guid==CComBSTR(IID_IUsgScanLineProperties)) str = "IUsgScanLineProperties";
		if (tmp_guid==CComBSTR(IID_IUsgScanLineProperties2)) str = "IUsgScanLineProperties2";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerPRF)) str = "IUsgDopplerPRF";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerColorPriority)) str = "IUsgDopplerColorPriority";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerSignalScale)) str = "IUsgDopplerSignalScale";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerSignalSmooth)) str = "IUsgDopplerSignalSmooth";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerWindow)) str = "IUsgDopplerWindow";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerSampleVolume)) str = "IUsgDopplerSampleVolume";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerWallFilter)) str = "IUsgDopplerWallFilter";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerSteerAngle)) str = "IUsgDopplerSteerAngle";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerCorrectionAngle)) str = "IUsgDopplerCorrectionAngle";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerColorMap)) str = "IUsgDopplerColorMap";
		if (tmp_guid==CComBSTR(IID_IUsgLineDensity)) str = "IUsgLineDensity";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerColorThreshold)) str = "IUsgDopplerColorThreshold";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerBaseLine)) str = "IUsgDopplerBaseLine";
		if (tmp_guid==CComBSTR(IID_IUsgDopplerPacketSize)) str = "IUsgDopplerPacketSize";
		if (tmp_guid==CComBSTR(IID_IUsgPulsesNumber)) str = "IUsgPulsesNumber";
		if (tmp_guid==CComBSTR(IID_IUsgCineStream)) str = "IUsgCineStream";
		if (tmp_guid==CComBSTR(IID_IUsgCineSink)) str = "IUsgCineSink";
		if (tmp_guid==CComBSTR(IID_IUsgSpatialFilter)) str = "IUsgSpatialFilter";
		if (tmp_guid==CComBSTR(IID_IUsgAudioVolume)) str = "IUsgAudioVolume";
		if (tmp_guid==CComBSTR(IID_IUsgDataSourceRF)) str = "IUsgDataSourceRF";
		if (tmp_guid==CComBSTR(IID_IUsgWindowRF)) str = "IUsgWindowRF";
		if (tmp_guid==CComBSTR(IID_IUsgBioeffectsIndices)) str = "IUsgBioeffectsIndices";

		CString mode_str = L"";
		mode_str.Format(L"%d", scanMode);

		if (scanMode == SCAN_MODE_B) mode_str = L"B";
		else if (scanMode == SCAN_MODE_M) mode_str = L"M";
		else if (scanMode == SCAN_MODE_CFM) mode_str = L"CFM";
		else if (scanMode == SCAN_MODE_PDI) mode_str = L"PDI";
		else if (scanMode == SCAN_MODE_DPDI) mode_str = L"DPDI";
		else if (scanMode == SCAN_MODE_PW) mode_str = L"PW";
		else if (scanMode == SCAN_MODE_RFB) mode_str = L"BRF";

		CString other_str = L"";
		other_str.Format(L"Stream id:%d; DispId:%d; Flags:%d;", streamId, dispId, flags);

		CString log_str = L"";

		log_str += str;
		log_str += L"; ";
		log_str += L"Mode:";
		log_str += mode_str;
		log_str += L"; ";
		log_str += other_str;

		WriteToLog(log_str);
	*/
}


// if scanning parameters are changed - update user interface controls or on-ultrasound graphics
HRESULT CsampleDlg::OnControlChangedBSTR(BSTR ctrlGUID, LONG scanMode, LONG streamId, IUsgControl *pControlObject, LONG dispId, LONG flags)
{
	bool update_alpha_bitmap = false;
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgScanType))
		ScanTypeUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgDepth))
		DepthUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgGain))
		B_GainUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgPower))
		B_PowerUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgProbeFrequency3))
		B_FrequencyUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgLineDensity))
	{
		B_LinesDensityUpdateGUI();
		UpdateLineIndicator();
	}
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgFocus))
	{
		B_FocusUpdateGUI();
		update_alpha_bitmap = true;
	}
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgImageOrientation))
	{
		B_ChangeScanDirectionUpdateGui();
		update_alpha_bitmap = true;
	}
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgSteeringAngle))
		B_SteeringAngleUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgViewAngle))
		WideViewAngleUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgCompoundAngle))
		CompoundAngleUpdateGUI();
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgCompoundFrames))
	{
		CompoundFramesUpdateGUI();
		CompoundSubframeUpdateGUI();
		update_alpha_bitmap = true;
	}
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgImageProperties))
		update_alpha_bitmap = true;
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgWindowRF))
	{
		RF_WindowUpdatePosition();
		UpdateLineIndicator();
		RegsUpdateLinesNum();
		update_alpha_bitmap = true;
	}
	if (CComBSTR(ctrlGUID) == CComBSTR(IID_IUsgCineStream))
	{
		//UpdateFrameSliderRange();

		// changed cine state
		if ((scanMode == SCAN_MODE_B) && (dispId == 5) && (flags == 1))
		{
			UpdateCineFrameIndicator();
			UpdateFrameSliderCurrent();
		}
	}

	if (update_alpha_bitmap)
		PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);

	DisplayChangedParam(ctrlGUID, scanMode, streamId, pControlObject, dispId, flags);
	return S_OK;
}


// begin: IUnknown
STDMETHODIMP_(ULONG) CsampleDlg::AddRef()
{
	++m_refCount;
	return m_refCount;
}

STDMETHODIMP_(ULONG) CsampleDlg::Release()
{
	--m_refCount;
	return m_refCount;
}

STDMETHODIMP CsampleDlg::QueryInterface(REFIID riid, void** ppv)
{
	if (riid == IID_IUnknown)
	{
		*ppv = (IUsgCtrlChangeCommon*)this;
		return S_OK;
	}
	if (riid == IID_IUsgCtrlChangeCommon)
	{
		*ppv = (IUsgCtrlChangeCommon*)this;
		return S_OK;
	}
	if (riid == IID_IUsgDeviceChangeSink)
	{
		*ppv = (IUsgDeviceChangeSink*)this;
		return S_OK;
	}
	if (riid == IID_IDispatch)
	{
		*ppv = (IDispatch*)(IUsgCtrlChangeCommon*)this;
		return S_OK;
	}

	return E_NOINTERFACE;
}

// end: IUnknown
// begin: IDispatch

HRESULT CsampleDlg::GetTypeInfoCount(UINT *pctinfo)
{
	if (pctinfo == NULL) return E_INVALIDARG;
	*pctinfo = 0;
	return S_OK;
}

HRESULT CsampleDlg::GetTypeInfo(UINT itinfo, LCID lcid, ITypeInfo** pptinfo)
{
	if (pptinfo == NULL) return E_INVALIDARG;
	*pptinfo = NULL;
	if (itinfo != 0) return DISP_E_BADINDEX;
	return S_OK;
}

HRESULT CsampleDlg::GetIDsOfNames(const IID &riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgdispid)
{
	// this is not used - must use the same fixed dispid's from Usgfw2 idl file
	return S_OK;
}

HRESULT CsampleDlg::Invoke(DISPID dispIdMember, const IID &riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
{
	if ((dispIdMember >= 1) && (dispIdMember <= 6) && (pDispParams->cArgs == 2))
	{
		IUnknown *unkn = NULL;
		ULONG *res = NULL;

		VARIANTARG* p1;
		VARIANTARG* p;
		p1 = pDispParams->rgvarg;

		p = p1;
		if (p->vt == (VT_BYREF | VT_UI4))
			res = p->pulVal;
		p1++;

		p = p1;
		if (p->vt == VT_UNKNOWN)
			unkn = (IUnknown*)(p->punkVal);

		if (dispIdMember == 1)
			OnProbeArrive(unkn, res);
		else if (dispIdMember == 2)
			OnBeamformerArrive(unkn, res);
		else if (dispIdMember == 3)
			OnProbeRemove(unkn, res);
		else if (dispIdMember == 4)
			OnBeamformerRemove(unkn, res);
		else if (dispIdMember == 5)
			OnProbeStateChanged(unkn, res);
		else if (dispIdMember == 6)
			OnBeamformerStateChanged(unkn, res);
	}
	else if ((dispIdMember == 2) && (pDispParams->cArgs == 6)) // arrived OnControlChangedBSTR notification
	{
		BSTR ctrlGUID = NULL;
		LONG scanMode = 0;
		LONG streamId = 0;
		IUsgControl *pControlObject = NULL;
		LONG dispId = 0;
		LONG flags = 0;

		VARIANTARG* p1;
		VARIANTARG* p;
		p1 = pDispParams->rgvarg;

		p = p1;
		flags = p->lVal;
		p1++;

		p = p1;
		dispId = p->lVal;
		p1++;

		p = p1;
		pControlObject = (IUsgControl*)(p->pdispVal);
		p1++;

		p = p1;
		streamId = p->lVal;
		p1++;

		p = p1;
		scanMode = p->lVal;
		p1++;

		p = p1;
		ctrlGUID = p->bstrVal;
		p1++;

		OnControlChangedBSTR(ctrlGUID, scanMode, streamId, pControlObject, dispId, flags);
	}
	return S_OK;

}

// HRESULT CsampleDlg::Invoke
// end: IDispatch

LRESULT CsampleDlg::OnProbeArriveMessage(WPARAM w, LPARAM l)
{
	w; l;
	LRESULT lRetValue = 0;
	do
	{
		CreateUsgControls();
		InitializeHid();
		LoadUsgValues(GetSettingsFn(), false);
		InitializeUserInterfaceControls();
		EnableDisableUserInterfaceControls();
	} while (false);
	return lRetValue;
}

LRESULT CsampleDlg::OnProbeRemoveMessage(WPARAM w, LPARAM l)
{
	w; l;
	LRESULT lRetValue = 0;
	do
	{
		SaveUsgValues(GetSettingsFn());
		ReleaseUsgControls(false);
		EnableDisableUserInterfaceControls();

	} while (false);
	return lRetValue;
}

// release ultrasound scanning controls
void CsampleDlg::ReleaseUsgControls(bool final_release)
{
	usg_wnd = NULL;
	if (data_view != NULL)
		data_view->put_ScanState(SCAN_STATE_STOP);
	StopTimer();

	if (m_sample_grabber_plugin != NULL)
		m_sample_grabber_plugin->SetCallback(NULL, 0); // unregister callback
	if (m_sample_grabber_callback != NULL)
	{
		//m_sample_grabber_callback->Release();
		delete m_sample_grabber_callback;
		m_sample_grabber_callback = NULL;
	}
	SAFE_RELEASE(sample_grabber_filter);
	SAFE_RELEASE(m_sample_grabber_plugin);

	if (m_plugin != NULL)
		m_plugin->SetCallback(NULL, USPC_BUFFER_INPUT_OUTPUT); // unregister callback
	SAFE_RELEASE(m_scan_converter_callback);
	SAFE_RELEASE(m_plugin);
	SAFE_RELEASE(scp);

	if (usg_control_change_cpnt != NULL)
	{
		usg_control_change_cpnt->Unadvise(usg_control_change_cpnt_cookie);
		usg_control_change_cpnt_cookie = 0;
		SAFE_RELEASE(usg_control_change_cpnt);
	}
	if (final_release)
	{
		if (m_usg_device_change_cpnt != NULL)
		{
			m_usg_device_change_cpnt->Unadvise(m_usg_device_change_cpnt_cookie);
			m_usg_device_change_cpnt_cookie = 0;
			SAFE_RELEASE(m_usg_device_change_cpnt);
		}
	}

	RegsReset();

	SAFE_RELEASE(m_pUsgCineB);
	SAFE_RELEASE(m_pUsgCineRF);
	m_pUsgCine = NULL;
	SAFE_RELEASE(m_pUsgCineSink);
	SAFE_RELEASE(m_pUsgQualProp);
	SAFE_RELEASE(m_pUsgDataSourceRF);
	SAFE_RELEASE(m_pUsgWindowRF);
	SAFE_RELEASE(scan_type_ctrl);
	SAFE_RELEASE(depth_ctrl);
	SAFE_RELEASE(view_area_ctrl);
	SAFE_RELEASE(b_gain_ctrl);
	SAFE_RELEASE(b_power_ctrl);
	SAFE_RELEASE(b_frequency_ctrl);
	SAFE_RELEASE(b_lines_density_ctrl);
	SAFE_RELEASE(b_focus_ctrl);
	SAFE_RELEASE(b_image_orientation_ctrl);
	SAFE_RELEASE(b_steering_angle_ctrl);
	SAFE_RELEASE(wideview_angle_ctrl);
	SAFE_RELEASE(compound_angle_ctrl);
	SAFE_RELEASE(compound_frames_ctrl);
	SAFE_RELEASE(mixer_control);
	SAFE_RELEASE(image_properties_b);
	SAFE_RELEASE(scan_line_properties_b);
	SAFE_RELEASE(data_view);
	SAFE_RELEASE(probe);
	if (final_release)
	{
		SAFE_RELEASE(usgfw2);
	}

	RegsClose();

} // void CsampleDlg::ReleaseUsgControls

void CsampleDlg::EnableCtrl(int id, BOOL enable)
{
	::EnableWindow((HWND)(*(GetDlgItem(id))), enable);
}

// enable or disable user interface controls depending on scanning state (freeze, run)
void CsampleDlg::EnableDisableUserInterfaceControls()
{
	BOOL have_probe;
	BOOL is_run;
	BOOL have_rf;
	have_probe = FALSE;
	is_run = FALSE;
	have_rf = FALSE;

	if (data_view != NULL)
	{
		have_probe = TRUE;
		SCAN_STATE tmp_state;
		data_view->get_ScanState(&tmp_state);
		if (tmp_state == SCAN_STATE_RUN)
			is_run = TRUE;
		if (m_pUsgWindowRF != NULL)
			have_rf = TRUE;
	}

	bool enable_regs_controls;
	if (!m_iKsControl)
		enable_regs_controls = false;
	else
		enable_regs_controls = have_probe && is_run && have_rf;

	EnableCtrl(IDC_DEPTH_PREV_BUTTON, is_run);
	EnableCtrl(IDC_DEPTH_NEXT_BUTTON, is_run);
	EnableCtrl(IDC_B_GAIN_SLIDER, is_run);
	EnableCtrl(IDC_B_POWER_SLIDER, is_run);
	EnableCtrl(IDC_FREQUENCY_PREV_BUTTON, is_run);
	EnableCtrl(IDC_FREQUENCY_NEXT_BUTTON, is_run);
	EnableCtrl(IDC_LINES_DENSITY_PREV_BUTTON, is_run && !using_regs_line_mode);
	EnableCtrl(IDC_LINES_DENSITY_NEXT_BUTTON, is_run && !using_regs_line_mode);
	EnableCtrl(IDC_FOCUS_PREV_BUTTON, is_run && !using_regs_line_mode);
	EnableCtrl(IDC_FOCUS_NEXT_BUTTON, is_run && !using_regs_line_mode);

	EnableCtrl(IDC_CHANGE_SCAN_DIRECTION_CHECK, is_run);

	EnableCtrl(IDC_RADIO_B_STANDARD, is_run && !using_regs_line_mode);
	EnableCtrl(IDC_RADIO_B_WIDEVIEW, is_run && wideview_is_supported && !using_regs_line_mode);
	EnableCtrl(IDC_RADIO_B_COMPOUND, is_run && compound_is_supported && !using_regs_line_mode);

	EnableCtrl(IDC_STEERING_ANGLE_PREV_BUTTON, is_run && (b_steering_angle_ctrl != NULL) && !using_regs_line_mode);
	EnableCtrl(IDC_STEERING_ANGLE_NEXT_BUTTON, is_run && (b_steering_angle_ctrl != NULL) && !using_regs_line_mode);

	EnableCtrl(IDC_WIDEVIEW_ANGLE_PREV_BUTTON, is_run && (wideview_angle_ctrl != NULL) && !using_regs_line_mode);
	EnableCtrl(IDC_WIDEVIEW_ANGLE_NEXT_BUTTON, is_run && (wideview_angle_ctrl != NULL) && !using_regs_line_mode);

	EnableCtrl(IDC_COMPOUND_ANGLE_PREV_BUTTON, is_run && (compound_angle_ctrl != NULL) && !using_regs_line_mode);
	EnableCtrl(IDC_COMPOUND_ANGLE_NEXT_BUTTON, is_run && (compound_angle_ctrl != NULL) && !using_regs_line_mode);

	EnableCtrl(IDC_COMPOUND_FRAMES_PREV_BUTTON, is_run && (compound_frames_ctrl != NULL) && compound_frames_enabled && !using_regs_line_mode);
	EnableCtrl(IDC_COMPOUND_FRAMES_NEXT_BUTTON, is_run && (compound_frames_ctrl != NULL) && compound_frames_enabled && !using_regs_line_mode);

	EnableCtrl(IDC_COMPOUND_SUBFRAME_PREV_BUTTON, is_run && (compound_frames_ctrl != NULL) && compound_frames_enabled && !using_regs_line_mode);
	EnableCtrl(IDC_COMPOUND_SUBFRAME_NEXT_BUTTON, is_run && (compound_frames_ctrl != NULL) && compound_frames_enabled && !using_regs_line_mode);

	EnableCtrl(IDC_I_SCALE_PREV_BUTTON, have_rf);
	EnableCtrl(IDC_I_SCALE_NEXT_BUTTON, have_rf);
	EnableCtrl(IDC_Q_SCALE_PREV_BUTTON, have_rf);
	EnableCtrl(IDC_Q_SCALE_NEXT_BUTTON, have_rf);

	EnableCtrl(IDC_RADIO_WRITE_RF, have_rf && is_run && (!record_n_frames));
	EnableCtrl(IDC_RADIO_STOP_RF, have_rf && is_run && (!record_n_frames));
	EnableCtrl(IDC_LBL_FSize_RF, have_rf && have_probe);

	EnableCtrl(IDC_RECORD_SINGLE_FRAME_BUTTON, have_rf && have_probe && (!check_for_stop) && (!record_n_frames));

	EnableCtrl(IDC_RECORD_N_FRAMES_BUTTON, have_rf && have_probe && (!check_for_stop) && (!record_n_frames));
	EnableCtrl(IDC_RECORD_N_FRAMES_EDIT, have_rf && have_probe && (!check_for_stop) && (!record_n_frames));

	EnableCtrl(IDC_RADIO_AFTER_SUM, have_rf && is_run);
	EnableCtrl(IDC_RADIO_AFTER_TFC, have_rf && is_run && !using_regs_line_mode);
	EnableCtrl(IDC_RADIO_AFTER_APODIZATION, have_rf && is_run && !using_regs_line_mode);
	EnableCtrl(IDC_RADIO_AFTER_HILBERT, have_rf && is_run);
	EnableCtrl(IDC_BUTTON_POSITION_UP, have_rf && is_run);
	EnableCtrl(IDC_BUTTON_POSITION_LEFT, have_rf && is_run && !using_regs_line_mode);
	EnableCtrl(IDC_BUTTON_POSITION_RIGHT, have_rf && is_run && !using_regs_line_mode);
	EnableCtrl(IDC_BUTTON_POSITION_DOWN, have_rf && is_run);
	EnableCtrl(IDC_BUTTON_SIZE_UP, have_rf && is_run);
	EnableCtrl(IDC_BUTTON_SIZE_LEFT, have_rf && is_run && !using_regs_line_mode);
	EnableCtrl(IDC_BUTTON_SIZE_RIGHT, have_rf && is_run && !using_regs_line_mode);
	EnableCtrl(IDC_BUTTON_SIZE_DOWN, have_rf && is_run);
	EnableCtrl(IDC_LINE_PREV_BUTTON, have_rf);
	EnableCtrl(IDC_LINE_NEXT_BUTTON, have_rf);

	EnableCtrl(IDC_CINE_NEXT, have_probe && !is_run);
	EnableCtrl(IDC_CINE_PREV, have_probe && !is_run);
	EnableCtrl(IDC_CINE_STOP, have_probe && !is_run);
	EnableCtrl(IDC_CINE_PLAY, have_probe && !is_run);
	EnableCtrl(IDC_GO_TO_FRAME_BUTTON, have_probe && !is_run);
	EnableCtrl(IDC_GO_TO_FRAME_EDIT, have_probe && !is_run);
	EnableCtrl(IDC_FRAME_SLIDER, have_probe && !is_run);

	EnableCtrl(IDC_FREEZE_AFTER_SCANNING_CHECK, have_probe && !is_run);
	EnableCtrl(IDC_FREEZE_AFTER_SCANNING_STATIC, have_probe && !is_run);
	EnableCtrl(IDC_FREEZE_AFTER_SCANNING_EDIT, have_probe && !is_run);

	EnableCtrl(IDC_AUTOFIT_CHECK, have_rf);

	EnableCtrl(IDC_DISPLAY_RANGE_PREV_BUTTON, have_rf && !rf_iq_autofit);
	EnableCtrl(IDC_DISPLAY_RANGE_NEXT_BUTTON, have_rf && !rf_iq_autofit);

	EnableCtrl(IDC_REGS_WRITE_BUTTON, enable_regs_controls);
	EnableCtrl(IDC_LINE1_EDIT, enable_regs_controls);
	EnableCtrl(IDC_LINE2_EDIT, enable_regs_controls);

	if (using_regs_line_mode)
	{
		SetDlgItemText(IDC_RADIO_AFTER_SUM, L"Line1, Line2 interleaving");
		SetDlgItemText(IDC_RADIO_AFTER_TFC, L"-");
		SetDlgItemText(IDC_RADIO_AFTER_APODIZATION, L"-");
		SetDlgItemText(IDC_RADIO_AFTER_HILBERT, L"Line1, Line2 simultaneously");
	}
	else
	{
		SetDlgItemText(IDC_RADIO_AFTER_SUM, L"Beamformer output (I, 16 bit)");
		SetDlgItemText(IDC_RADIO_AFTER_TFC, L"TFC filter output (I, 16 bit)");
		SetDlgItemText(IDC_RADIO_AFTER_APODIZATION, L"Angle apodization output (I, 16 bit)");
		SetDlgItemText(IDC_RADIO_AFTER_HILBERT, L"Hilbert transform output (I+Q, 16 bit + 16 bit)");
	}

	EnableCtrl(IDC_SAVE_FROM_EDIT, have_rf && have_probe && !is_run);
	EnableCtrl(IDC_SAVE_TO_EDIT, have_rf && have_probe && !is_run);
	EnableCtrl(IDC_SAVE_BUTTON, have_rf && have_probe && !is_run);

	if (!is_run && have_probe)
	{
		SetDlgItemText(IDC_RECORD_N_FRAMES_BUTTON, L"Save N Frames");
		SetDlgItemText(IDC_RECORD_SINGLE_FRAME_BUTTON, L"Save Single Frame");
	}
	else
	{
		SetDlgItemText(IDC_RECORD_N_FRAMES_BUTTON, L"Record N Frames");
		SetDlgItemText(IDC_RECORD_SINGLE_FRAME_BUTTON, L"Record Single Frame");
	}

	EnableCtrl(IDC_OPTIONS_BUTTON, have_probe);

} // void CsampleDlg::EnableDisableUserInterfaceControls


CsampleDlg::CsampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CsampleDlg::IDD, pParent)
	, m_pUsgWindowRF(NULL)
	, m_pUsgDataSourceRF(NULL)
	, m_pUsgQualProp(NULL)
	, m_nTimer(0)
	, m_nFramesNumberRF(0)
	, m_nFramesDrawnB(0)
	, m_nFramesScannedB(0)
	, m_nFramesScannedRF(0)
	, m_pUsgCine(NULL)
	, m_pUsgCineB(NULL)
	, m_pUsgCineRF(NULL)
	, m_pUsgCineSink(NULL)
	, number_of_KBs(0)
	, m_read_thread(NULL)
	, m_read_thread_id(0)
	, m_nSubFrameIndex(0)
	, m_probe_idx(0)
	, rf_iq_autofit(true)
	, display_range(8)
	, using_regs_line_mode(false)
	, regs_line1_idx(-1)
	, regs_line2_idx(-1)
	, regs_lines_num(1)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	freeze_after_scanning_frames_num = 0;
	first_painted = false;
	change_scan_direction_updating_gui = false;
	m_vid = 0;
	m_pid = 0;
	m_usage_page = 0;
	m_usage = 0;
	m_hid_device = NULL;
	ZeroMemory(&m_overlapped, sizeof(OVERLAPPED));
	g_thread_finished = 1;
	g_app_closing = 0;
	m_bUseOverlapped = true;
	SetSupportedHidDevice(0xffff, 0); // support all devices with VID=0xffff
	CHidManager::Create(AfxGetApp()->m_hInstance, ((CDialog*)this)->m_hWnd, (PHidCallback)(&CsampleDlg::HIDCallbackFunction), this);
}

// actions on dialog closing
void CsampleDlg::OnClose()
{
	OptionsSave();
	SaveUsgValues(GetSettingsFn());
	ReleaseUsgControls(true);

	InterlockedIncrement(&g_app_closing);
	StopReading();

	__super::OnClose();
}

// destructor
CsampleDlg::~CsampleDlg()
{
	CoUninitialize();

	if (rf_data_i_arr != NULL)
		delete[] rf_data_i_arr;
	if (rf_data_q_arr != NULL)
		delete[] rf_data_q_arr;

	if (paint_hdc != NULL)
		DeleteDC(paint_hdc);
	if (paint_hbmp != NULL)
		DeleteObject(paint_hbmp);

	DeleteCriticalSection(&paint_cs);
}

void CsampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LBL_FPS_B, m_lblFpsB);
	DDX_Control(pDX, IDC_LBL_FPS_RF, m_lblFpsRF);
	DDX_Control(pDX, IDC_LBL_FSize_RF, RFFileSize);
	DDX_Control(pDX, IDC_RECORD_N_FRAMES_EDIT, m_record_n_frames_edit);
	DDX_Control(pDX, IDC_LINE1_EDIT, m_edtCtrl1);
	DDX_Control(pDX, IDC_LINE2_EDIT, m_edtCtrl2);
	DDX_Control(pDX, IDC_GO_TO_FRAME_EDIT, go_to_frame_edit_ctrl);
	DDX_Control(pDX, IDC_FREEZE_AFTER_SCANNING_CHECK, freeze_after_scanning_check);
	DDX_Control(pDX, IDC_FREEZE_AFTER_SCANNING_EDIT, freeze_after_scanning_edit);
	DDX_Control(pDX, IDC_SAVE_FROM_EDIT, save_from_edit);
	DDX_Control(pDX, IDC_SAVE_TO_EDIT, save_to_edit);
	DDX_Control(pDX, IDC_CHANGE_SCAN_DIRECTION_CHECK, m_change_scan_direction_ctrl);
}

BEGIN_MESSAGE_MAP(CsampleDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_FREEZE_BUTTON, &CsampleDlg::OnBnClickedFreezeButton)
	ON_BN_CLICKED(IDC_DEPTH_PREV_BUTTON, &CsampleDlg::OnBnClickedDepthPrevButton)
	ON_BN_CLICKED(IDC_DEPTH_NEXT_BUTTON, &CsampleDlg::OnBnClickedDepthNextButton)
	ON_BN_CLICKED(IDC_FREQUENCY_PREV_BUTTON, &CsampleDlg::OnBnClickedBFrequencyPrevButton)
	ON_BN_CLICKED(IDC_FREQUENCY_NEXT_BUTTON, &CsampleDlg::OnBnClickedBFrequencyNextButton)
	ON_BN_CLICKED(IDC_LINES_DENSITY_PREV_BUTTON, &CsampleDlg::OnBnClickedBLinesDensityPrevButton)
	ON_BN_CLICKED(IDC_LINES_DENSITY_NEXT_BUTTON, &CsampleDlg::OnBnClickedBLinesDensityNextButton)
	ON_BN_CLICKED(IDC_FOCUS_PREV_BUTTON, &CsampleDlg::OnBnClickedBFocusPrevButton)
	ON_BN_CLICKED(IDC_FOCUS_NEXT_BUTTON, &CsampleDlg::OnBnClickedBFocusNextButton)
	ON_WM_HSCROLL()
	ON_WM_CLOSE()
	ON_WM_QUERYENDSESSION()
	ON_WM_ENDSESSION()
	ON_BN_CLICKED(IDC_RADIO_AFTER_SUM, &CsampleDlg::OnBnClickedRadioAfterSum)

	ON_BN_CLICKED(IDC_RADIO_WRITE_RF, &CsampleDlg::OnBnClickedRadioWriteRf)
	ON_BN_CLICKED(IDC_RADIO_STOP_RF, &CsampleDlg::OnBnClickedRadioStopRf)

	ON_BN_CLICKED(IDC_RADIO_AFTER_TFC, &CsampleDlg::OnBnClickedRadioAfterTfc)
	ON_BN_CLICKED(IDC_RADIO_AFTER_APODIZATION, &CsampleDlg::OnBnClickedRadioAfterApodization)
	ON_BN_CLICKED(IDC_RADIO_AFTER_HILBERT, &CsampleDlg::OnBnClickedRadioAfterHilbert)
	ON_BN_CLICKED(IDC_BUTTON_POSITION_UP, &CsampleDlg::OnBnClickedButtonPositionUp)
	ON_BN_CLICKED(IDC_BUTTON_POSITION_DOWN, &CsampleDlg::OnBnClickedButtonPositionDown)
	ON_BN_CLICKED(IDC_BUTTON_POSITION_LEFT, &CsampleDlg::OnBnClickedButtonPositionLeft)
	ON_BN_CLICKED(IDC_BUTTON_POSITION_RIGHT, &CsampleDlg::OnBnClickedButtonPositionRight)
	ON_BN_CLICKED(IDC_BUTTON_SIZE_UP, &CsampleDlg::OnBnClickedButtonSizeUp)
	ON_BN_CLICKED(IDC_BUTTON_SIZE_DOWN, &CsampleDlg::OnBnClickedButtonSizeDown)
	ON_BN_CLICKED(IDC_BUTTON_SIZE_LEFT, &CsampleDlg::OnBnClickedButtonSizeLeft)
	ON_BN_CLICKED(IDC_BUTTON_SIZE_RIGHT, &CsampleDlg::OnBnClickedButtonSizeRight)
	ON_BN_CLICKED(IDC_LINE_PREV_BUTTON, &CsampleDlg::OnBnClickedLinePrevButton)
	ON_BN_CLICKED(IDC_LINE_NEXT_BUTTON, &CsampleDlg::OnBnClickedLineNextButton)
	ON_BN_CLICKED(IDC_I_SCALE_PREV_BUTTON, &CsampleDlg::OnBnClickedIScalePrevButton)
	ON_BN_CLICKED(IDC_I_SCALE_NEXT_BUTTON, &CsampleDlg::OnBnClickedIScaleNextButton)
	ON_BN_CLICKED(IDC_Q_SCALE_PREV_BUTTON, &CsampleDlg::OnBnClickedQScalePrevButton)
	ON_BN_CLICKED(IDC_Q_SCALE_NEXT_BUTTON, &CsampleDlg::OnBnClickedQScaleNextButton)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_CINE_PREV, &CsampleDlg::OnBnClickedCinePrev)
	ON_BN_CLICKED(IDC_CINE_PLAY, &CsampleDlg::OnBnClickedCinePlay)
	ON_BN_CLICKED(IDC_CINE_STOP, &CsampleDlg::OnBnClickedCineStop)
	ON_BN_CLICKED(IDC_CINE_NEXT, &CsampleDlg::OnBnClickedCineNext)
	ON_MESSAGE(WM_MESSAGE_UPDATE_BITMAP, &CsampleDlg::OnUpdateBitmapMessage)
	ON_MESSAGE(WM_MESSAGE_PROBE_ARRIVE, &CsampleDlg::OnProbeArriveMessage)
	ON_MESSAGE(WM_MESSAGE_PROBE_REMOVE, &CsampleDlg::OnProbeRemoveMessage)
	ON_BN_CLICKED(IDC_SAVE_SETTINGS_TO_FILE, &CsampleDlg::OnBnClickedSaveSettingsToFile)
	ON_BN_CLICKED(IDC_LOAD_SETTINGS_FROM_FILE, &CsampleDlg::OnBnClickedLoadSettingsFromFile)
	ON_BN_CLICKED(IDC_WIDEVIEW_ANGLE_PREV_BUTTON, &CsampleDlg::OnBnClickedWideviewAnglePrevButton)
	ON_BN_CLICKED(IDC_WIDEVIEW_ANGLE_NEXT_BUTTON, &CsampleDlg::OnBnClickedWideviewAngleNextButton)
	ON_BN_CLICKED(IDC_COMPOUND_ANGLE_PREV_BUTTON, &CsampleDlg::OnBnClickedCompoundAnglePrevButton)
	ON_BN_CLICKED(IDC_COMPOUND_ANGLE_NEXT_BUTTON, &CsampleDlg::OnBnClickedCompoundAngleNextButton)
	ON_BN_CLICKED(IDC_COMPOUND_FRAMES_PREV_BUTTON, &CsampleDlg::OnBnClickedCompoundFramesPrevButton)
	ON_BN_CLICKED(IDC_COMPOUND_FRAMES_NEXT_BUTTON, &CsampleDlg::OnBnClickedCompoundFramesNextButton)
	ON_BN_CLICKED(IDC_RADIO_B_STANDARD, &CsampleDlg::OnBnClickedRadioBStandard)
	ON_BN_CLICKED(IDC_RADIO_B_WIDEVIEW, &CsampleDlg::OnBnClickedRadioBWideview)
	ON_BN_CLICKED(IDC_RADIO_B_COMPOUND, &CsampleDlg::OnBnClickedRadioBCompound)
	ON_BN_CLICKED(IDC_STEERING_ANGLE_PREV_BUTTON, &CsampleDlg::OnBnClickedSteeringAnglePrevButton)
	ON_BN_CLICKED(IDC_STEERING_ANGLE_NEXT_BUTTON, &CsampleDlg::OnBnClickedSteeringAngleNextButton)
	ON_BN_CLICKED(IDC_RECORD_SINGLE_FRAME_BUTTON, &CsampleDlg::OnBnClickedRecordSingleFrameButton)
	ON_MESSAGE(WM_MESSAGE_HID_FREEZE, OnHidFreeze)
	ON_BN_CLICKED(IDC_COMPOUND_SUBFRAME_PREV_BUTTON, &CsampleDlg::OnBnClickedCompoundSubframePrevButton)
	ON_BN_CLICKED(IDC_COMPOUND_SUBFRAME_NEXT_BUTTON, &CsampleDlg::OnBnClickedCompoundSubframeNextButton)
	ON_BN_CLICKED(IDC_PROBE_NAME_BUTTON, &CsampleDlg::OnBnClickedProbeNameButton)
	ON_BN_CLICKED(IDC_RECORD_N_FRAMES_BUTTON, &CsampleDlg::OnBnClickedRecordNFramesButton)
	ON_BN_CLICKED(IDC_AUTOFIT_CHECK, &CsampleDlg::OnBnClickedAutofitCheck)
	ON_BN_CLICKED(IDC_DISPLAY_RANGE_PREV_BUTTON, &CsampleDlg::OnBnClickedDisplayRangePrevButton)
	ON_BN_CLICKED(IDC_DISPLAY_RANGE_NEXT_BUTTON, &CsampleDlg::OnBnClickedDisplayRangeNextButton)
	ON_BN_CLICKED(IDC_REGS_WRITE_BUTTON, &CsampleDlg::OnBnClickedRegsWriteButton)
	ON_BN_CLICKED(IDC_GO_TO_FRAME_BUTTON, &CsampleDlg::OnBnClickedGoToFrameButton)
	ON_MESSAGE(WM_MESSAGE_FREEZE_AFTER_SCANNING, OnFreezeAfterScanning)
	ON_BN_CLICKED(IDC_SAVE_BUTTON, &CsampleDlg::OnBnClickedSaveButton)
	ON_BN_CLICKED(IDC_OPTIONS_BUTTON, &CsampleDlg::OnBnClickedOptionsButton)
	ON_BN_CLICKED(IDC_CHANGE_SCAN_DIRECTION_CHECK, &CsampleDlg::OnBnClickedChangeScanDirectionCheck)
END_MESSAGE_MAP()


// dialog initialization
BOOL CsampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	ShowWaitCursor();

	// Set the icon for this dialog.  The framework does this automatically
	// when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	CString str;
	str = APP_NAME;
	str += " ";
	str += APP_VERSION;
	str += " ";
	str += APP_BITS;
	str += "    Loading... Please wait...";
	SetWindowText(str);


	InitializeCriticalSection(&paint_cs);
	InitializeCriticalSection(&paint_rf_data_cs);
	paint_hdc = NULL;
	paint_hbmp = NULL;
	img_w = 512;
	img_h = 512;
	m_refCount = 0;
	usg_wnd = NULL;
	usgfw2 = NULL;
	data_view = NULL;
	probe = NULL;
	mixer_control = NULL;
	image_properties_b = NULL;
	scan_line_properties_b = NULL;
	m_pUsgWindowRF = NULL;
	m_pUsgDataSourceRF = NULL;
	m_pUsgQualProp = NULL;
	m_pUsgCine = NULL;
	m_pUsgCineB = NULL;
	m_pUsgCineRF = NULL;
	m_pUsgCineSink = NULL;
	sample_grabber_filter = NULL;
	m_sample_grabber_plugin = NULL;
	m_sample_grabber_callback = NULL;
	m_usg_device_change_cpnt = NULL;
	m_usg_device_change_cpnt_cookie = 0;
	m_plugin = NULL;
	scp = NULL;
	m_scan_converter_callback = NULL;

	using_regs_line_mode = false;
	wideview_is_supported = false;
	compound_is_supported = false;
	scan_type_ctrl = NULL;
	depth_ctrl = NULL;
	b_gain_ctrl = NULL;
	b_power_ctrl = NULL;
	b_frequency_ctrl = NULL;
	b_lines_density_ctrl = NULL;
	b_focus_ctrl = NULL;
	b_image_orientation_ctrl = NULL;
	b_steering_angle_ctrl = NULL;
	wideview_angle_ctrl = NULL;
	compound_angle_ctrl = NULL;
	compound_frames_ctrl = NULL;
	compound_frames_enabled = false;

	usg_control_change_cpnt = NULL;
	usg_control_change_cpnt_cookie = 0;

	rf_data_source_point_id = 4;
	m_nLineIndex = 0;
	rf_data_i_arr = NULL;
	rf_data_q_arr = NULL;
	rf_data_i_len = 0;
	rf_data_file_handle = NULL;
	rf_data_file_name = L"";

	rf_data_q_len = 0;
	rf_data_i_scale = 1;
	rf_data_q_scale = 1;
	rf_data_i_scale2 = 1;
	rf_data_q_scale2 = 1;

	CheckDlgButton(IDC_RADIO_AFTER_HILBERT, BST_CHECKED);
	CheckDlgButton(IDC_RADIO_STOP_RF, BST_CHECKED);
	CheckDlgButton(IDC_RADIO_B_STANDARD, BST_CHECKED);

	check_for_stop = false;
	do_stop = false;
	record_single_frame = false;
	record_n_frames = false;
	recorded_n_frames = 0;
	m_record_n_frames_edit.SetWindowText(L"100");
	QueryPerformanceCounter(&fps_time);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CsampleDlg::ShowWaitCursor()
{
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_WAIT));
}

void CsampleDlg::ShowNormalCursor()
{
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
}

BOOL Is64BitWindows()
{
#if defined(_WIN64)
	return TRUE;  // 64-bit programs run only on Win64
#elif defined(_WIN32)
	// 32-bit programs run on both 32-bit and 64-bit Windows
	// so must sniff
	BOOL f64 = FALSE;
	return IsWow64Process(GetCurrentProcess(), &f64) && f64;
#else
	return FALSE; // Win64 does not support Win16
#endif
}

// If you add a minimize button to your dialog, you will need the code below
// to draw the icon.  For MFC applications using the document/view model,
// this is automatically done for you by the framework.

void CsampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();

		if (first_painted == false)
		{
			first_painted = true;

			ShowWaitCursor();

			CreateUsgControls();
			OptionsLoad();
			if ((m_pUsgCineSink != NULL) && (cine_size_mb > 0))
			{
				if (cine_size_mb > MAX_CINE_BUFFER_SIZE_MB)
					cine_size_mb = MAX_CINE_BUFFER_SIZE_MB;
				m_pUsgCineSink->SetBufferSize((LONG)cine_size_mb * 1024 * 1024);
			}
			InitializeHid();
			LoadUsgValues(GetSettingsFn(), false);
			InitializeUserInterfaceControls();
			EnableDisableUserInterfaceControls();

			ShowNormalCursor();

			CString str;
			str = APP_NAME;
			str += " ";
			str += APP_VERSION;
			str += " ";
			str += APP_BITS;
#ifndef _WIN64
			if (Is64BitWindows())
				str += " running on 64-bit Windows";
#endif
			SetWindowText(str);
		}

		if (mixer_control != NULL)
		{
			if (usg_wnd != NULL)
			{
				HDC hdc;
				hdc = ::GetDC(usg_wnd);
				// repaint ultrasound (important if we are in freeze)
				mixer_control->Repaint((LONG)usg_wnd, (LONG)hdc);
				::ReleaseDC(usg_wnd, hdc);
			}
		}

		// paint RF data (I, Q) near ultrasound
		PaintRFData();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR CsampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// load scanning parameters from file and set to controls
void CsampleDlg::LoadUsgValues(CString fn, bool show_error_messages)
{
	if (data_view == NULL)
		return;

	const int BUF_SIZE = 64;
	TCHAR buf[BUF_SIZE];
	CString section = L"Default";

	CString str;
	LONG val;
	LONG tmp_probe_code = 0;
	LONG tmp_beamformer_code = 0;

	if (probe != NULL)
	{
		probe->get_Code(&tmp_probe_code);

		IUnknown* tmp_obj = NULL;
		probe->get_Beamformer(&tmp_obj);
		if (tmp_obj != NULL)
		{
			IBeamformer* tmp_beamformer = NULL;
			HRESULT hr = S_OK;
			hr = tmp_obj->QueryInterface(IID_IBeamformer, (void**)&tmp_beamformer);
			tmp_obj->Release();
			if ((hr != S_OK) || (tmp_beamformer == NULL))
				tmp_beamformer = NULL;
			if (tmp_beamformer != NULL)
				tmp_beamformer->get_Code(&tmp_beamformer_code);
			SAFE_RELEASE(tmp_beamformer);
		}
	}

	CString err_msg = L"";
	bool tmp_err = false;
	if (GetPrivateProfileString(section, L"beamformer_code", L"", buf, BUF_SIZE, fn) > 0)
	{
		val = _wtoi(buf);
		if (tmp_beamformer_code != val)
		{
			tmp_err = true;
			err_msg += L"Settings were saved for different beamformer.\r\n";
		}
	}

	if (GetPrivateProfileString(section, L"probe_code", L"", buf, BUF_SIZE, fn) > 0)
	{
		val = _wtoi(buf);
		if (tmp_probe_code != val)
		{
			tmp_err = true;
			err_msg += L"Settings were saved for different probe.\r\n";
		}
	}

	if (show_error_messages && tmp_err)
	{
		err_msg += L"Apply anyway?";
		int tmp_ret;
		tmp_ret = MessageBox(err_msg, L"Warning", MB_YESNOCANCEL | MB_ICONQUESTION | MB_DEFBUTTON2);
		if (tmp_ret != IDYES)
			return;
	}

	if (scan_type_ctrl != NULL)
	{
		val = SCAN_TYPE_NORMAL;
		if (GetPrivateProfileString(section, L"scan_type", L"", buf, BUF_SIZE, fn) > 0)
			val = _wtoi(buf);
		scan_type_ctrl->put_Current(val);
	}

	if (depth_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"depth", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			depth_ctrl->put_Current(val);
		}
	}

	if (b_steering_angle_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"steering_angle", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			b_steering_angle_ctrl->put_Current(val);
		}
	}

	if (wideview_angle_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"wideview_angle", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			wideview_angle_ctrl->put_Current(val);
		}
	}

	if (compound_angle_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"compound_angle", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			compound_angle_ctrl->put_Current(val);
		}
	}

	if (compound_frames_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"compound_frames", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			compound_frames_ctrl->put_Current(val);
		}
		m_nSubFrameIndex = 0;
		if (GetPrivateProfileString(section, L"compound_subframe", L"", buf, BUF_SIZE, fn) > 0)
		{
			m_nSubFrameIndex = _wtoi(buf);
		}
	}

	if (b_lines_density_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"lines_density_id", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			b_lines_density_ctrl->put_Current(val);
		}
	}

	if (b_focus_ctrl != NULL)
	{
		LONG focus_mode = 0;
		b_focus_ctrl->get_CurrentMode(&focus_mode);
		if (GetPrivateProfileString(section, L"focus_mode", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			focus_mode = val;
			b_focus_ctrl->put_CurrentMode(focus_mode);
		}

		LONG focus_set = 0;
		b_focus_ctrl->get_FocusSet(focus_mode, &focus_set);
		if (GetPrivateProfileString(section, L"focus_set", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			focus_set = val;
			b_focus_ctrl->put_FocusSet(focus_mode, focus_set);
		}

		if (GetPrivateProfileString(section, L"focus_zone", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			LONG focal_zones_count = 0;
			b_focus_ctrl->GetFocalZonesCount(focus_mode, focus_set, &focal_zones_count);
			if ((val >= 0) && (val < focal_zones_count))
			{
				LONG *arr;
				arr = new LONG[focal_zones_count];
				for (int i1 = 0; i1 < focal_zones_count; i1++)
					arr[i1] = 0;
				arr[val] = 1;
				b_focus_ctrl->SetFocusState(focus_mode, focus_set, focal_zones_count, arr);
				delete[] arr;
			}
		}
	} // if (b_focus_ctrl != NULL)

	if (b_gain_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"gain", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			b_gain_ctrl->put_Current(val);
		}
	}

	if (b_power_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"power", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			b_power_ctrl->put_Current(val);
		}
	}

	if (b_image_orientation_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"change_scan_direction", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			b_image_orientation_ctrl->put_Mirror(val);
		}
	}

	if (b_frequency_ctrl != NULL)
	{
		if (GetPrivateProfileString(section, L"frequency_val", L"", buf, BUF_SIZE, fn) > 0)
		{
			FREQUENCY3 fval;
			val = _wtoi(buf);
			fval.nFrequency = val;
			fval.nThiMode = (THI_MODE)1;
			if (GetPrivateProfileString(section, L"frequency_thi_mode", L"", buf, BUF_SIZE, fn) > 0)
			{
				val = _wtoi(buf);
				fval.nThiMode = (THI_MODE)val;
			}
			b_frequency_ctrl->put_Current(fval);
		}
	}

	if (m_pUsgWindowRF != NULL)
	{
		LONG depth1, depth2;
		LONG line1, line2;
		depth1 = depth2 = line1 = line2 = 0;
		if (m_pUsgWindowRF != NULL)
		{
			m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

			// initialize window size variables to some default values
			VALUE_STEPPING depths_range;
			VALUE_STEPPING lines_range;
			m_pUsgWindowRF->get_DepthsRange(&depths_range);
			m_pUsgWindowRF->get_LinesRange(&lines_range);
			line1 = lines_range.minimum + (lines_range.maximum - lines_range.minimum) / 4;
			line2 = lines_range.maximum - (lines_range.maximum - lines_range.minimum) / 4;
			depth1 = depths_range.minimum + (depths_range.maximum - depths_range.minimum) / 4;
			depth2 = depths_range.maximum - (depths_range.maximum - depths_range.minimum) / 4;
		}

		if (GetPrivateProfileString(section, L"rf_window_line1", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			line1 = val;
		}

		if (GetPrivateProfileString(section, L"rf_window_line2", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			line2 = val;
		}

		if (GetPrivateProfileString(section, L"rf_window_depth1", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			depth1 = val;
		}

		if (GetPrivateProfileString(section, L"rf_window_depth2", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			depth2 = val;
		}

		if (m_pUsgWindowRF != NULL)
			m_pUsgWindowRF->SetWindow(line1, line2, depth1, depth2);

		if (GetPrivateProfileString(section, L"rf_iq_line", L"", buf, BUF_SIZE, fn) > 0)
		{
			val = _wtoi(buf);
			m_nLineIndex = val;
		}

	} // if (m_pUsgWindowRF != NULL)

} // void CsampleDlg::LoadUsgValues

// save scanning parameters to file
void CsampleDlg::SaveUsgValues(CString fn)
{
	if (data_view == NULL)
		return;

	CString section = L"Default";

	CString str;
	LONG val;

	if (probe != NULL)
	{
		probe->get_Code(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"probe_code", str, fn);
		// get probe name
		CComBSTR bstr;
		probe->get_Name(&bstr);
		WritePrivateProfileString(section, L"probe_name", bstr, fn);

		IUnknown* tmp_obj = NULL;
		probe->get_Beamformer(&tmp_obj);
		if (tmp_obj != NULL)
		{
			IBeamformer* tmp_beamformer = NULL;
			HRESULT hr = S_OK;
			hr = tmp_obj->QueryInterface(IID_IBeamformer, (void**)&tmp_beamformer);
			tmp_obj->Release();
			if ((hr != S_OK) || (tmp_beamformer == NULL))
				tmp_beamformer = NULL;
			if (tmp_beamformer != NULL)
			{
				tmp_beamformer->get_Code(&val);
				str.Format(L"%d", val);
				WritePrivateProfileString(section, L"beamformer_code", str, fn);
				// get beamformer name
				CComBSTR bstr;
				tmp_beamformer->get_Name(&bstr);
				WritePrivateProfileString(section, L"beamformer_name", bstr, fn);
			}
			SAFE_RELEASE(tmp_beamformer);
		}
	}

	if (scan_type_ctrl != NULL)
	{
		scan_type_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"scan_type", str, fn);
	}

	if (depth_ctrl != NULL)
	{
		depth_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"depth", str, fn);
	}

	if (b_gain_ctrl != NULL)
	{
		b_gain_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"gain", str, fn);
	}

	if (b_power_ctrl != NULL)
	{
		b_power_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"power", str, fn);
	}

	if (b_image_orientation_ctrl != NULL)
	{
		b_image_orientation_ctrl->get_Mirror(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"change_scan_direction", str, fn);
	}

	if (b_frequency_ctrl != NULL)
	{
		FREQUENCY3 fval;
		b_frequency_ctrl->get_Current(&fval);

		str.Format(L"%d", fval.nFrequency);
		WritePrivateProfileString(section, L"frequency_val", str, fn);

		str.Format(L"%d", fval.nThiMode);
		WritePrivateProfileString(section, L"frequency_thi_mode", str, fn);
	}

	if (b_lines_density_ctrl != NULL)
	{
		b_lines_density_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"lines_density_id", str, fn);
	}

	if (b_focus_ctrl != NULL)
	{
		LONG focus_mode = 0;
		LONG focus_set = 0;
		LONG focal_zones_count = 0;

		b_focus_ctrl->get_CurrentMode(&focus_mode);
		b_focus_ctrl->get_FocusSet(focus_mode, &focus_set);
		b_focus_ctrl->GetFocalZonesCount(focus_mode, focus_set, &focal_zones_count);

		// find the index of first active focus zone
		int focus_zone_idx = -1;
		LONG *arr;
		arr = new LONG[focal_zones_count];
		b_focus_ctrl->GetFocusState(focus_mode, focus_set, focal_zones_count, arr);
		for (int i1 = 0; i1 < focal_zones_count; i1++)
		{
			if (arr[i1] != 0)
			{
				focus_zone_idx = i1;
				break;
			}
		}
		delete[] arr;

		str.Format(L"%d", focus_zone_idx);
		WritePrivateProfileString(section, L"focus_zone", str, fn);
		str.Format(L"%d", focus_mode);
		WritePrivateProfileString(section, L"focus_mode", str, fn);
		str.Format(L"%d", focus_set);
		WritePrivateProfileString(section, L"focus_set", str, fn);
	}

	if (b_steering_angle_ctrl != NULL)
	{
		b_steering_angle_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"steering_angle", str, fn);
	}

	if (wideview_angle_ctrl != NULL)
	{
		wideview_angle_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"wideview_angle", str, fn);
	}

	if (compound_angle_ctrl != NULL)
	{
		compound_angle_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"compound_angle", str, fn);
	}

	if (compound_frames_ctrl != NULL)
	{
		compound_frames_ctrl->get_Current(&val);
		str.Format(L"%d", val);
		WritePrivateProfileString(section, L"compound_frames", str, fn);

		str.Format(L"%d", m_nSubFrameIndex);
		WritePrivateProfileString(section, L"compound_subframe", str, fn);
	}

	if (m_pUsgWindowRF != NULL)
	{
		LONG depth1, depth2;
		LONG line1, line2;
		depth1 = depth2 = line1 = line2 = 0;
		if (m_pUsgWindowRF != NULL)
			m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

		str.Format(L"%d", line1);
		WritePrivateProfileString(section, L"rf_window_line1", str, fn);

		str.Format(L"%d", line2);
		WritePrivateProfileString(section, L"rf_window_line2", str, fn);

		str.Format(L"%d", depth1);
		WritePrivateProfileString(section, L"rf_window_depth1", str, fn);

		str.Format(L"%d", depth2);
		WritePrivateProfileString(section, L"rf_window_depth2", str, fn);

		str.Format(L"%d", m_nLineIndex);
		WritePrivateProfileString(section, L"rf_iq_line", str, fn);
	}

} // void CsampleDlg::SaveUsgValues

// get the name of file where are stored scanning parameters
CString CsampleDlg::GetSettingsFn()
{

	LONG beamformer_code = 0;
	LONG probe_code = 0;
	if (probe != NULL)
	{
		// get probe code and beamformer code and add to file name _0000_0000.ini, where first number is beamformer code, and second number is probe code
		HRESULT hr = S_OK;
		probe->get_Code(&probe_code);

		IBeamformer* tmp_beamformer = NULL;
		IUnknown* tmp_obj = NULL;
		probe->get_Beamformer(&tmp_obj);
		hr = tmp_obj->QueryInterface(IID_IBeamformer, (void**)&tmp_beamformer);
		tmp_obj->Release();
		if ((hr != S_OK) || (tmp_beamformer == NULL))
			tmp_beamformer = NULL;
		if (tmp_beamformer != NULL)
			tmp_beamformer->get_Code(&beamformer_code);
		SAFE_RELEASE(tmp_beamformer);
	}

	TCHAR path[_MAX_PATH];
	TCHAR fn[_MAX_PATH];
	TCHAR drive[_MAX_DRIVE];
	TCHAR dir[_MAX_DIR];
	TCHAR fname[_MAX_FNAME];
	TCHAR fname2[_MAX_FNAME];
	TCHAR ext[_MAX_EXT];

	GetModuleFileName(NULL, path, sizeof(path));
	_wsplitpath(path, drive, dir, fname, ext);

	swprintf(fname2, L"%s_%04d_%04d", fname, beamformer_code, probe_code);

	_wmakepath(fn, drive, dir, fname2, L"ini");

	return CString(fn);
}

// message handler to freeze after scanning desired number of frames
LRESULT CsampleDlg::OnFreezeAfterScanning(WPARAM wParam, LPARAM lParam)
{
	if (data_view == NULL)
		return 0;
	if (freeze_after_scanning_frames_num > 0)
		return 0;

	SCAN_STATE state;
	data_view->get_ScanState(&state);

	if (state == SCAN_STATE_RUN)
	{
		FreezeRun();
	}

	return 0;
}

// freeze/run ultrasound scanning
void CsampleDlg::FreezeRun()
{
	freeze_after_scanning_frames_num = 0;
	record_n_frames = false;
	record_single_frame = false;

	if (data_view == NULL) return;

	SCAN_STATE state;
	data_view->get_ScanState(&state);

	if (state == SCAN_STATE_RUN)
	{
		// Freeze
		data_view->put_ScanState(SCAN_STATE_FREEZE);
		EnableDisableUserInterfaceControls();
		PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);

		if (rf_data_file_handle != NULL)
		{
			check_for_stop = false;
			fclose(rf_data_file_handle);
			rf_data_file_handle = NULL;
			number_of_KBs = 0;
			record_single_frame = false;
			record_n_frames = false;
			do_stop = false;
			check_for_stop = false;

			wchar_t dir[MAX_PATH];
			GetCurrentDirectory(MAX_PATH, dir);
			CString src_fn = L"";
			src_fn = rf_data_file_name;
			CString fn2;
			fn2 = rf_data_file_name.Mid(0, rf_data_file_name.GetLength() - 4);
			fn2 += ".bin";
			MoveFile(src_fn, fn2);

			CheckDlgButton(IDC_RADIO_STOP_RF, BST_CHECKED);
			CheckDlgButton(IDC_RADIO_WRITE_RF, BST_UNCHECKED);
		}
	}
	else
	{
		// Run
		m_nFramesNumberRF = 0;
		m_nFramesDrawnB = 0;

		LONGLONG nFrames = 0;
		if (m_pUsgCineB)
		{
			m_pUsgCineB->get_CurrentPosition(&nFrames);
			m_nFramesScannedB = nFrames;
		}
		else
			m_nFramesScannedB = 0;

		if (m_pUsgCineRF)
		{
			m_pUsgCineRF->get_CurrentPosition(&nFrames);
			m_nFramesScannedRF = nFrames;
		}
		else
			m_nFramesScannedRF = 0;

		if (freeze_after_scanning_check.GetCheck() == BST_CHECKED)
		{
			CString str;
			freeze_after_scanning_edit.GetWindowText(str);
			int val;
			val = _wtoi(str);
			if (val > 0)
				freeze_after_scanning_frames_num = val + 1;
		}

		QueryPerformanceCounter(&fps_time);

		data_view->put_ScanState(SCAN_STATE_RUN);
		EnableDisableUserInterfaceControls();
		PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
	}

	UpdateCineFrameIndicator();
	UpdateFrameSliderRange();
	UpdateFrameSliderCurrent();

}

// Freeze/Run button click
void CsampleDlg::OnBnClickedFreezeButton()
{
	FreezeRun();
}

// load current values of controls on startup
void CsampleDlg::InitializeUserInterfaceControls()
{
	ScanTypeUpdateGUI();
	DepthUpdateGUI();
	B_GainUpdateGUI();
	B_PowerUpdateGUI();
	B_FrequencyUpdateGUI();
	B_LinesDensityUpdateGUI();
	B_FocusUpdateGUI();
	B_ChangeScanDirectionUpdateGui();
	WideViewAngleUpdateGUI();
	CompoundAngleUpdateGUI();
	CompoundFramesUpdateGUI();
	CompoundSubframeUpdateGUI();
	RfDataSourcePointUpdateGui();
	UpdateLineIndicator();
	UpdateIScaleGUI();
	UpdateQScaleGUI();
	UpdateCineFrameIndicator();
	UpdateFrameSliderRange();
	UpdateFrameSliderCurrent();
	UpdateAutofitGUI();
	RecordNFramesUpdateGUI();
	DisplayRangeUpdateGUI();
}

// update displayed depth label value
void CsampleDlg::DepthUpdateGUI()
{
	if (depth_ctrl == NULL) return;
	LONG val;
	CString str;
	depth_ctrl->get_Current(&val);
	str.Format(L"%d mm", val);
	SetDlgItemText(IDC_DEPTH_STATIC, str);
}

// increase or decrease depth
void CsampleDlg::DepthSetPrevNext(int dir)
{
	if (depth_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	depth_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	depth_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			usg_values->Item(val_idx, &item);
			val_cur = item.lVal;
			LONG val;
			depth_ctrl->get_Current(&val);
			if (val != val_cur)
				depth_ctrl->put_Current(val_cur);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);

} // void DepthSetPrevNext

// pevious depth (decrease)
void CsampleDlg::OnBnClickedDepthPrevButton()
{
	DepthSetPrevNext(-1);
	DepthUpdateGUI();
}

// next depth (increase)
void CsampleDlg::OnBnClickedDepthNextButton()
{
	DepthSetPrevNext(1);
	DepthUpdateGUI();
}

// process messages from sliders
void CsampleDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	int slider_id;
	slider_id = pScrollBar->GetDlgCtrlID();

	if (
		(slider_id == IDC_B_GAIN_SLIDER)
		|| (slider_id == IDC_B_POWER_SLIDER)
		|| (slider_id == IDC_FRAME_SLIDER)
		)
	{
		int tmp_min, tmp_max, tmp_pos;
		HWND hslider = NULL;

		if (slider_id == IDC_B_GAIN_SLIDER)
			hslider = (HWND)(*(GetDlgItem(IDC_B_GAIN_SLIDER)));
		else if (slider_id == IDC_B_POWER_SLIDER)
			hslider = (HWND)(*(GetDlgItem(IDC_B_POWER_SLIDER)));
		else if (slider_id == IDC_FRAME_SLIDER)
			hslider = (HWND)(*(GetDlgItem(IDC_FRAME_SLIDER)));

		if (hslider != NULL)
		{
			tmp_min = (int)::SendMessage(hslider, TBM_GETRANGEMIN, 0L, 0L);
			tmp_max = (int)::SendMessage(hslider, TBM_GETRANGEMAX, 0L, 0L);
			tmp_pos = (int)::SendMessage(hslider, TBM_GETPOS, 0, 0);

			switch (nSBCode)
			{
			case SB_LEFT:
				tmp_pos = tmp_min;
				break;

			case SB_RIGHT:
				tmp_pos = tmp_max;
				break;

			case SB_ENDSCROLL:
				break;

			case SB_LINELEFT:
			case SB_PAGELEFT:
				//if (tmp_pos > tmp_min)
				//	tmp_pos--;
				break;

			case SB_LINERIGHT:
			case SB_PAGERIGHT:
				//if (tmp_pos < tmp_max)
				//	tmp_pos++;
				break;

			case SB_THUMBPOSITION:
			case SB_THUMBTRACK:
				tmp_pos = nPos;
				break;
			}

			if (nSBCode != SB_ENDSCROLL)
			{
				if (slider_id == IDC_B_GAIN_SLIDER)
					B_GainSetByIdx(tmp_pos);
				else if (slider_id == IDC_B_POWER_SLIDER)
					B_PowerSetByIdx(tmp_pos);
				else if (slider_id == IDC_FRAME_SLIDER)
					FrameSliderGoToFrame(tmp_pos);
			}

		}

	}

} // void CsampleDlg::OnHScroll


// set gain value by passed index
void CsampleDlg::B_GainSetByIdx(int idx)
{
	if (b_gain_ctrl == NULL) return;

	LONG val_cur;
	LONG val_count;

	b_gain_ctrl->get_Current(&val_cur);

	IUsgValues* usg_values = NULL;
	b_gain_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	if (idx >= val_count) idx = val_count - 1;
	if (idx < 0) idx = 0;

	if ((idx >= 0) && (idx < val_count))
	{
		VARIANT item;
		usg_values->Item(idx, &item);
		val_cur = item.lVal;
		LONG val;
		b_gain_ctrl->get_Current(&val);
		if (val != val_cur)
			b_gain_ctrl->put_Current(val_cur);
		VariantClear(&item);
	}

	SAFE_RELEASE(usg_values);
} // void CsampleDlg::B_GainSetByIdx


// update displayed gain label value
void CsampleDlg::B_GainUpdateGUI()
{
	if (b_gain_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	b_gain_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	b_gain_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	CString str;
	str.Format(L"%d %%", val_cur);
	SetDlgItemText(IDC_B_GAIN_STATIC, str);

	HWND hslider = NULL;
	hslider = (HWND)(*(GetDlgItem(IDC_B_GAIN_SLIDER)));

	int tmp_page;
	tmp_page = (int)::SendMessage(hslider, TBM_GETPAGESIZE, 0L, 0L);
	if (tmp_page != 1)
		::SendMessage(hslider, TBM_SETPAGESIZE, 0, 1);

	int tmp_min, tmp_max;
	tmp_min = (int)::SendMessage(hslider, TBM_GETRANGEMIN, 0L, 0L);
	tmp_max = (int)::SendMessage(hslider, TBM_GETRANGEMAX, 0L, 0L);

	if ((tmp_min != 0) || (tmp_max != val_count - 1))
		::SendMessage(hslider, TBM_SETRANGE, TRUE, (LPARAM)MAKELONG(0, val_count - 1));

	// perform asynchronous trackbar update in order to avoid locking
	if (val_idx >= 0)
	{
		int tmp_pos;
		tmp_pos = (int)::SendMessage(hslider, TBM_GETPOS, 0, 0);
		if (val_idx != tmp_pos)
			::PostMessage(hslider, TBM_SETPOS, 1, val_idx);
	}

	SAFE_RELEASE(usg_values);

} // void CsampleDlg::B_GainUpdateGUI

// --------------------------------------------------------

// set power value by passed index
void CsampleDlg::B_PowerSetByIdx(int idx)
{
	if (b_power_ctrl == NULL) return;

	LONG val_cur;
	LONG val_count;

	b_power_ctrl->get_Current(&val_cur);

	IUsgValues* usg_values = NULL;
	b_power_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	if (idx >= val_count) idx = val_count - 1;
	if (idx < 0) idx = 0;

	if ((idx >= 0) && (idx < val_count))
	{
		VARIANT item;
		usg_values->Item(idx, &item);
		val_cur = item.lVal;
		LONG val;
		b_power_ctrl->get_Current(&val);
		if (val != val_cur)
			b_power_ctrl->put_Current(val_cur);
		VariantClear(&item);
	}

	SAFE_RELEASE(usg_values);
} // void CsampleDlg::B_PowerSetByIdx


// update displayed power label value
void CsampleDlg::B_PowerUpdateGUI()
{
	if (b_power_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	b_power_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	b_power_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	CString str;
	str.Format(L"%d dB", val_cur);
	SetDlgItemText(IDC_B_POWER_STATIC, str);

	HWND hslider = NULL;
	hslider = (HWND)(*(GetDlgItem(IDC_B_POWER_SLIDER)));

	int tmp_page;
	tmp_page = (int)::SendMessage(hslider, TBM_GETPAGESIZE, 0L, 0L);
	if (tmp_page != 1)
		::SendMessage(hslider, TBM_SETPAGESIZE, 0, 1);

	int tmp_min, tmp_max;
	tmp_min = (int)::SendMessage(hslider, TBM_GETRANGEMIN, 0L, 0L);
	tmp_max = (int)::SendMessage(hslider, TBM_GETRANGEMAX, 0L, 0L);

	if ((tmp_min != 0) || (tmp_max != val_count - 1))
		::SendMessage(hslider, TBM_SETRANGE, TRUE, (LPARAM)MAKELONG(0, val_count - 1));

	// perform asynchronous trackbar update in order to avoid locking
	if (val_idx >= 0)
	{
		int tmp_pos;
		tmp_pos = (int)::SendMessage(hslider, TBM_GETPOS, 0, 0);
		if (val_idx != tmp_pos)
			::PostMessage(hslider, TBM_SETPOS, 1, val_idx);
	}

	SAFE_RELEASE(usg_values);

} // void CsampleDlg::B_PowerUpdateGUI


// --------------------------------------------------------

// update displayed B frequency label value
void CsampleDlg::B_FrequencyUpdateGUI()
{
	if (b_frequency_ctrl == NULL) return;
	FREQUENCY3 val;
	CString str;
	CString thi_str = L"";
	b_frequency_ctrl->get_Current(&val);
	if (val.nThiMode == THI_MODE2)
		thi_str = L"THI";
	else if (val.nThiMode == THI_MODE2_ITHI)
		thi_str = L"ITHI";
	str.Format(L"%.1f MHz %s", (double)val.nFrequency / 1000000, thi_str);
	SetDlgItemText(IDC_FREQUENCY_STATIC, str);
}

// increase or decrease B frequency
void CsampleDlg::B_FrequencySetPrevNext(int dir)
{
	if (b_frequency_ctrl == NULL) return;

	FREQUENCY3 val_cur;
	LONG val_idx;
	LONG val_count;

	b_frequency_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	b_frequency_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		FREQUENCY3* val;
		usg_values->Item(i1, &item);
		val = (FREQUENCY3*)(item.pvRecord);
		if ((val_cur.nFrequency == val->nFrequency) && (val_cur.nThiMode == val->nThiMode))
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			FREQUENCY3* val;
			usg_values->Item(val_idx, &item);
			val = (FREQUENCY3*)(item.pvRecord);
			if ((val_cur.nFrequency != val->nFrequency) || (val_cur.nThiMode != val->nThiMode))
				b_frequency_ctrl->put_Current(*val);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);

} // void B_FrequencySetPrevNext

// pevious B frequency (decrease)
void CsampleDlg::OnBnClickedBFrequencyPrevButton()
{
	B_FrequencySetPrevNext(-1);
	B_FrequencyUpdateGUI();
}

// next B frequency (increase)
void CsampleDlg::OnBnClickedBFrequencyNextButton()
{
	B_FrequencySetPrevNext(1);
	B_FrequencyUpdateGUI();
}

// --------------------------------------------------------

// update displayed B lines density label value
void CsampleDlg::B_LinesDensityUpdateGUI()
{
	if (b_lines_density_ctrl == NULL) return;
	LONG val;
	CString str = L"";
	b_lines_density_ctrl->get_Current(&val);
	if (val >= 1000)
	{
		val = val - 1000;
		str.Format(L"%d", val);
	}
	else
	{
		if (val == LINE_DENSITY_LOW)
			str = L"Low";
		else if (val == LINE_DENSITY_MEDIUM_FAST)
			str = L"Medium S";
		else if (val == LINE_DENSITY_MEDIUM)
			str = L"Medium";
		else if (val == LINE_DENSITY_STANDARD_FAST)
			str = L"Standard S";
		else if (val == LINE_DENSITY_STANDARD)
			str = L"Standard";
		else if (val == LINE_DENSITY_HIGH)
			str = L"High";
		else
			str.Format(L"%d", val);
	}
	SetDlgItemText(IDC_LINES_DENSITY_STATIC, str);
}

// set B lines density
void CsampleDlg::B_LinesDensitySet(LONG val)
{
	if (b_lines_density_ctrl == NULL) return;
	b_lines_density_ctrl->put_Current(val);
}

// increase or decrease B lines density
void CsampleDlg::B_LinesDensitySetPrevNext(int dir)
{
	if (b_lines_density_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	b_lines_density_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	b_lines_density_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			usg_values->Item(val_idx, &item);
			val_cur = item.lVal;
			LONG val;
			b_lines_density_ctrl->get_Current(&val);
			if (val != val_cur)
				b_lines_density_ctrl->put_Current(val_cur);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);

} // void B_LinesDensitySetPrevNext

// pevious B lines density (decrease)
void CsampleDlg::OnBnClickedBLinesDensityPrevButton()
{
	B_LinesDensitySetPrevNext(-1);
	B_LinesDensityUpdateGUI();
}

// next B lines density (increase)
void CsampleDlg::OnBnClickedBLinesDensityNextButton()
{
	B_LinesDensitySetPrevNext(1);
	B_LinesDensityUpdateGUI();
}

// --------------------------------------------------------

void CsampleDlg::SetSingleStaticFocus()
{
	if (data_view == NULL)
		return;

	IUnknown* tmp_obj = NULL;
	IUsgFocus* b_focus_ctrl = NULL;
	CreateUsgControl(data_view, IID_IUsgFocus, SCAN_MODE_B, 0, (void**)&tmp_obj);
	if (tmp_obj != NULL)
		b_focus_ctrl = (IUsgFocus*)tmp_obj;

	if (b_focus_ctrl == NULL)
		return;

	int focus_mode;
	int focus_set;
	LONG focal_zones_count;

	focus_mode = FOCUS_MODE_MULTI;
	focus_set = 0;

	b_focus_ctrl->put_CurrentMode(focus_mode);
	b_focus_ctrl->put_FocusSet(focus_mode, focus_set);

	b_focus_ctrl->GetFocalZonesCount(focus_mode, focus_set, &focal_zones_count);

	LONG *arr;
	arr = new LONG[focal_zones_count];

	for (int i1 = 0; i1 < focal_zones_count; i1++)
		arr[i1] = 0;

	arr[0] = 1;

	b_focus_ctrl->SetFocusState(focus_mode, focus_set, focal_zones_count, arr);

	delete[] arr;

	SAFE_RELEASE(b_focus_ctrl);
}

// update displayed B focus label value
void CsampleDlg::B_FocusUpdateGUI()
{
	if (b_focus_ctrl == NULL) return;

	LONG focus_mode = 0;
	LONG focus_set = 0;
	LONG focal_zones_count = 0;

	b_focus_ctrl->get_CurrentMode(&focus_mode);
	b_focus_ctrl->get_FocusSet(focus_mode, &focus_set);
	b_focus_ctrl->GetFocalZonesCount(focus_mode, focus_set, &focal_zones_count);

	// find the index of first active focus zone
	int focus_zone_idx = -1;
	LONG *arr;
	arr = new LONG[focal_zones_count];
	b_focus_ctrl->GetFocusState(focus_mode, focus_set, focal_zones_count, arr);
	for (int i1 = 0; i1 < focal_zones_count; i1++)
	{
		if (arr[i1] != 0)
		{
			focus_zone_idx = i1;
			break;
		}
	}
	delete[] arr;

	if (focus_zone_idx < 0)
		return;

	FOCAL_ZONE zone;
	b_focus_ctrl->GetFocalZone(focus_mode, focus_set, focus_zone_idx, &zone);

	CString str;
	str.Format(L"%d mm", zone.nFocusPoint);
	SetDlgItemText(IDC_FOCUS_STATIC, str);
}

// shift B focus
void CsampleDlg::B_FocusSetPrevNext(int dir)
{
	if (b_focus_ctrl == NULL) return;

	LONG focus_mode = 0;
	LONG focus_set = 0;
	LONG focal_zones_count = 0;

	b_focus_ctrl->get_CurrentMode(&focus_mode);
	b_focus_ctrl->get_FocusSet(focus_mode, &focus_set);
	b_focus_ctrl->GetFocalZonesCount(focus_mode, focus_set, &focal_zones_count);

	// find the index of first active focus zone
	int focus_zone_idx = -1;
	LONG *arr;
	arr = new LONG[focal_zones_count];
	b_focus_ctrl->GetFocusState(focus_mode, focus_set, focal_zones_count, arr);
	for (int i1 = 0; i1 < focal_zones_count; i1++)
	{
		if (arr[i1] != 0)
		{
			focus_zone_idx = i1;
			break;
		}
	}

	if (focus_zone_idx >= 0)
	{
		focus_zone_idx += dir;
		if ((focus_zone_idx >= 0) && (focus_zone_idx < focal_zones_count))
		{
			for (int i1 = 0; i1 < focal_zones_count; i1++)
				arr[i1] = 0;
			arr[focus_zone_idx] = 1;
			b_focus_ctrl->SetFocusState(focus_mode, focus_set, focal_zones_count, arr);
		}
	}

	delete[] arr;

} // void B_FocusSetPrevNext

// pevious B focus
void CsampleDlg::OnBnClickedBFocusPrevButton()
{
	B_FocusSetPrevNext(-1);
	B_FocusUpdateGUI();
}

// next B focus
void CsampleDlg::OnBnClickedBFocusNextButton()
{
	B_FocusSetPrevNext(1);
	B_FocusUpdateGUI();
}

// --------------------------------------------------------
bool CsampleDlg::B_ChangeScanDirectionGet()
{
	if (b_image_orientation_ctrl == NULL)
		return false;
	LONG val;
	b_image_orientation_ctrl->get_Mirror(&val);
	if (val == 1)
		return true;
	return false;
}

void CsampleDlg::OnBnClickedChangeScanDirectionCheck()
{
	if (change_scan_direction_updating_gui)
		return;
	if (data_view == NULL)
		return;
	bool val = B_ChangeScanDirectionGet();
	if ((m_change_scan_direction_ctrl.GetCheck() == BST_CHECKED) && !val)
		b_image_orientation_ctrl->put_Mirror(1);
	else if ((m_change_scan_direction_ctrl.GetCheck() != BST_CHECKED) && val)
		b_image_orientation_ctrl->put_Mirror(0);
}

void CsampleDlg::B_ChangeScanDirectionUpdateGui()
{
	if (b_image_orientation_ctrl == NULL)
		return;
	change_scan_direction_updating_gui = true;
	bool val = B_ChangeScanDirectionGet();
	if ((m_change_scan_direction_ctrl.GetCheck() == BST_CHECKED) && !val)
		m_change_scan_direction_ctrl.SetCheck(BST_UNCHECKED);
	else if ((m_change_scan_direction_ctrl.GetCheck() != BST_CHECKED) && val)
		m_change_scan_direction_ctrl.SetCheck(BST_CHECKED);
	change_scan_direction_updating_gui = false;
}

// --------------------------------------------------------

int CsampleDlg::B_RotationAngleGet()
{
	if (b_image_orientation_ctrl == NULL)
		return 0;
	tagImageRotate val;
	b_image_orientation_ctrl->get_Rotate(&val);
	if (val == IMAGE_ROTATE_NONE)
		return 0;
	else if (val == IMAGE_ROTATE_90CCW)
		return 90;
	else if (val == IMAGE_ROTATE_180CCW)
		return 180;
	else if (val == IMAGE_ROTATE_270CCW)
		return 270;
	return 0;
}

// --------------------------------------------------------

void CsampleDlg::B_SteeringAngleUpdateGUI()
{
	if (b_steering_angle_ctrl == NULL) return;
	LONG val;
	CString str = L"";
	b_steering_angle_ctrl->get_Current(&val);
	str.Format(L"%d", val);
	SetDlgItemText(IDC_STEERING_ANGLE_STATIC, str);
}

void CsampleDlg::B_SteeringAngleSetPrevNext(int dir)
{
	if (b_steering_angle_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	b_steering_angle_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	b_steering_angle_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			usg_values->Item(val_idx, &item);
			val_cur = item.lVal;
			LONG val;
			b_steering_angle_ctrl->get_Current(&val);
			if (val != val_cur)
				b_steering_angle_ctrl->put_Current(val_cur);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);
}

void CsampleDlg::OnBnClickedSteeringAnglePrevButton()
{
	B_SteeringAngleSetPrevNext(-1);
	B_SteeringAngleUpdateGUI();
}

void CsampleDlg::OnBnClickedSteeringAngleNextButton()
{
	B_SteeringAngleSetPrevNext(1);
	B_SteeringAngleUpdateGUI();
}

// --------------------------------------------------------
void CsampleDlg::WideViewAngleUpdateGUI()
{
	if (wideview_angle_ctrl == NULL) return;
	LONG val;
	CString str = L"";
	wideview_angle_ctrl->get_Current(&val);
	str.Format(L"%d", val);
	SetDlgItemText(IDC_WIDEVIEW_ANGLE_STATIC, str);
}

void CsampleDlg::WideViewAngleSetPrevNext(int dir)
{
	if (wideview_angle_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	wideview_angle_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	wideview_angle_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			usg_values->Item(val_idx, &item);
			val_cur = item.lVal;
			LONG val;
			wideview_angle_ctrl->get_Current(&val);
			if (val != val_cur)
				wideview_angle_ctrl->put_Current(val_cur);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);
}

void CsampleDlg::OnBnClickedWideviewAnglePrevButton()
{
	WideViewAngleSetPrevNext(-1);
	WideViewAngleUpdateGUI();
}

void CsampleDlg::OnBnClickedWideviewAngleNextButton()
{
	WideViewAngleSetPrevNext(1);
	WideViewAngleUpdateGUI();
}

// --------------------------------------------------------

void CsampleDlg::CompoundAngleUpdateGUI()
{
	if (compound_angle_ctrl == NULL) return;
	LONG val;
	CString str = L"";
	compound_angle_ctrl->get_Current(&val);
	str.Format(L"%d", val);
	SetDlgItemText(IDC_COMPOUND_ANGLE_STATIC, str);
}

void CsampleDlg::CompoundAngleSetPrevNext(int dir)
{
	if (compound_angle_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	compound_angle_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	compound_angle_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			usg_values->Item(val_idx, &item);
			val_cur = item.lVal;
			LONG val;
			compound_angle_ctrl->get_Current(&val);
			if (val != val_cur)
				compound_angle_ctrl->put_Current(val_cur);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);
}

void CsampleDlg::OnBnClickedCompoundAnglePrevButton()
{
	CompoundAngleSetPrevNext(-1);
	CompoundAngleUpdateGUI();
}

void CsampleDlg::OnBnClickedCompoundAngleNextButton()
{
	CompoundAngleSetPrevNext(1);
	CompoundAngleUpdateGUI();
}

// --------------------------------------------------------

void CsampleDlg::CompoundFramesUpdateGUI()
{
	if (compound_frames_ctrl == NULL) return;
	LONG val;
	CString str = L"";
	compound_frames_ctrl->get_Current(&val);
	str.Format(L"%d", val);
	SetDlgItemText(IDC_COMPOUND_FRAMES_STATIC, str);
}

void CsampleDlg::CompoundFramesSetPrevNext(int dir)
{
	if (compound_frames_ctrl == NULL) return;

	LONG val_cur;
	LONG val_idx;
	LONG val_count;

	compound_frames_ctrl->get_Current(&val_cur);
	val_idx = -1;

	IUsgValues* usg_values = NULL;
	compound_frames_ctrl->get_Values(&usg_values);
	if (usg_values == NULL) return;

	usg_values->get_Count(&val_count);

	for (LONG i1 = 0; i1 < val_count; i1++)
	{
		VARIANT item;
		LONG val;
		usg_values->Item(i1, &item);
		val = item.lVal;
		if (val_cur == val)
			val_idx = i1;
		VariantClear(&item);
	}

	if (val_idx >= 0)
	{
		val_idx += dir;
		if ((val_idx >= 0) && (val_idx < val_count))
		{
			VARIANT item;
			usg_values->Item(val_idx, &item);
			val_cur = item.lVal;
			LONG val;
			compound_frames_ctrl->get_Current(&val);
			if (val != val_cur)
				compound_frames_ctrl->put_Current(val_cur);
			VariantClear(&item);
		}
	}

	SAFE_RELEASE(usg_values);
}

void CsampleDlg::OnBnClickedCompoundFramesPrevButton()
{
	CompoundFramesSetPrevNext(-1);
	CompoundFramesUpdateGUI();
	CompoundSubframeUpdateGUI();
	PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
}

void CsampleDlg::OnBnClickedCompoundFramesNextButton()
{
	CompoundFramesSetPrevNext(1);
	CompoundFramesUpdateGUI();
	CompoundSubframeUpdateGUI();
	PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
}
// --------------------------------------------------------

void CsampleDlg::CompoundSubframeCheckRange()
{
	if (compound_frames_ctrl == NULL) return;
	LONG val_cur;
	compound_frames_ctrl->get_Current(&val_cur);
	int val;
	val = m_nSubFrameIndex;
	if (val >= val_cur)
		val = val_cur - 1;
	if (val < 0)
		val = 0;
	if (m_nSubFrameIndex != val)
	{
		m_nSubFrameIndex = val;
		PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
	}
}

void CsampleDlg::CompoundSubframeUpdateGUI()
{
	CompoundSubframeCheckRange();

	CString str = L"";
	str.Format(L"%d", m_nSubFrameIndex);
	SetDlgItemText(IDC_COMPOUND_SUBFRAME_STATIC, str);
}

void CsampleDlg::CompoundSubframeSetPrevNext(int dir)
{
	int val;
	val = m_nSubFrameIndex;
	if (dir > 0)
		val++;
	if (dir < 0)
		val--;

	LONG val_cur;
	compound_frames_ctrl->get_Current(&val_cur);
	if (val >= val_cur)
		val = val_cur - 1;
	if (val < 0)
		val = 0;
	m_nSubFrameIndex = val;
}

void CsampleDlg::OnBnClickedCompoundSubframePrevButton()
{
	CompoundSubframeSetPrevNext(-1);
	CompoundSubframeUpdateGUI();
	PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
}

void CsampleDlg::OnBnClickedCompoundSubframeNextButton()
{
	CompoundSubframeSetPrevNext(1);
	CompoundSubframeUpdateGUI();
	PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
}

// --------------------------------------------------------

BOOL CsampleDlg::OnQueryEndSession()
{
	if (!__super::OnQueryEndSession())
		return FALSE;

	return TRUE;
}

// ations when Windows session is ending
void CsampleDlg::OnEndSession(BOOL bEnding)
{
	__super::OnEndSession(bEnding);

	ReleaseUsgControls(true);
}

LRESULT CsampleDlg::OnUpdateBitmapMessage(WPARAM w, LPARAM l)
{
	w; l;
	LRESULT lRetValue = 0;
	do
	{
		UpdateAlphaBitmap();
	} while (false);
	return lRetValue;
}

// draw scale lines and maybe other graphics (alpha bitmap) over ultrasound image
void CsampleDlg::UpdateAlphaBitmap()
{
	if (mixer_control == NULL) return;
	EnterCriticalSection(&paint_cs);

	if (paint_hdc == NULL)
	{
		paint_hdc = CreateCompatibleDC(::GetDC(usg_wnd));
		paint_hbmp = CreateCompatibleBitmap(::GetDC(usg_wnd), img_w, img_h);
		SelectObject(paint_hdc, paint_hbmp);
	}


	tagUsgMixingBitmap BmpParams;
	BmpParams.clrSrcKey = 0;
	BmpParams.fAlpha = 1;
	BmpParams.rcSrc.left = 0;
	BmpParams.rcSrc.top = 0;
	BmpParams.rcSrc.right = img_w;
	BmpParams.rcSrc.bottom = img_h;
	BmpParams.rcDest.left = 20;
	BmpParams.rcDest.top = 20;
	BmpParams.rcDest.right = 20 + img_w;
	BmpParams.rcDest.bottom = 20 + img_h;
	BmpParams.dwFlags = UsgMixingBitmap_SrcColorKey | UsgMixingBitmap_hDC;
	BmpParams.hdc = paint_hdc;


	// fill everything with black color in order to have transparent mixed image
	FillRect(paint_hdc, &(BmpParams.rcSrc), (HBRUSH)GetStockObject(BLACK_BRUSH));

	SelectObject(paint_hdc, GetStockObject(DC_PEN));
	// SelectObject(paint_hdc, GetStockObject(DC_BRUSH));
	SelectObject(paint_hdc, GetStockObject(NULL_BRUSH));

	tagRECT rect1;
	rect1.left = 0;
	rect1.top = 0;
	rect1.right = rect1.left + img_w - 1;
	rect1.bottom = rect1.top + img_h - 1;

	// draw B mode scale lines
	DrawBImage(paint_hdc, rect1.left, rect1.top, rect1.right, rect1.bottom);

	LeaveCriticalSection(&paint_cs);

	// set image to mixer control that will paint passed image over ultrasound
	mixer_control->SetAlphaBitmap(&BmpParams);
}

int round2int(float r)
{
	return (int)((r >= 0.0) ? floor(r + 0.5) : ceil(r - 0.5));
}

// draw transparent image over B mode ultrasound
void CsampleDlg::DrawBImage(HDC& hdc, int x1, int y1, int x2, int y2)
{

	// SetDCBrushColor(hdc, RGB(255,0,0));

	SetDCPenColor(hdc, RGB(255, 0, 0));

	// draw lines to see image bounds
	// MoveToEx(hdc, x1, y1, NULL);
	// LineTo(hdc, x2, y1);
	// LineTo(hdc, x2, y2);
	// LineTo(hdc, x1, y2);
	// LineTo(hdc, x1, y1);

	if (image_properties_b == NULL)
		return;

	POINT probe_center_pnt;
	image_properties_b->GetOrigin(&probe_center_pnt, PIXELS_ORIGIN_OUTPUTRECT, 0);

	// draw ellipse at probe coordinate system center (0,0)
	//SetDCPenColor(hdc, RGB(255,0,0));
	//Ellipse(hdc, probe_center_pnt.x-2, probe_center_pnt.y-2, probe_center_pnt.x+4, probe_center_pnt.y+4);

	// resolution - pixels per meter
	tagImageResolution resolution;
	image_properties_b->GetResolution(&resolution, 0);


	// begin: paint scan direction marker
	int marker_x;
	int marker_y;
	int marker_r;
	int marker_dist;
	marker_dist = 20;
	marker_x = marker_dist;
	marker_y = marker_dist;
	marker_r = 5;

	bool is_mirror = B_ChangeScanDirectionGet();
	int r = B_RotationAngleGet();
	if (is_mirror == false)
	{
		if (r == 0)
		{
			marker_x = marker_dist;
			marker_y = marker_dist;
		}
		else if (r == 90)
		{
			marker_x = marker_dist;
			marker_y = y2 - marker_dist;
		}
		else if (r == 180)
		{
			marker_x = x2 - marker_dist;
			marker_y = y2 - marker_dist;
		}
		else if (r == 270)
		{
			marker_x = x2 - marker_dist;
			marker_y = marker_dist;
		}
	}
	else
	{
		if (r == 0)
		{
			marker_x = x2 - marker_dist;
			marker_y = marker_dist;
		}
		else if (r == 90)
		{
			marker_x = marker_dist;
			marker_y = marker_dist;
		}
		else if (r == 180)
		{
			marker_x = marker_dist;
			marker_y = y2 - marker_dist;
		}
		else if (r == 270)
		{
			marker_x = x2 - marker_dist;
			marker_y = y2 - marker_dist;
		}
	}

	SetDCPenColor(hdc, RGB(255, 255, 255));
	SelectObject(hdc, GetStockObject(WHITE_BRUSH));
	Ellipse(hdc, marker_x - marker_r, marker_y - marker_r, marker_x + marker_r, marker_y + marker_r);
	SelectObject(hdc, GetStockObject(NULL_BRUSH));

	// end: paint scan direction marker

	SetDCPenColor(hdc, RGB(0, 255, 255));

	// draw vertical scale line (ticks every 10 mm)
	float tmp_y1;
	tmp_y1 = (float)(probe_center_pnt.y);
	while (tmp_y1 < (float)y2)
	{
		MoveToEx(hdc, x1, round2int(tmp_y1), NULL);
		LineTo(hdc, x1 + 5, round2int(tmp_y1));
		tmp_y1 = tmp_y1 + (float)resolution.nYPelsPerUnit / 100;
	}

	// draw horizontal scale line (ticks every 10 mm)
	float tmp_x1;
	tmp_x1 = (float)x1;
	while (tmp_x1 < (float)x2)
	{
		MoveToEx(hdc, round2int(tmp_x1), y2, NULL);
		LineTo(hdc, round2int(tmp_x1), y2 - 5);
		tmp_x1 = tmp_x1 + (float)resolution.nXPelsPerUnit / 100;
	}

	int wnd_line1_num_x, wnd_line1_num_y;
	int wnd_line2_num_x, wnd_line2_num_y;
	int wnd_depth1_num_x, wnd_depth1_num_y;
	int wnd_depth2_num_x, wnd_depth2_num_y;

	if (m_pUsgWindowRF && scan_line_properties_b)
	{

		SetDCPenColor(hdc, RGB(0, 255, 255));

		LONG nStartLine, nEndLine, nStartDepth, nEndDepth;
		//		FLOAT m_line1_x1, m_line1_y1, m_line1_x2, m_line1_y2;
		//		FLOAT m_line2_x1, m_line2_y1, m_line2_x2, m_line2_y2;
		m_pUsgWindowRF->GetWindow(&nStartLine, &nEndLine, &nStartDepth, &nEndDepth);

		int nLinesNumber = nEndLine - nStartLine + 1;
		FLOAT *pCoordinates = new FLOAT[nLinesNumber * 4];
		int i;
		for (i = 0; i < nLinesNumber; i++)
		{
			// get line coordinates in meters
			scan_line_properties_b->GetCoordinates4(i + nStartLine, m_nSubFrameIndex, (pCoordinates + i * 4 + 0), (pCoordinates + i * 4 + 1), (pCoordinates + i * 4 + 2), (pCoordinates + i * 4 + 3), (FLOAT)nStartDepth / 1000, (FLOAT)nEndDepth / 1000);

			// and convert coordinates into pixels
			image_properties_b->UnitsToPixels(*(pCoordinates + i * 4 + 0), *(pCoordinates + i * 4 + 1), (pCoordinates + i * 4 + 0), (pCoordinates + i * 4 + 1), PIXELS_ORIGIN_OUTPUTRECT);
			image_properties_b->UnitsToPixels(*(pCoordinates + i * 4 + 2), *(pCoordinates + i * 4 + 3), (pCoordinates + i * 4 + 2), (pCoordinates + i * 4 + 3), PIXELS_ORIGIN_OUTPUTRECT);
		}

		// paint RF window line/depth values
		{
			SetBkColor(hdc, RGB(0, 0, 0));
			SetTextColor(hdc, RGB(0, 255, 255));
			CString str = L"";
			int line1_val, line2_val;
			line1_val = nStartLine + 1;  // +1 to show window line numbers starting from 1
			line2_val = nEndLine + 1;

			//---
			// window line1
			wnd_line1_num_x = round2int(*(pCoordinates + 0 * 4 + 0));
			wnd_line1_num_y = round2int(*(pCoordinates + 0 * 4 + 1));
			str.Format(L"%d", line1_val);

			wnd_line1_num_x -= 5;
			if (wnd_line1_num_x < 0) wnd_line1_num_x = 0;
			if ((wnd_line1_num_x + 60) > 512) wnd_line1_num_x = 512 - 60;

			wnd_line1_num_y -= 30;
			if (wnd_line1_num_y < 0) wnd_line1_num_y = 0;
			if ((wnd_line1_num_y + 20) > 512)
				wnd_line1_num_y = 512 - 20;
			if ((wnd_line1_num_x < 30) && (wnd_line1_num_y > (512 - 60))) wnd_line1_num_x = 30;

			TextOut(hdc, wnd_line1_num_x, wnd_line1_num_y, str, str.GetLength());
			//---

			//---
			// window line2
			wnd_line2_num_x = round2int(*(pCoordinates + (nLinesNumber - 1) * 4 + 0));
			wnd_line2_num_y = round2int(*(pCoordinates + (nLinesNumber - 1) * 4 + 1));
			str.Format(L"%d", line2_val);

			wnd_line2_num_x -= 5;
			if (wnd_line2_num_x < 0) wnd_line2_num_x = 0;
			if (!is_mirror)
			{
				if (wnd_line2_num_x < (wnd_line1_num_x + 30))
					wnd_line2_num_x = wnd_line1_num_x + 30;
			}
			else
			{
				if (wnd_line1_num_x < (wnd_line2_num_x + 30))
					wnd_line1_num_x = wnd_line2_num_x + 30;
			}
			if ((wnd_line2_num_x + 30) > 512) wnd_line2_num_x = 512 - 30;

			wnd_line2_num_y -= 30;
			if (wnd_line2_num_y < 0) wnd_line2_num_y = 0;
			if ((wnd_line2_num_y + 20) > 512) wnd_line2_num_y = 512 - 20;

			TextOut(hdc, wnd_line2_num_x, wnd_line2_num_y, str, str.GetLength());
			//---

			//---
			// window depth1
			wnd_depth1_num_x = round2int(*(pCoordinates + 0 * 4 + 0));
			wnd_depth1_num_y = round2int(*(pCoordinates + 0 * 4 + 1));
			str.Format(L"%d", nStartDepth);

			if (!is_mirror)
				wnd_depth1_num_x -= 30;
			else
				wnd_depth1_num_x += 15;
			if (wnd_depth1_num_x < 0) wnd_depth1_num_x = 0;
			if (wnd_depth1_num_x > (512 - 20)) wnd_depth1_num_x = (512 - 20);

			wnd_depth1_num_y -= 10;
			if (wnd_depth1_num_y < 0) wnd_depth1_num_y = 0;
			if ((wnd_depth1_num_x == 0) && (wnd_depth1_num_y == 0)) wnd_depth1_num_y = wnd_line1_num_y + 25; // move depth number below line1 number
			if ((wnd_depth1_num_y + 40) > 512) wnd_depth1_num_y = 512 - 40;
			TextOut(hdc, wnd_depth1_num_x, wnd_depth1_num_y, str, str.GetLength());
			//---

			//---
			// window depth2
			wnd_depth2_num_x = round2int(*(pCoordinates + 0 * 4 + 2));
			wnd_depth2_num_y = round2int(*(pCoordinates + 0 * 4 + 3));
			str.Format(L"%d", nEndDepth);

			if (!is_mirror)
				wnd_depth2_num_x -= 30;
			else
				wnd_depth2_num_x += 15;
			if (wnd_depth2_num_x < 0) wnd_depth2_num_x = 0;
			if (wnd_depth2_num_x > (512 - 20)) wnd_depth2_num_x = (512 - 20);

			wnd_depth2_num_y -= 10;
			if (wnd_depth2_num_y < 0) wnd_depth2_num_y = 0;
			if ((wnd_depth2_num_y + 20) > 512) wnd_depth2_num_y = 512 - 20;
			TextOut(hdc, wnd_depth2_num_x, wnd_depth2_num_y, str, str.GetLength());
			//---
		}

		// paint RF window
		int nX1, nY1, nY2, nX2;
		nX1 = round2int(*(pCoordinates + 0 * 4 + 2));
		nY1 = round2int(*(pCoordinates + 0 * 4 + 3));
		MoveToEx(hdc, nX1, nY1, NULL);
		i = 0;
		while (i < nLinesNumber)
		{
			nX2 = round2int(*(pCoordinates + i * 4 + 0));
			nY2 = round2int(*(pCoordinates + i * 4 + 1));
			if ((nX1 != nX2) || (nY1 != nY2))
			{
				LineTo(hdc, nX2, nY2);
				nX1 = nX2;
				nY1 = nY2;
			}
			i++;
		}
		i = nLinesNumber - 1;
		while (i >= 0)
		{
			nX2 = round2int(*(pCoordinates + i * 4 + 2));
			nY2 = round2int(*(pCoordinates + i * 4 + 3));
			if ((nX1 != nX2) || (nY1 != nY2))
			{
				LineTo(hdc, nX2, nY2);
				nX1 = nX2;
				nY1 = nY2;
			}

			// LineTo(hdc, round2int(*(pCoordinates + i * 4 + 2)), round2int(*(pCoordinates + i * 4 + 3)));
			i--;
		}

		if (!using_regs_line_mode)
		{
			// draw RF line position
			HGDIOBJ original = NULL;

			// save the original object
			HPEN hPen = CreatePen(PS_DOT, 1, RGB(255, 255, 0));
			original = SelectObject(hdc, hPen);

			SetBkColor(hdc, RGB(0, 0, 0));

			// SetDCPenColor(hdc, RGB(255,255,0));
			m_nLineIndex = min(nLinesNumber - 1, max(0, m_nLineIndex));

			{
				// paint RF line number
				int x, y;
				x = round2int(*(pCoordinates + m_nLineIndex * 4 + 2));
				y = round2int(*(pCoordinates + m_nLineIndex * 4 + 3));
				CString str = L"";
				str.Format(L"%d", m_nLineIndex + 1);
				SetBkColor(hdc, RGB(0, 0, 0));
				SetTextColor(hdc, RGB(255, 255, 0));
				x -= 5;
				if (x < 0) x = 0;
				if (x > (512 - 30))
					x = 512 - 30;

				y += 5;
				if (y < 0) y = 0;

				if ((y + 20) > 512)
				{
					y = 512 - 20;
					x += 10;
					if (x > (512 - 30))
						x = 512 - 30;
					if (x < 30) x = 30;
				}
				// if at right bottom we have window line number, move RF line number to the left
				if ((wnd_line2_num_x >= (512 - 30)) && (wnd_line2_num_y >= (512 - 40)) && (x >= (512 - 60)))
					x = 512 - 60;

				TextOut(hdc, x, y, str, str.GetLength());
			}

			// paint RF line inside RF window
			MoveToEx(hdc, round2int(*(pCoordinates + m_nLineIndex * 4 + 0)), round2int(*(pCoordinates + m_nLineIndex * 4 + 1)), NULL);
			LineTo(hdc, round2int(*(pCoordinates + m_nLineIndex * 4 + 2)), round2int(*(pCoordinates + m_nLineIndex * 4 + 3)));

			SelectObject(hdc, original);
			DeleteObject(hPen);
		}

		// begin: paint markers of lines that are written to device using "Write"
		if ((scan_line_properties_b != NULL) && (image_properties_b != NULL) && using_regs_line_mode)
		{
			LONG nStartLine, nEndLine, nStartDepth, nEndDepth;
			nStartLine = nEndLine = nStartDepth = nEndDepth = 0;
			if (m_pUsgWindowRF)
				m_pUsgWindowRF->GetWindow(&nStartLine, &nEndLine, &nStartDepth, &nEndDepth);

			int regs_line1_text_x = 0;
			if (regs_line1_idx >= 0)
			{
				HGDIOBJ original = NULL;
				HPEN hPen = CreatePen(PS_DOT, 1, RGB(34, 177, 76));
				original = SelectObject(hdc, hPen);
				SetBkColor(hdc, RGB(0, 0, 0));

				// paint line
				FLOAT rx1pt, ry1pt, rx2pt, ry2pt;
				rx1pt = *(pCoordinates + regs_line1_idx * 4 + 0);
				ry1pt = *(pCoordinates + regs_line1_idx * 4 + 1);
				rx2pt = *(pCoordinates + regs_line1_idx * 4 + 2);
				ry2pt = *(pCoordinates + regs_line1_idx * 4 + 3);
				MoveToEx(hdc, round2int(rx1pt), round2int(ry1pt), NULL);
				LineTo(hdc, round2int(rx2pt), round2int(ry2pt));

				// paint line number
				int x, y;
				x = round2int(rx2pt);
				y = round2int(ry2pt);
				CString str = L"";
				str.Format(L"%d", regs_line1_idx + 1);
				SetBkColor(hdc, RGB(0, 0, 0));
				SetTextColor(hdc, RGB(34, 177, 76));
				x -= 5;
				if (x < 0) x = 0;
				if (x > (512 - 30))
					x = 512 - 30;

				y += 5;
				if (y < 0) y = 0;

				if ((y + 20) > 512)
				{
					y = 512 - 20;
					x += 10;
					if (x > (512 - 30))
						x = 512 - 30;
					if (x < 30) x = 30;
				}
				// if at right bottom we have window line number, move line number to the left
				if ((wnd_line2_num_x >= (512 - 30)) && (wnd_line2_num_y >= (512 - 40)) && (x >= (512 - 60)))
					x = 512 - 60;

				regs_line1_text_x = x;

				TextOut(hdc, x, y, str, str.GetLength());

				SelectObject(hdc, original);
				DeleteObject(hPen);
			}

			if (regs_line2_idx >= 0)
			{
				HGDIOBJ original = NULL;
				HPEN hPen = CreatePen(PS_DOT, 1, RGB(237, 28, 36));
				original = SelectObject(hdc, hPen);
				SetBkColor(hdc, RGB(0, 0, 0));

				// paint line
				FLOAT rx1pt, ry1pt, rx2pt, ry2pt;
				rx1pt = *(pCoordinates + regs_line2_idx * 4 + 0);
				ry1pt = *(pCoordinates + regs_line2_idx * 4 + 1);
				rx2pt = *(pCoordinates + regs_line2_idx * 4 + 2);
				ry2pt = *(pCoordinates + regs_line2_idx * 4 + 3);
				MoveToEx(hdc, round2int(rx1pt), round2int(ry1pt), NULL);
				LineTo(hdc, round2int(rx2pt), round2int(ry2pt));

				// paint line number
				int x, y;
				x = round2int(rx2pt);
				y = round2int(ry2pt);
				CString str = L"";
				str.Format(L"%d", regs_line2_idx + 1);
				SetBkColor(hdc, RGB(0, 0, 0));
				SetTextColor(hdc, RGB(237, 28, 36));
				x -= 5;
				if (x < 0) x = 0;
				// if line 2 text is too close to line 1 text, move it to the right
				if ((x > regs_line1_text_x) && ((regs_line1_text_x + 30) > x))
					x = regs_line1_text_x + 30;
				if (x > (512 - 30))
					x = 512 - 30;

				y += 5;
				if (y < 0) y = 0;

				if ((y + 20) > 512)
				{
					y = 512 - 20;
					x += 10;
					if (x > (512 - 30))
						x = 512 - 30;
					if (x < 30) x = 30;
				}

				// if at right bottom we have window line number, move line number to the left
				if ((wnd_line2_num_x >= (512 - 30)) && (wnd_line2_num_y >= (512 - 40)) && (x >= (512 - 60)))
					x = 512 - 60;

				TextOut(hdc, x, y, str, str.GetLength());

				SelectObject(hdc, original);
				DeleteObject(hPen);
			}

		}
		// end: paint markers of lines that are written to device using "Write"

		delete[] pCoordinates;

	} // if (m_pUsgWindowRF && scan_line_properties_b)

	// draw focus marker
	if (b_focus_ctrl != NULL)
	{
		LONG focus_mode = 0;
		LONG focus_set = 0;
		LONG focal_zones_count = 0;

		b_focus_ctrl->get_CurrentMode(&focus_mode);
		b_focus_ctrl->get_FocusSet(focus_mode, &focus_set);
		b_focus_ctrl->GetFocalZonesCount(focus_mode, focus_set, &focal_zones_count);

		// find the index of first active focus zone
		int focus_zone_idx = -1;
		LONG *arr;
		arr = new LONG[focal_zones_count];
		b_focus_ctrl->GetFocusState(focus_mode, focus_set, focal_zones_count, arr);
		for (int i1 = 0; i1 < focal_zones_count; i1++)
		{
			if (arr[i1] != 0)
			{
				focus_zone_idx = i1;
				break;
			}
		}
		delete[] arr;

		if (focus_zone_idx >= 0)
		{
			FOCAL_ZONE zone;
			b_focus_ctrl->GetFocalZone(focus_mode, focus_set, focus_zone_idx, &zone);

			float ret_x, ret_y;
			image_properties_b->UnitsToPixels(0, (float)zone.nFocusPoint / 1000, &ret_x, &ret_y, PIXELS_ORIGIN_OUTPUTRECT);
			SetDCPenColor(hdc, RGB(255, 255, 100));
			MoveToEx(hdc, x1, round2int(ret_y), NULL);
			LineTo(hdc, x1, round2int(ret_y - 5));
			LineTo(hdc, x1 + 7, round2int(ret_y));
			LineTo(hdc, x1, round2int(ret_y + 5));
			LineTo(hdc, x1, round2int(ret_y));
		}

	} // if (b_focus_ctrl != NULL)


} // void CsampleDlg::DrawBImage


// write string to log/debug output
void CsampleDlg::WriteToLog(CString str)
{
	CString str2 = L"";
	CString t_str = L"";
	SYSTEMTIME t;
	GetLocalTime(&t);
	t_str.Format(L"%02d:%02d:%02d.%03d  ", t.wHour, t.wMinute, t.wSecond, t.wMilliseconds);
	str2 += t_str;
	str2 += str;
	str2 += L"\r\n";
	OutputDebugString(str2);
}

// set RF data I array when I is at every second position of 16-bit array
void CsampleDlg::SetRFDataI32(INT16 *arr, int len)
{
	if (rf_data_i_len != len)
	{
		delete[] rf_data_i_arr;
		rf_data_i_arr = NULL;
		rf_data_i_len = 0;
		if (len != 0)
		{
			rf_data_i_arr = new INT16[len];
			rf_data_i_len = len;
		}
	}
	if (len != 0)
	{
		INT16 *pData = arr;
		for (int i = 0; i < len; i++)
		{
			*(rf_data_i_arr + i) = *pData;
			pData += 2;
		}
		rf_data_i_len = len;
	}
}

// set RF data I array
void CsampleDlg::SetRFDataI16(INT16 *arr, int len)
{
	if (rf_data_i_len != len)
	{
		delete[] rf_data_i_arr;
		rf_data_i_arr = NULL;
		rf_data_i_len = 0;
		if (len != 0)
		{
			rf_data_i_arr = new INT16[len];
			rf_data_i_len = len;
		}
	}
	if (len != 0)
	{
		memcpy(rf_data_i_arr, arr, len * sizeof(INT16));
		rf_data_i_len = len;
	}
}


// set RF data Q array when Q is at every second position of 16-bit array
void CsampleDlg::SetRFDataQ32(INT16 *arr, int len)
{
	if (rf_data_q_len != len)
	{
		delete[] rf_data_q_arr;
		rf_data_q_arr = NULL;
		rf_data_q_len = 0;
		if (len != 0)
		{
			rf_data_q_arr = new INT16[len];
			rf_data_q_len = len;
		}
	}
	if (len != 0)
	{
		INT16 *pData = (arr + 1);
		for (int i = 0; i < len; i++)
		{
			*(rf_data_q_arr + i) = *pData;
			pData += 2;
		}
		rf_data_q_len = len;
	}
}

// calculate required scaling for arr values to fit them to (x2-x1) width
void CsampleDlg::UpdateScale(int x1, int y1, int x2, int y2, INT16* arr, int len, double& scale)
{
	int len2;
	//len2 = min(y2 - y1, len);		// calculate only for data that fits to rectangle height
	len2 = len;					// calculate for all data
	INT16 min_val, max_val;
	min_val = 0; max_val = 0;
	for (int i1 = 0; i1 < len2; i1++)
	{
		if (arr[i1] < min_val)
			min_val = arr[i1];
		if (arr[i1] > max_val)
			max_val = arr[i1];
	}

	double scale2;
	scale2 = (double)2 * max(abs(min_val), max_val) / (double)(x2 - x1);
	if (scale2 == 0)
		if (scale2 < 1)
			scale2 = 1;

	scale = max(scale, scale2);

	if (scale2 >= scale)
		scale = scale2;
	else if ((scale2 < scale) && (scale2 >= (scale * 80 / 100)))
	{
		// do nothing if current data can be fit using existing scale and it is in the interval of [80;100] percents of required scale
	}
	else
		scale = scale2;

	if (rf_iq_autofit == false)
	{
		//scale = 1;
		int range_val;
		range_val = (1 << display_range) - 1;
		scale = 2 * (double)range_val / (double)(x2 - x1); // we use +- bits display range, so here multiply by 2
	}
}

// to passed rectangle paint scaled RF data
void CsampleDlg::PaintArr(CDC& dc, COLORREF clr, int x1, int y1, int x2, int y2, INT16* arr, int len, double scale)
{
	CRgn rgn;
	rgn.CreateRectRgn(x1, y1, x2, y2);
	dc.SelectClipRgn(&rgn);
	dc.Rectangle(x1, y1, x2, y2);
	dc.SetDCPenColor(clr);
	int cx;
	cx = (x1 + x2) / 2;
	int nHeight = y2 - y1;
	int nLen1 = len;
	// nLen1 = min(len, nHeight);
	dc.MoveTo(cx + (int)floor((double)(arr[0]) / scale), y1);
	if (nLen1 < nHeight)
	{
		//		for (int i1 = 0; i1 < nLen1; i1++)
		//			dc.SetPixel(cx + (int)floor((double)(arr[i1])/scale), y1 + i1, clr);
		for (int i1 = 1; i1 < nLen1; i1++)
			dc.LineTo(cx + (int)floor((double)(arr[i1]) / scale), y1 + i1);
	}
	else
	{
		int nPixel = 0;
		int nSample = 0;
		int nDelta = 0;
		while (nPixel < nHeight)
		{
			//			 dc.SetPixel(cx + (int)floor((double)(arr[nSample])/scale), y1 + nPixel, clr);
			dc.LineTo(cx + (int)floor((double)(arr[nSample]) / scale), y1 + nPixel);
			nPixel++;
			nDelta += nLen1;
			while (nDelta > nHeight)
			{
				nDelta -= nHeight;
				nSample++;
			}

		}
	}

	dc.SelectClipRgn(NULL);
}


// paint RF data to window
void CsampleDlg::PaintRFData()
{
	static int nColor = 0;
	EnterCriticalSection(&paint_rf_data_cs);

	int rf_left, rf_right, rf_top, rf_bottom, dy;
	int bmp_w, bmp_h;

	dy = 20;
	rf_left = 550;
	rf_right = 750;
	rf_top = 1;
	rf_bottom = rf_top + dy + img_h;
	bmp_w = rf_right - rf_left + 1;
	bmp_h = rf_bottom - rf_top + 1;

	// at first paint to memory DC (bitmap) and then copy bitmap to client DC in order to avoid flicker
	CClientDC client_dc(this);
	CDC dc;
	CBitmap bmp;
	dc.CreateCompatibleDC(&client_dc);
	bmp.CreateCompatibleBitmap(&client_dc, bmp_w, bmp_h);
	dc.SelectObject(&bmp);

	COLORREF clr;
	clr = GetSysColor(COLOR_3DFACE);
	CBrush bk_brush(clr);
	CRect r;
	r = CRect(0, 0, bmp_w, bmp_h);
	dc.SelectObject(bk_brush);
	dc.FillRect(r, &bk_brush);
	dc.SetBkColor(clr);

	// paint I data
	if (rf_data_i_len > 0)
	{
		COLORREF clr2 = RGB(0, 0, 150);
		if (using_regs_line_mode)
			clr2 = RGB(34, 177, 76);

		CString str;
		str = L"I";
		int str_x = 40;

		if (using_regs_line_mode)
		{
			str_x = 26;
			str = L"Line 1";
			if (rf_data_source_point_id == 1)
			{
				int current_rf_line = m_nLineIndex + 1;
				if (current_rf_line % 2 == 0)
				{
					clr2 = RGB(237, 28, 36);
					str = L"Line 2";
				}
			}
		}

		CPen pen(PS_SOLID, 1, clr2);
		dc.SelectObject(pen);
		dc.SetTextColor(clr2);
		dc.TextOut(str_x, 1, str);
		UpdateScale(0, dy, bmp_w / 2 - 10, bmp_h, rf_data_i_arr, rf_data_i_len, rf_data_i_scale);
		PaintArr(dc, clr2, 0, dy, bmp_w / 2 - 10, bmp_h, rf_data_i_arr, rf_data_i_len, rf_data_i_scale / rf_data_i_scale2);
	}

	// paint Q data
	if ((rf_data_q_len > 0) && (rf_data_source_point_id == 4))
	{
		COLORREF clr2;
		clr2 = RGB(0, 150, 0);
		if (using_regs_line_mode)
			clr2 = RGB(237, 28, 36);

		CPen pen(PS_SOLID, 1, clr2);
		dc.SelectObject(pen);
		dc.SetTextColor(clr2);
		CString str;
		str = L"Q";
		int str_x = 150;

		if (using_regs_line_mode)
		{
			str_x = 135;
			str = L"Line 2";
		}

		dc.TextOut(str_x, 1, str);
		UpdateScale(bmp_w / 2 + 10, dy, bmp_w, bmp_h, rf_data_q_arr, rf_data_q_len, rf_data_q_scale);
		PaintArr(dc, clr2, bmp_w / 2 + 10, dy, bmp_w, bmp_h, rf_data_q_arr, rf_data_q_len, rf_data_q_scale / rf_data_q_scale2);
	}

	client_dc.BitBlt(rf_left, rf_top, bmp_w, bmp_h, &dc, 0, 0, SRCCOPY);

	nColor++;
	if (nColor > 4)
		nColor = 0;
	LeaveCriticalSection(&paint_rf_data_cs);
}

void CsampleDlg::OnBnClickedRadioAfterSum()
{
	if (IsDlgButtonChecked(IDC_RADIO_AFTER_SUM) == BST_CHECKED)
	{
		rf_data_source_point_id = 1;
		WriteToLog(L"RF Data: After sum");
		SetRfDataSourceByPointId(rf_data_source_point_id);
		UpdateIScaleGUI();
		UpdateQScaleGUI();
	}
}

void CsampleDlg::StartRfWrite()
{
	if (rf_data_file_handle != NULL)
		StopRfWrite();

	BSTR tmp_probe_name;
	probe->get_Name(&tmp_probe_name);

	std::ofstream out;
	rf_data_file_name = L"";

	wchar_t dir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, dir);
	rf_data_file_name += dir;
	rf_data_file_name += L"\\";

	CString fn2 = L"";
	time_t now;
	TCHAR date_str[MAX_DATE];
	date_str[0] = '\0';
	now = time(NULL);
	if (now != -1)
		wcsftime(date_str, MAX_DATE, L"%H.%M.%S_%d-%m-%Y", localtime(&now));

	fn2 += date_str;
	fn2 += L"_";
	fn2 += tmp_probe_name;
	fn2 += L".tmp";
	rf_data_file_name += fn2;

	SetDlgItemText(IDC_FILE_NAME_STATIC, fn2);

	rf_data_file_handle = _wfopen(rf_data_file_name, L"wb");
	fwrite("RF0003", sizeof(char), 6, rf_data_file_handle);	// write RF file format version
	number_of_KBs = 0;
}

void CsampleDlg::StopRfWrite()
{
	if (rf_data_file_handle == NULL)
		return;

	number_of_KBs = ftell(rf_data_file_handle) / 1024;
	fclose(rf_data_file_handle);
	rf_data_file_handle = NULL;

	check_for_stop = false;
	do_stop = false;
	record_single_frame = false;
	record_n_frames = false;
	recorded_n_frames = 0;

	wchar_t dir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, dir);
	CString src_fn = L"";
	src_fn = rf_data_file_name;
	TCHAR szFilters[] = _T("RF data file (*.bin)|*.bin||");
	CString fn2;
	fn2 = rf_data_file_name.Mid(0, rf_data_file_name.GetLength() - 4);

	CFileDialog fileDlg(FALSE, _T("bin"), fn2, OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY, szFilters, this);

	if (fileDlg.DoModal() == IDOK)
	{
		CString pathName = fileDlg.GetPathName();
		CString fn3 = fileDlg.GetFileName();
		DeleteFile(pathName);
		MoveFile(src_fn, pathName);
		SetDlgItemText(IDC_FILE_NAME_STATIC, fn3);
	}
	else
	{
		DeleteFile(src_fn);
		SetDlgItemText(IDC_FILE_NAME_STATIC, L"");
	}

	EnableDisableUserInterfaceControls();
}

void CsampleDlg::OnBnClickedRadioWriteRf()
{
	if (IsDlgButtonChecked(IDC_RADIO_WRITE_RF) == BST_CHECKED)
	{
		record_single_frame = false;
		record_n_frames = false;
		do_stop = false;
		StartRfWrite();
		check_for_stop = true;
	}
	EnableDisableUserInterfaceControls();
}

void CsampleDlg::OnBnClickedRadioStopRf()
{
	if (IsDlgButtonChecked(IDC_RADIO_STOP_RF) == BST_CHECKED)
	{
		do_stop = true;
		record_single_frame = false;
		record_n_frames = false;
	}
	EnableDisableUserInterfaceControls();
}

void CsampleDlg::SetRfDataSourceByPointId(int point_id)
{
	int source_id = 0;
	if (point_id == 1)
		source_id = 3;
	else if (point_id == 2)
		source_id = 2;
	else if (point_id == 3)
		source_id = 1;
	else if (point_id == 4)
		source_id = 0;
	else
		return;

	PaintRFData();
	ShowHideQScaleControls();
	if (m_pUsgDataSourceRF)
		m_pUsgDataSourceRF->put_Current(source_id);
}

void CsampleDlg::OnBnClickedRadioAfterTfc()
{
	if (IsDlgButtonChecked(IDC_RADIO_AFTER_TFC) == BST_CHECKED)
	{
		rf_data_source_point_id = 2;
		WriteToLog(L"RF Data: After TFC filter");
		SetRfDataSourceByPointId(rf_data_source_point_id);
		UpdateIScaleGUI();
		UpdateQScaleGUI();
	}
}

void CsampleDlg::OnBnClickedRadioAfterApodization()
{
	if (IsDlgButtonChecked(IDC_RADIO_AFTER_APODIZATION) == BST_CHECKED)
	{
		rf_data_source_point_id = 3;
		WriteToLog(L"RF Data: After apodization");
		SetRfDataSourceByPointId(rf_data_source_point_id);
		UpdateIScaleGUI();
		UpdateQScaleGUI();
	}
}

void CsampleDlg::OnBnClickedRadioAfterHilbert()
{
	if (IsDlgButtonChecked(IDC_RADIO_AFTER_HILBERT) == BST_CHECKED)
	{
		rf_data_source_point_id = 4;
		WriteToLog(L"RF Data: After Hilbert transform");
		SetRfDataSourceByPointId(rf_data_source_point_id);
		UpdateIScaleGUI();
		UpdateQScaleGUI();
	}
}


// RF window position up
void CsampleDlg::OnBnClickedButtonPositionUp()
{
	RF_WindowMove(0, -1);
}

// RF window position down
void CsampleDlg::OnBnClickedButtonPositionDown()
{
	RF_WindowMove(0, 1);
}

// RF window position left
void CsampleDlg::OnBnClickedButtonPositionLeft()
{
	RF_WindowMove(-1, 0);
}

// RF window position right
void CsampleDlg::OnBnClickedButtonPositionRight()
{
	RF_WindowMove(1, 0);
}

// RF window size up
void CsampleDlg::OnBnClickedButtonSizeUp()
{
	RF_WindowSize(0, 1);
}

// RF window size down
void CsampleDlg::OnBnClickedButtonSizeDown()
{
	RF_WindowSize(0, -1);
}

// RF window size left
void CsampleDlg::OnBnClickedButtonSizeLeft()
{
	RF_WindowSize(-1, 0);
}

// RF window size right
void CsampleDlg::OnBnClickedButtonSizeRight()
{
	RF_WindowSize(1, 0);
}

// update RF line indicator
void CsampleDlg::UpdateLineIndicator()
{
	CString str = L"RF Scan Line: -/-";
	if (m_pUsgWindowRF != NULL)
	{
		int tmp_current = 0;
		int tmp_total = 0;
		tmp_current = m_nLineIndex + 1;

		LONG depth1, depth2;
		LONG line1, line2;
		depth1 = depth2 = line1 = line2 = 0;
		m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

		tmp_total = line2 - line1 + 1;

		if (using_regs_line_mode)
			str.Format(L"Viewing Line: %d/%d", tmp_current, tmp_total);
		else
			str.Format(L"RF Scan Line: %d/%d", tmp_current, tmp_total);
	}
	SetDlgItemText(IDC_RF_LINE, str);
}

// previous RF scan line
void CsampleDlg::OnBnClickedLinePrevButton()
{
	RF_LineMove(-1);
}

// next RF scan line
void CsampleDlg::OnBnClickedLineNextButton()
{
	RF_LineMove(1);
}

void CsampleDlg::RF_LineMove(int dx)
{
	if (!m_pUsgWindowRF)
		return;

	LONG depth1, depth2;
	LONG line1, line2;
	depth1 = depth2 = line1 = line2 = 0;
	m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

	int dx2;
	dx2 = dx;

	bool is_mirror = B_ChangeScanDirectionGet();
	int r = B_RotationAngleGet();
	if (is_mirror == false)
	{
		if (r == 0)
			dx2 = dx;
		else if (r == 90)
			dx2 = -dx;
		else if (r == 180)
			dx2 = -dx;
		else if (r == 270)
			dx2 = dx;
	}
	else
	{
		if (r == 0)
			dx2 = -dx;
		else if (r == 90)
			dx2 = dx;
		else if (r == 180)
			dx2 = dx;
		else if (r == 270)
			dx2 = -dx;
	}

	if (dx2 > 0)
	{
		if (m_nLineIndex < (line2 - line1))
			m_nLineIndex++;
	}
	else if (dx2 < 0)
	{
		if (m_nLineIndex > 0)
			m_nLineIndex--;
	}

	// if line position is changed in freeze, push frame to get data through callback
	SCAN_STATE scan_state;
	data_view->get_ScanState(&scan_state);
	if (scan_state == SCAN_STATE_FREEZE)
	{
		CINE_STATE state = CINE_STATE_PLAY;
		m_pUsgCine->get_State(&state);
		if (state != CINE_STATE_PLAY)
		{
			INT64 position;
			m_pUsgCine->get_CurrentPosition(&position);
			m_pUsgCine->put_CurrentPosition(position);
		}
	}

	UpdateLineIndicator();
	UpdateIScaleGUI();
	PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
}

void CsampleDlg::RF_WindowUpdatePosition()
{
	if (!m_pUsgWindowRF)
		return;

	LONG depth1, depth2;
	LONG line1, line2;

	// get current RF window size
	m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

	if (m_nLineIndex > (line2 - line1))
	{
		int nOldLineIndex = m_nLineIndex;
		m_nLineIndex = (line2 - line1) / 2;
	}
}

// change position of RF window
void CsampleDlg::RF_WindowMove(int dx, int dy)
{
	if (!m_pUsgWindowRF)
		return;

	LONG depth1, depth2;
	LONG line1, line2;

	// get current RF window size
	m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

	int dx2, dy2;
	dx2 = dx;
	dy2 = dy;

	bool is_mirror = B_ChangeScanDirectionGet();
	int r = B_RotationAngleGet();
	if (is_mirror == false)
	{
		if (r == 0)
		{
			dx2 = dx;
			dy2 = dy;
		}
		else if (r == 90)
		{
			dx2 = -dy;
			dy2 = dx;
		}
		else if (r == 180)
		{
			dx2 = -dx;
			dy2 = -dy;
		}
		else if (r == 270)
		{
			dx2 = dy;
			dy2 = -dx;
		}
	}
	else
	{
		if (r == 0)
		{
			dx2 = -dx;
			dy2 = dy;
		}
		else if (r == 90)
		{
			dx2 = dy;
			dy2 = dx;
		}
		else if (r == 180)
		{
			dx2 = dx;
			dy2 = -dy;
		}
		else if (r == 270)
		{
			dx2 = -dy;
			dy2 = -dx;
		}
	}

	// calculate new RF window size
	line1 += dx2;
	line2 += dx2;
	depth1 += dy2;
	depth2 += dy2;
	VALUE_STEPPING LinesRange;
	VALUE_STEPPING DepthsRange;
	m_pUsgWindowRF->get_LinesRange(&LinesRange);
	m_pUsgWindowRF->get_DepthsRange(&DepthsRange);

	// if new values are out of range - return
	if (line1 < LinesRange.minimum) return;
	if (line2 > LinesRange.maximum) return;

	if (depth1 < DepthsRange.minimum) return;
	if (depth2 > DepthsRange.maximum) return;

	// set new RF window size
	m_pUsgWindowRF->SetWindow(line1, line2, depth1, depth2);

}

// change size of RF window
void CsampleDlg::RF_WindowSize(int dx, int dy)
{
	if (!m_pUsgWindowRF)
		return;

	LONG depth1, depth2;
	LONG line1, line2;

	// get current RF window size
	m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

	int dx2, dy2;
	dx2 = dx;
	dy2 = dy;

	bool is_mirror = B_ChangeScanDirectionGet();
	int r = B_RotationAngleGet();
	if (is_mirror == false)
	{
		if (r == 0)
		{
			dx2 = dx;
			dy2 = dy;
		}
		else if (r == 90)
		{
			dx2 = dy;
			dy2 = dx;
		}
		else if (r == 180)
		{
			dx2 = dx;
			dy2 = dy;
		}
		else if (r == 270)
		{
			dx2 = dy;
			dy2 = dx;
		}
	}
	else
	{
		if (r == 0)
		{
			dx2 = dx;
			dy2 = dy;
		}
		else if (r == 90)
		{
			dx2 = dy;
			dy2 = dx;
		}
		else if (r == 180)
		{
			dx2 = dx;
			dy2 = dy;
		}
		else if (r == 270)
		{
			dx2 = dy;
			dy2 = dx;
		}
	}

	// calculate new RF window size
	line1 -= dx2;
	line2 += dx2;
	depth1 -= dy2;
	depth2 += dy2;

	VALUE_STEPPING WidthRange;
	VALUE_STEPPING VolumeRange;
	m_pUsgWindowRF->get_WidthRange(&WidthRange);
	m_pUsgWindowRF->get_VolumesRange(&VolumeRange);

	VALUE_STEPPING LinesRange;
	VALUE_STEPPING DepthsRange;
	m_pUsgWindowRF->get_LinesRange(&LinesRange);
	m_pUsgWindowRF->get_DepthsRange(&DepthsRange);

	if (line1 < LinesRange.minimum)
		line1 = LinesRange.minimum;
	else if (line1 > LinesRange.maximum)
		line1 = LinesRange.maximum;

	if (line2 < LinesRange.minimum)
		line2 = LinesRange.minimum;
	else if (line2 > LinesRange.maximum)
		line2 = LinesRange.maximum;

	if (depth1 < DepthsRange.minimum)
		depth1 = DepthsRange.minimum;
	else if (depth1 > DepthsRange.maximum)
		depth1 = DepthsRange.maximum;

	if (depth2 < DepthsRange.minimum)
		depth2 = DepthsRange.minimum;
	else if (depth2 > DepthsRange.maximum)
		depth2 = DepthsRange.maximum;

	if ((line2 - line1) < WidthRange.minimum) return;
	if ((line2 - line1) > WidthRange.maximum) return;
	if ((depth2 - depth1) < VolumeRange.minimum) return;
	if ((depth2 - depth1) > VolumeRange.maximum) return;

	// set new RF window size
	m_pUsgWindowRF->SetWindow(line1, line2, depth1, depth2);

}

// additional amplitude scaling for I;
// this scaling may be used if automatic scaling fits everything to near zero because of several large array values

void CsampleDlg::UpdateIScaleGUI()
{
	HWND hwnd = NULL;
	hwnd = (HWND)(*(GetDlgItem(IDC_I_SCALE_STATIC)));
	TCHAR buf[MAX_PATH];

	if (using_regs_line_mode)
	{
		swprintf_s(buf, MAX_PATH, L"Line 1 Scale: %.1f", rf_data_i_scale2);
		if (rf_data_source_point_id == 1)
		{
			int current_rf_line = m_nLineIndex + 1;
			if (current_rf_line % 2 == 0)
				swprintf_s(buf, MAX_PATH, L"Line 2 Scale: %.1f", rf_data_i_scale2);
		}
	}
	else
		swprintf_s(buf, MAX_PATH, L"I Scale: %.1f", rf_data_i_scale2);

	::SetWindowText(hwnd, buf);
}

void CsampleDlg::OnBnClickedIScalePrevButton()
{
	double val;
	val = rf_data_i_scale2 - 0.1;
	if (val < 0.1)
		val = 0.1;
	rf_data_i_scale2 = val;
	UpdateIScaleGUI();
	PaintRFData();
}

void CsampleDlg::OnBnClickedIScaleNextButton()
{
	double val;
	val = rf_data_i_scale2 + 0.1;
	if (val > 5)
		val = 5;
	rf_data_i_scale2 = val;
	UpdateIScaleGUI();
	PaintRFData();
}

// additional amplitude scaling for Q
void CsampleDlg::ShowHideQScaleControls()
{
	int show = SW_HIDE;
	if (rf_data_source_point_id == 4)
		show = SW_SHOW;
	//if (m_pUsgWindowRF == NULL)
	//	show = SW_HIDE;

	::ShowWindow((HWND)(*(GetDlgItem(IDC_Q_SCALE_STATIC))), show);
	::ShowWindow((HWND)(*(GetDlgItem(IDC_Q_SCALE_PREV_BUTTON))), show);
	::ShowWindow((HWND)(*(GetDlgItem(IDC_Q_SCALE_NEXT_BUTTON))), show);
}

void CsampleDlg::UpdateQScaleGUI()
{
	HWND hwnd = NULL;
	hwnd = (HWND)(*(GetDlgItem(IDC_Q_SCALE_STATIC)));
	TCHAR buf[MAX_PATH];

	if (using_regs_line_mode)
		swprintf_s(buf, MAX_PATH, L"Line 2 Scale: %.1f", rf_data_q_scale2);
	else
		swprintf_s(buf, MAX_PATH, L"Q Scale: %.1f", rf_data_q_scale2);

	::SetWindowText(hwnd, buf);
}

void CsampleDlg::OnBnClickedQScalePrevButton()
{
	double val;
	val = rf_data_q_scale2 - 0.1;
	if (val < 0.1)
		val = 0.1;
	rf_data_q_scale2 = val;
	UpdateQScaleGUI();
	PaintRFData();
}

void CsampleDlg::OnBnClickedQScaleNextButton()
{
	double val;
	val = rf_data_q_scale2 + 0.1;
	if (val > 5)
		val = 5;
	rf_data_q_scale2 = val;
	UpdateQScaleGUI();
	PaintRFData();
}

// process received RF data sample
void CsampleDlg::ProcessSample(PBYTE pBuffer, int nBufferLen, PULTRASOUNDRF_INFO pFrameInfo, int nSubFrameIndex, bool updateGUI)
{
	do
	{
		if (!pBuffer || !nBufferLen || !pFrameInfo)
			break;

		m_nLineIndex = min((LONG)pFrameInfo->UsgFrameInfoHeader.BeamsPerFrame - 1, max(0, m_nLineIndex));

		int nOffset = pFrameInfo->UsgFrameInfoHeader.Pitch*(m_nLineIndex);

		if (pFrameInfo->UsgFrameInfoHeader.ChannelsPerBeam)
			nOffset *= pFrameInfo->UsgFrameInfoHeader.ChannelsPerBeam;

		if (pFrameInfo->UsgFrameInfoHeader.SubBeamsNumber)
			nOffset *= pFrameInfo->UsgFrameInfoHeader.SubBeamsNumber;

		bool set_iq = true;
		if (scan_type_ctrl != NULL)
		{
			LONG scan_type;
			scan_type_ctrl->get_Current(&scan_type);
			if ((compound_is_supported) && (scan_type == SCAN_TYPE_COMPOUND) && (nSubFrameIndex != m_nSubFrameIndex))
				set_iq = false;
		}

		if (pFrameInfo->UsgFrameInfoHeader.SampleBitCount == 32)
		{
			if (set_iq)
			{
				SetRFDataI32((INT16*)(pBuffer + nOffset + 4), pFrameInfo->UsgFrameInfoHeader.SamplesPerChannel - 2);	// 4 bytes (two 16-bit samples) at the beginning are used for timestamp
				SetRFDataQ32((INT16*)(pBuffer + nOffset + 4), pFrameInfo->UsgFrameInfoHeader.SamplesPerChannel - 2);
			}

			// Writes full frame to file with step 2 (+= flag) for Hilbert transform output    

			if (check_for_stop || record_single_frame || record_n_frames)
			{
				SetRFDataI32_full_frame(pFrameInfo, pBuffer, 2, nSubFrameIndex);

				recorded_n_frames++;
				if ((recorded_n_frames >= number_of_frames_to_record) && record_n_frames)
				{
					record_n_frames = false;
					StopRfWrite();
				}
				else if (record_single_frame)
				{
					record_single_frame = false;
					StopRfWrite();
				}
				else if (check_for_stop && do_stop)
				{
					check_for_stop = false;
					do_stop = false;
					StopRfWrite();
				}
			}
		}

		else
		{
			if (set_iq)
			{
				SetRFDataI16((INT16*)(pBuffer + nOffset + 4), pFrameInfo->UsgFrameInfoHeader.SamplesPerChannel - 2);	// 4 bytes (two 16-bit samples) at the beginning are used for timestamp
				SetRFDataQ32(NULL, 0);		// do not paint
			}

			// Writes full frame to file (if exists) with step 1 (+= flag) for other 3 outputs  

			if (check_for_stop || record_single_frame || record_n_frames)
			{
				SetRFDataI32_full_frame(pFrameInfo, pBuffer, 1, nSubFrameIndex); // writes full frame while scanning

				recorded_n_frames++;
				if ((recorded_n_frames >= number_of_frames_to_record) && record_n_frames)
				{
					record_n_frames = false;
					StopRfWrite();
				}
				else if (record_single_frame)
				{
					record_single_frame = false;
					StopRfWrite();
				}
				else if (check_for_stop && do_stop)
				{
					check_for_stop = false;
					do_stop = false;
					StopRfWrite();
				}

			}

		}

		PaintRFData();
		// WriteToLog(_T("RF frame processed"));

		if (updateGUI)
			PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);

		m_nFramesNumberRF++;

	} while (false);

	if (using_rf_stream == true) // if using RF stream
	{
		if (freeze_after_scanning_frames_num > 0)
		{
			freeze_after_scanning_frames_num--;
			if (freeze_after_scanning_frames_num <= 0)
			{
				freeze_after_scanning_frames_num = 0;
				PostMessage(WM_MESSAGE_FREEZE_AFTER_SCANNING, 0, 0);
				//SendMessage(WM_MESSAGE_FREEZE_AFTER_SCANNING, 0, 0);
			}
		}
	}
}

void CsampleDlg::SetRFDataI32_full_frame(PULTRASOUNDRF_INFO pFrameInfo, PBYTE pBuffer, int flag, int nSubFrameIndex)
{
	if (rf_data_file_handle == NULL)
		return;

	// sampling period (25 ns)
	long int sampling_period = pFrameInfo->SamplingTime;
	// number of samples in a row
	long int length = pFrameInfo->UsgFrameInfoHeader.SamplesPerChannel - 2; // -2 to drop 4 bytes for time stamp 
	LONG nStartLine, nEndLine, nStartDepth, nEndDepth;

	m_pUsgWindowRF->GetWindow(&nStartLine, &nEndLine, &nStartDepth, &nEndDepth);
	// number of RF rows
	long int number_of_RF_rows = nEndLine - nStartLine + 1;
	// Tx frequency
	long int tx_frequency;
	FREQUENCY3 freq;
	b_frequency_ctrl->get_Current(&freq);
	tx_frequency = freq.nFrequency;
	// Average frame rate
	long int AvgFPS;
	m_pUsgQualProp->get_AvgFrameRate(&AvgFPS); // B mode properties

	long int no_of_bits = pFrameInfo->UsgFrameInfoHeader.SampleBitCount;

	if (!record_single_frame)
		fwrite(&number_of_frames_to_record, sizeof(int), 1, rf_data_file_handle);
	else
	{
		int tmp_val = 1;
		fwrite(&tmp_val, sizeof(int), 1, rf_data_file_handle);
	}

	// write size of header and RF data frame in bytes
	int header_size = 11 * 4 + number_of_RF_rows * 3 * 4 + number_of_RF_rows * 4;
	int rf_frame_size = 2 * number_of_RF_rows*length;
	if (rf_data_source_point_id == 4)
		rf_frame_size = rf_frame_size * 2;

	fwrite(&header_size, sizeof(int), 1, rf_data_file_handle);
	fwrite(&rf_frame_size, sizeof(int), 1, rf_data_file_handle);

	fwrite(&rf_data_source_point_id, sizeof(int), 1, rf_data_file_handle);

	fwrite(&tx_frequency, sizeof(int), 1, rf_data_file_handle);
	fwrite(&AvgFPS, sizeof(int), 1, rf_data_file_handle);
	fwrite(&length, sizeof(int), 1, rf_data_file_handle);
	fwrite(&number_of_RF_rows, sizeof(int), 1, rf_data_file_handle);
	fwrite(&sampling_period, sizeof(int), 1, rf_data_file_handle);
	fwrite(&no_of_bits, sizeof(int), 1, rf_data_file_handle);
	// start depth of RF window
	fwrite(&nStartDepth, sizeof(int), 1, rf_data_file_handle);

	int row_start, row_end, row_step;
	LONG val;
	b_image_orientation_ctrl->get_Mirror(&val);
	if (val == 0)
	{
		row_start = 0;
		row_end = number_of_RF_rows - 1;
		row_step = 1;
	}
	else
	{
		row_start = number_of_RF_rows - 1;
		row_end = 0;
		row_step = -1;
	}



	if (val == 1)
	{
		for (int j = row_start; j >= row_end; j += row_step)
		{

			int beam_x = -((int)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.StartPoint.x);
			int beam_y = ((int)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.StartPoint.y);
			int angle = -(int)(1000000 * (double)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.Direction.value *(double)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.Direction.ScaleFactor.Numerator / (double)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.Direction.ScaleFactor.Denominator);
			fwrite(&beam_x, sizeof(int), 1, rf_data_file_handle);
			fwrite(&beam_y, sizeof(int), 1, rf_data_file_handle);
			fwrite(&angle, sizeof(int), 1, rf_data_file_handle);
		}

		// writes time stamps to RF data files
		if (rf_data_source_point_id != 4) {
			WORD A[2];
			for (int j = row_start; j >= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr;
				for (int i = 0; i < 2; i++)
				{
					A[i] = (WORD)pData[i];
				}
				unsigned int time_stamps = A[1] * 256 * 256 + A[0];

				if (rf_data_file_handle != NULL)

					fwrite(&time_stamps, sizeof(unsigned int), 1, rf_data_file_handle);
			}
		}
		// for rf_data_source_point_id = 4 
		else {

			WORD A[2];
			for (int j = row_start; j >= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr;
				INT16 *pData1 = arr + 1;

				A[0] = (WORD)pData[0];
				A[1] = (WORD)pData1[0];

				unsigned int time_stamps = A[1] * 256 * 256 + A[0];

				if (rf_data_file_handle != NULL)

					fwrite(&time_stamps, sizeof(unsigned int), 1, rf_data_file_handle);
			}
		}

		if (rf_data_source_point_id == 4)
		{
			for (int j = row_start; j >= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr + 2;
				for (int i = 0; i < length; i++)
				{
					if (rf_data_file_handle != NULL)
						fwrite(pData, sizeof(INT16), 1, rf_data_file_handle);
					pData += flag;
				}
			}
			for (int j = row_start; j >= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = (arr + 3);
				for (int i = 0; i < length; i++)
				{
					if (rf_data_file_handle != NULL)
						fwrite(pData, sizeof(INT16), 1, rf_data_file_handle);
					pData += flag;
				}
			}
		}
		else
		{
			for (int j = row_start; j >= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr + 2;
				for (int i = 0; i < length; i++)
				{
					if (rf_data_file_handle != NULL)
						fwrite(pData, sizeof(INT16), 1, rf_data_file_handle);
					pData += flag;
				}
			}
		}

	}

	else {
		for (int j = row_start; j <= row_end; j += row_step)
		{
			int beam_x = ((int)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.StartPoint.x);
			int beam_y = ((int)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.StartPoint.y);
			int angle = (int)(1000000 * (double)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.Direction.value *(double)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.Direction.ScaleFactor.Numerator / (double)pFrameInfo->UsgBeamVector[j + nSubFrameIndex * number_of_RF_rows].rpVector.Direction.ScaleFactor.Denominator);
			fwrite(&beam_x, sizeof(int), 1, rf_data_file_handle);
			fwrite(&beam_y, sizeof(int), 1, rf_data_file_handle);
			fwrite(&angle, sizeof(int), 1, rf_data_file_handle);
		}

		// writes time stamps to RF data files
		if (rf_data_source_point_id != 4) {
			WORD A[2];
			for (int j = row_start; j <= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr;
				for (int i = 0; i < 2; i++)
				{
					A[i] = (WORD)pData[i];
				}
				unsigned int time_stamps = A[1] * 256 * 256 + A[0];

				if (rf_data_file_handle != NULL)

					fwrite(&time_stamps, sizeof(unsigned int), 1, rf_data_file_handle);
			}
		}
		// for rf_data_source_point_id = 4 
		else {

			WORD A[2];
			for (int j = row_start; j <= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr;
				INT16 *pData1 = arr + 1;

				A[0] = (WORD)pData[0];
				A[1] = (WORD)pData1[0];

				unsigned int time_stamps = A[1] * 256 * 256 + A[0];

				if (rf_data_file_handle != NULL)

					fwrite(&time_stamps, sizeof(unsigned int), 1, rf_data_file_handle);
			}
		}


		if (rf_data_source_point_id == 4)
		{
			for (int j = row_start; j <= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr + 2;
				for (int i = 0; i < length; i++)
				{
					if (rf_data_file_handle != NULL)
						fwrite(pData, sizeof(INT16), 1, rf_data_file_handle);
					pData += flag;
				}
			}
			for (int j = row_start; j <= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = (arr + 3);
				for (int i = 0; i < length; i++)
				{
					if (rf_data_file_handle != NULL)
						fwrite(pData, sizeof(INT16), 1, rf_data_file_handle);
					pData += flag;
				}
			}
		}
		else
		{
			for (int j = row_start; j <= row_end; j += row_step)
			{
				int nOffSet_full = pFrameInfo->UsgFrameInfoHeader.Pitch*j;
				INT16* arr = (INT16*)(pBuffer + nOffSet_full);
				INT16 *pData = arr + 2;
				for (int i = 0; i < length; i++)
				{
					if (rf_data_file_handle != NULL)
						fwrite(pData, sizeof(INT16), 1, rf_data_file_handle);
					pData += flag;
				}
			}
		}
	}

	number_of_KBs = ftell(rf_data_file_handle) / 1024;

	bool tmp_stop = false;
	if ((check_for_stop == false) && (record_single_frame == false) && (recorded_n_frames >= number_of_frames_to_record))
		tmp_stop = true;
	if (do_stop)
		tmp_stop = true;
	if (tmp_stop)
		StopRfWrite();

}

//----------------------------------------------------------------
// begin: sample grabber  callback
//----------------------------------------------------------------
HRESULT __stdcall SampleGrabberCallback::QueryInterface(REFIID riid, void **ppObj)
{
	if ((riid == IID_IUnknown) || (riid == IID_ISampleGrabberCB))
	{
		*ppObj = (void *)(this);
		AddRef();
		return S_OK;
	}
	*ppObj = NULL;
	return E_NOINTERFACE;
}

ULONG STDMETHODCALLTYPE SampleGrabberCallback::AddRef(void)
{
	return InterlockedIncrement(&m_nRefCount);
}

ULONG STDMETHODCALLTYPE SampleGrabberCallback::Release(void)
{
	long nRefCount = 0;
	nRefCount = InterlockedDecrement(&m_nRefCount);
	if (nRefCount == 0)
		delete this;
	return nRefCount;
}

SampleGrabberCallback::SampleGrabberCallback()
	: m_pMediaType(NULL)
{
	m_nRefCount = 0;
	m_plugin = NULL;
	m_usg_wnd = NULL;
	m_usg_x = 0;
	m_usg_y = 0;
}

SampleGrabberCallback::~SampleGrabberCallback()
{
	if (m_pMediaType)
		DeleteMediaType(m_pMediaType);
	m_pMediaType = NULL;
}

void SampleGrabberCallback::SetUsgWnd(CsampleDlg* wnd, int x, int y)
{
	m_usg_wnd = wnd;
	m_usg_x = x;
	m_usg_y = y;
}

void SampleGrabberCallback::SetPlugin(ISampleGrabber* plugin)
{
	m_plugin = plugin;
	if (!m_plugin)
		return;

	AM_MEDIA_TYPE mt;
	if (SUCCEEDED(m_plugin->GetConnectedMediaType(&mt)))
	{
		if (m_pMediaType)
			DeleteMediaType(m_pMediaType);
		m_pMediaType = NULL;
		m_pMediaType = CreateMediaType(&mt);
		FreeMediaType(mt);
	}
}

// Return the number of bytes used to store a scanline for the DIB.
// Rounded up to the nearest 32-bit boundary.
// ScanBytes(infoHeader.biWidth, infoHeader.biBitCount)
int ScanBytes(long w, int bpp)
{
	return (int)(((w*bpp + 31)&(~31)) / 8);
}

STDMETHODIMP SampleGrabberCallback::BufferCB(double SampleTime, BYTE *pBuffer, long BufferLen)
{
	return S_OK;
}

// here we receive RF data
STDMETHODIMP SampleGrabberCallback::SampleCB(double SampleTime, IMediaSample *pSample)
{
	HRESULT hr = S_OK;
	bool fUpdateGUI = false;
	do
	{
		if (!pSample)
			break;

		// first try to retrieve media type
		AM_MEDIA_TYPE *pmt;
		hr = pSample->GetMediaType(&pmt);
		if (hr == S_OK)
		{
			if (m_pMediaType)
				DeleteMediaType(m_pMediaType);
			m_pMediaType = pmt;
			fUpdateGUI = true;
		}

		int nSubFrameIndex = 0;
		IMediaSample2* pSample2;
		AM_SAMPLE2_PROPERTIES aspSampleProps;
		if (SUCCEEDED(pSample->QueryInterface(IID_IMediaSample2, (void **)&pSample2)))
		{
			HRESULT hr = pSample2->GetProperties(sizeof(aspSampleProps), (PBYTE)&aspSampleProps);
			//ATLTRACE2(__FUNCTION__" dwTypeSpecificFlags = 0x%08x\n", aspSampleProps.dwTypeSpecificFlags);
			pSample2->Release();

			if (SUCCEEDED(hr))
			{
				nSubFrameIndex = (aspSampleProps.dwTypeSpecificFlags >> 4) & 0xf;
			}
		}


		if (!m_usg_wnd)
			break;
		if (!m_pMediaType || !m_pMediaType->pbFormat)
			break;

		if (!IsEqualGUID(m_pMediaType->majortype, KSDATAFORMAT_TYPE_ULTRASOUND))
			break;
		if (!IsEqualGUID(m_pMediaType->subtype, KSDATAFORMAT_SUBTYPE_ARTUSDEVICE))
			break;
		if (!IsEqualGUID(m_pMediaType->formattype, KSDATAFORMAT_SPECIFIER_ULTRASOUNDINFORF))
			break;

		BYTE *pBuffer = 0;
		long BufferLen = 0;

		hr = pSample->GetPointer(&pBuffer);
		if (FAILED(hr))
			break;

		if (!pBuffer)
			break;

		BufferLen = pSample->GetActualDataLength();
		if (!BufferLen)
			break;

		PULTRASOUNDRF_INFO pFrameInfo = (PULTRASOUNDRF_INFO)m_pMediaType->pbFormat;

		if (BufferLen >= (long)pFrameInfo->UsgFrameInfoHeader.FrameSize)
			m_usg_wnd->ProcessSample(pBuffer, BufferLen, pFrameInfo, nSubFrameIndex, fUpdateGUI);

	}

	while (false);


	return S_OK;
}


//----------------------------------------------------------------
// end: sample grabber callback
//----------------------------------------------------------------

//----------------------------------------------------------------
// begin: scan converter callback
//----------------------------------------------------------------

HRESULT __stdcall ScanConverterCallback::QueryInterface(REFIID riid, void **ppObj)
{
	if (riid == IID_IUnknown || riid == IID_IUsgfwScanConverterPluginCB)
	{
		*ppObj = (void *)(this);
		AddRef();
		return S_OK;
	}
	*ppObj = NULL;
	return E_NOINTERFACE;
}

//------------------------------

ULONG STDMETHODCALLTYPE ScanConverterCallback::AddRef(void)
{
	return InterlockedIncrement(&m_nRefCount);
}

//------------------------------

ULONG STDMETHODCALLTYPE ScanConverterCallback::Release(void)
{
	long nRefCount = 0;
	nRefCount = InterlockedDecrement(&m_nRefCount);
	if (nRefCount == 0)
		delete this;
	return nRefCount;
}

//------------------------------

ScanConverterCallback::ScanConverterCallback()
{
	m_nRefCount = 0;
	m_plugin = NULL;
	m_usg_wnd = NULL;
	m_usg_x = 0;
	m_usg_y = 0;
}

//------------------------------

ScanConverterCallback::~ScanConverterCallback()
{
}

//------------------------------

void ScanConverterCallback::SetUsgWnd(HWND wnd, int x, int y)
{
	m_usg_wnd = wnd;
	m_usg_x = x;
	m_usg_y = y;
}

//------------------------------

void ScanConverterCallback::SetPlugin(IUsgfwScanConverterPlugin* plugin)
{
	m_plugin = plugin;
}

//------------------------------

HRESULT ScanConverterCallback::BufferCB(THIS_
	PBYTE pBufferIn,
	int nInBufferLen,
	PBYTE pBufferOut,
	int nOutBufferLen,
	int nOutX1,
	int nOutY1,
	int nOutX2,
	int nOutY2
)
{
	if ((m_plugin == NULL) || (m_usg_wnd == NULL) || (pBufferOut == NULL) || (nOutBufferLen == NULL))
		return S_OK;

	AM_MEDIA_TYPE mtOut;
	VIDEOINFO *pVideoInfo = NULL;

	if (m_plugin->GetConnectedMediaType(PINDIR_OUTPUT, &mtOut) != S_OK)
		return S_OK;

	// get output image format
	pVideoInfo = (VIDEOINFO*)mtOut.pbFormat;
	if (pVideoInfo == NULL)
	{
		FreeMediaType(mtOut);
		return S_OK;
	}

	int output_width, output_height, output_bits_per_pixel, output_pitch;
	output_width = pVideoInfo->bmiHeader.biWidth;
	output_height = abs(pVideoInfo->bmiHeader.biHeight);
	output_bits_per_pixel = pVideoInfo->bmiHeader.biBitCount;
	output_pitch = ScanBytes(output_width, output_bits_per_pixel);

	if (IsEqualGUID(mtOut.subtype, MEDIASUBTYPE_RGB555))
		output_bits_per_pixel = 15;

	FreeMediaType(mtOut);

	//------------------------------------------------------
	if (using_rf_stream == false) // if using B stream
	{
		if (freeze_after_scanning_frames_num > 0)
		{
			freeze_after_scanning_frames_num--;
			if (freeze_after_scanning_frames_num <= 0)
			{
				freeze_after_scanning_frames_num = 0;
				::PostMessage(m_usg_wnd, WM_MESSAGE_FREEZE_AFTER_SCANNING, 0, 0);
				//::SendMessage(m_usg_wnd, WM_MESSAGE_FREEZE_AFTER_SCANNING, 0, 0);
			}
		}
	}
	//------------------------------------------------------

	return S_OK;
} // HRESULT ScanConverterCallback::BufferCB

//----------------------------------------------------------------
// end: scan converter callback
//----------------------------------------------------------------

// timer to update FPS
void CsampleDlg::OnTimer(UINT_PTR nIDEvent)
{
	SCAN_STATE scan_state;
	data_view->get_ScanState(&scan_state);
	CINE_STATE state = CINE_STATE_PLAY;
	m_pUsgCine->get_State(&state);

	if (number_of_KBs != 0)
	{
		TCHAR buf[MAX_PATH];
		if (number_of_KBs > 1024)
			swprintf_s(buf, MAX_PATH, L"File size, MB: %d", number_of_KBs / 1024);
		else
			swprintf_s(buf, MAX_PATH, L"File size, KB: %d", number_of_KBs);
		RFFileSize.SetWindowText(buf);
	}

	if ((scan_state == SCAN_STATE_RUN) && (state == CINE_STATE_STOP))
	{
		LARGE_INTEGER fps_time2;
		LARGE_INTEGER fps_freq;
		QueryPerformanceCounter(&fps_time2);
		QueryPerformanceFrequency(&fps_freq);
		double time_diff_s = 1;
		if (fps_freq.QuadPart != 0)
			time_diff_s = (double)(fps_time2.QuadPart - fps_time.QuadPart) / fps_freq.QuadPart;

		fps_time.QuadPart = fps_time2.QuadPart;

		TCHAR buf[MAX_PATH];
		LONGLONG nFramesScannedRF = 0;

		if (m_pUsgCineRF)
			m_pUsgCineRF->get_CurrentPosition(&nFramesScannedRF);
		else
			nFramesScannedRF = m_nFramesScannedRF = 0;

		int tmp_fps = (int)round((double)m_nFramesNumberRF / time_diff_s);
		int tmp_fps2 = (int)round((double)(nFramesScannedRF - m_nFramesScannedRF) / time_diff_s);
		if (using_regs_line_mode)
		{
			if (rf_data_source_point_id == 1) // lines 1, 2 interleaving
			{
				tmp_fps = m_nFramesNumberRF * regs_lines_num / 2;
				tmp_fps2 = tmp_fps2 * regs_lines_num / 2;
			}
			else if (rf_data_source_point_id == 4) // lines 1, 2 simultaneously
			{
				tmp_fps = m_nFramesNumberRF * regs_lines_num;
				tmp_fps2 = tmp_fps2 * regs_lines_num;
			}
			swprintf_s(buf, MAX_PATH, L"Lines FPS: %d / %d", tmp_fps, tmp_fps2);
		}
		else
			swprintf_s(buf, MAX_PATH, L"RF FPS: %d / %d", tmp_fps, tmp_fps2);
		m_nFramesScannedRF = nFramesScannedRF;
		m_lblFpsRF.SetWindowText(buf);

		LONG nFramesDrawn = 0;
		LONGLONG nFramesScanned = 0;

		if (m_pUsgCineB)
			m_pUsgCineB->get_CurrentPosition(&nFramesScanned);
		else
			nFramesScanned = m_nFramesScannedB = 0;

		if (m_pUsgQualProp)
			m_pUsgQualProp->get_FramesDrawn(&nFramesDrawn);
		else
			nFramesDrawn = m_nFramesDrawnB = 0;
		swprintf_s(buf, MAX_PATH, L"B FPS: %d / %d", (int)round((double)(nFramesDrawn - m_nFramesDrawnB) / time_diff_s), (int)round((double)(nFramesScanned - m_nFramesScannedB) / time_diff_s));
		m_nFramesDrawnB = nFramesDrawn;
		m_nFramesScannedB = nFramesScanned;
		m_lblFpsB.SetWindowText(buf);

		if (!m_pUsgQualProp)
			m_nFramesDrawnB = 0;

		m_nFramesNumberRF = 0;
	} // if (scan_state == SCAN_STATE_RUN)

	UpdateCineFrameIndicator();

	if ((scan_state == SCAN_STATE_RUN) && (state == CINE_STATE_PLAY))
		UpdateFrameSliderCurrent();

	__super::OnTimer(nIDEvent);
}

void CsampleDlg::StartTimer(void)
{
	// start timer
	m_nTimer = (int)SetTimer(1, 1000, 0);
}

void CsampleDlg::StopTimer(void)
{
	KillTimer(m_nTimer);
	m_nTimer = 0;
}

// previous cine frame button click
void CsampleDlg::OnBnClickedCinePrev()
{
	do
	{
		if (!m_pUsgCine)
			break;
		CINE_STATE state = CINE_STATE_PLAY;
		m_pUsgCine->get_State(&state);
		if (state == CINE_STATE_PLAY)
			m_pUsgCine->put_State(CINE_STATE_STOP);

		INT64 position;
		INT64 start;
		INT64 end;

		// get current cine position
		m_pUsgCine->get_CurrentPosition(&position);

		// get continuous interval (without time gaps)
		m_pUsgCine->GetContinuousInterval(position, &start, &end);

		// calculate new current position
		position = max(start, position - 1);

		// set new current position
		m_pUsgCine->put_CurrentPosition(position);

		UpdateCineFrameIndicator();
		UpdateFrameSliderCurrent();

	} while (false);
}

// cine play button click
void CsampleDlg::OnBnClickedCinePlay()
{
	do
	{
		if (!m_pUsgCine)
			break;

		CINE_STATE state = CINE_STATE_PLAY;
		m_pUsgCine->get_State(&state);

		if (state == CINE_STATE_PLAY)
			break;

		INT64 position;
		INT64 start;
		INT64 end;

		// get current cine position
		m_pUsgCine->get_CurrentPosition(&position);

		// get continuous interval (without time gaps)
		m_pUsgCine->GetContinuousInterval(position, &start, &end);

		// set stop position
		m_pUsgCine->put_StopPosition(end);


		// if we are at the end of cine, set new current positions at the beginning of cine
		if (position == end)
			m_pUsgCine->put_CurrentPosition(start);

		m_pUsgCine->put_State(CINE_STATE_PLAY);

		UpdateCineFrameIndicator();
		UpdateFrameSliderCurrent();
	}

	while (false);
}

// cine stop button click
void CsampleDlg::OnBnClickedCineStop()
{
	do
	{
		if (!m_pUsgCine)
			break;

		CINE_STATE state = CINE_STATE_STOP;
		m_pUsgCine->get_State(&state);

		if (state == CINE_STATE_STOP)
			break;

		m_pUsgCine->put_State(CINE_STATE_STOP);

		UpdateCineFrameIndicator();
		UpdateFrameSliderCurrent();

	} while (false);
}

// next cine frame button click
void CsampleDlg::OnBnClickedCineNext()
{
	do
	{
		if (!m_pUsgCine)
			break;
		CINE_STATE state = CINE_STATE_PLAY;
		m_pUsgCine->get_State(&state);

		if (state == CINE_STATE_PLAY)
			m_pUsgCine->put_State(CINE_STATE_STOP);

		INT64 position;
		INT64 start;
		INT64 end;

		// get current cine position
		m_pUsgCine->get_CurrentPosition(&position);

		// get continuous interval (without time gaps)
		m_pUsgCine->GetContinuousInterval(position, &start, &end);

		// calculate new current position
		position = min(end, position + 1);

		// set new current position
		m_pUsgCine->put_CurrentPosition(position);

		UpdateCineFrameIndicator();
		UpdateFrameSliderCurrent();

	} while (false);
}

// update string value on cine frame indicator
void CsampleDlg::UpdateCineFrameIndicator()
{
	CString str = L"-/-";

	if ((m_pUsgCine != NULL) && (data_view != NULL))
	{
		SCAN_STATE scan_state;
		data_view->get_ScanState(&scan_state);
		CINE_STATE state = CINE_STATE_PLAY;
		m_pUsgCine->get_State(&state);

		if ((scan_state == SCAN_STATE_RUN) && (state == CINE_STATE_STOP))
		{
			INT64 position;
			INT64 start;
			INT64 end;

			// get current cine position
			m_pUsgCine->get_CurrentPosition(&position);

			// get continuous interval (without time gaps)
			m_pUsgCine->GetContinuousInterval(position, &start, &end);

			int tmp_total;
			tmp_total = (int)(end - start + 1);

			str.Format(L"-/%d", tmp_total);
		}
		else
		{
			INT64 position;
			INT64 start;
			INT64 end;

			// get current cine position
			m_pUsgCine->get_CurrentPosition(&position);

			// get continuous interval (without time gaps)
			m_pUsgCine->GetContinuousInterval(position, &start, &end);

			int tmp_current;
			int tmp_total;
			tmp_current = (int)(position - start + 1);
			tmp_total = (int)(end - start + 1);

			str.Format(L"%d/%d", tmp_current, tmp_total);
		}
	}

	SetDlgItemText(IDC_CINE_FRAME, str);
}

void CsampleDlg::OnBnClickedProbeNameButton()
{

	CMenu menu;
	menu.CreatePopupMenu();

	HRESULT hr = S_OK;
	IUnknown* tmp_obj = NULL;

	IUsgCollection* probes_collection = NULL;

	// get collection of connected probes
	tmp_obj = NULL;
	usgfw2->get_ProbesCollection(&tmp_obj);
	if (tmp_obj == NULL)
	{
		probes_collection = NULL;
		return;
	}

	hr = tmp_obj->QueryInterface(IID_IUsgCollection, (void**)&probes_collection);
	tmp_obj->Release();
	if ((hr != S_OK) || (probes_collection == NULL))
	{
		probes_collection = NULL;
		return;
	}

	// get the number of connected probes
	LONG probes_count = 0;
	probes_collection->get_Count(&probes_count);
	if (probes_count == 0)
	{
		SAFE_RELEASE(probes_collection);
		return;
	}

	for (int i1 = 0; i1 < probes_count; i1++)
	{
		tmp_obj = NULL;
		probes_collection->Item(i1, &tmp_obj);

		IProbe* tmp_probe = NULL;
		if (tmp_obj == NULL)
		{
			tmp_probe = NULL;
			return;
		}
		hr = tmp_obj->QueryInterface(IID_IProbe, (void**)&tmp_probe);
		tmp_obj->Release();

		if ((hr != S_OK) || (probe == NULL))
		{
			tmp_probe = NULL;
			return;
		}

		// get probe name
		CComBSTR bstr;
		tmp_probe->get_Name(&bstr);
		menu.AppendMenu(MF_STRING, i1 + 1, bstr);

		SAFE_RELEASE(tmp_probe);
	}
	SAFE_RELEASE(probes_collection);

	POINT pos;
	::GetCursorPos(&pos);
	int cmd_id;
	cmd_id = menu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD, pos.x, pos.y, this);
	if (cmd_id <= 0)
		return;

	m_probe_idx = cmd_id - 1;

	SaveUsgValues(GetSettingsFn());
	ReleaseUsgControls(false);
	CreateUsgControls();
	InitializeHid();
	LoadUsgValues(GetSettingsFn(), false);
	InitializeUserInterfaceControls();
	EnableDisableUserInterfaceControls();

}

// connected probe
HRESULT CsampleDlg::OnProbeArrive(IUnknown *pUsgProbe, ULONG *reserved)
{
	CString str;
	// *reserved - index in Usgfw2 ProbesCollection
	str.Format(L"OnProbeArrive; idx=%d;", *reserved);
	WriteToLog(str);
	PostMessage(WM_MESSAGE_PROBE_ARRIVE, 0, 0);
	return S_OK;
}

// connected beamformer
HRESULT CsampleDlg::OnBeamformerArrive(IUnknown *pUsgBeamformer, ULONG *reserved)
{
	CString str;
	// *reserved - index in Usgfw2 BeamformersCollection
	str.Format(L"OnBeamformerArrive; idx=%d;", *reserved);
	WriteToLog(str);
	return S_OK;
}

// removed probe
HRESULT CsampleDlg::OnProbeRemove(IUnknown *pUsgProbe, ULONG *reserved)
{
	CString str;
	// *reserved - index in Usgfw2 ProbesCollection
	str.Format(L"OnProbeRemove; idx=%d;", *reserved);
	WriteToLog(str);
	PostMessage(WM_MESSAGE_PROBE_REMOVE, 0, 0);
	return S_OK;
}

// removed beamformer
HRESULT CsampleDlg::OnBeamformerRemove(IUnknown *pUsgBeamformer, ULONG *reserved)
{
	CString str;
	// *reserved - index in Usgfw2 BeamformersCollection
	str.Format(L"OnBeamformerRemove; idx=%d;", *reserved);
	WriteToLog(str);
	return S_OK;
}

HRESULT CsampleDlg::OnProbeStateChanged(IUnknown *pUsgProbe, ULONG *reserved)
{
	WriteToLog(L"OnProbeStateChanged");
	return S_OK;
}

HRESULT CsampleDlg::OnBeamformerStateChanged(IUnknown *pUsgBeamformer, ULONG *reserved)
{
	WriteToLog(L"OnBeamformerStateChanged");
	return S_OK;
}

// save current ultrasound scanning parameters to file
void CsampleDlg::OnBnClickedSaveSettingsToFile()
{
	CFileDialog dlg(FALSE, L"ini", NULL, OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY, L"Settings Files (*.ini)|*.ini|");
	if (dlg.DoModal() == IDOK)
	{
		CString fn;
		fn = dlg.GetPathName();
		SaveUsgValues(fn);
	}
}

// load ultrasound scanning parameters from file and apply
void CsampleDlg::OnBnClickedLoadSettingsFromFile()
{
	CFileDialog dlg(TRUE, L"ini", NULL, OFN_FILEMUSTEXIST, L"Settings Files (*.ini)|*.ini|");
	if (dlg.DoModal() == IDOK)
	{
		CString fn;
		fn = dlg.GetPathName();
		LoadUsgValues(fn, true);
	}
}

// check scanning type radiobutton depending on current value
void CsampleDlg::ScanTypeUpdateGUI()
{
	CheckDlgButton(IDC_RADIO_B_STANDARD, BST_UNCHECKED);
	CheckDlgButton(IDC_RADIO_B_WIDEVIEW, BST_UNCHECKED);
	CheckDlgButton(IDC_RADIO_B_COMPOUND, BST_UNCHECKED);

	if (scan_type_ctrl == NULL)
	{
		CheckDlgButton(IDC_RADIO_B_STANDARD, BST_CHECKED);
		return;
	}
	LONG val;
	scan_type_ctrl->get_Current(&val);
	if (val == SCAN_TYPE_NORMAL)
		CheckDlgButton(IDC_RADIO_B_STANDARD, BST_CHECKED);
	else if (val == SCAN_TYPE_VIRTUALCONVEX)
		CheckDlgButton(IDC_RADIO_B_WIDEVIEW, BST_CHECKED);
	else if (val == SCAN_TYPE_COMPOUND)
		CheckDlgButton(IDC_RADIO_B_COMPOUND, BST_CHECKED);
	else
		CheckDlgButton(IDC_RADIO_B_STANDARD, BST_CHECKED);
}

void CsampleDlg::ScanTypeSet(int id)
{
	if (scan_type_ctrl == NULL) return;
	scan_type_ctrl->put_Current(id);
}

void CsampleDlg::OnBnClickedRadioBStandard()
{
	ScanTypeSet(SCAN_TYPE_NORMAL);
	ScanTypeUpdateGUI();
}

void CsampleDlg::OnBnClickedRadioBWideview()
{
	ScanTypeSet(SCAN_TYPE_VIRTUALCONVEX);
	ScanTypeUpdateGUI();
}

void CsampleDlg::OnBnClickedRadioBCompound()
{
	ScanTypeSet(SCAN_TYPE_COMPOUND);
	ScanTypeUpdateGUI();
}

// record single RF frame button click
void CsampleDlg::OnBnClickedRecordSingleFrameButton()
{
	if (data_view == NULL)
		return;

	record_n_frames = false;
	check_for_stop = false;
	do_stop = false;
	CheckDlgButton(IDC_RADIO_STOP_RF, BST_CHECKED);

	SCAN_STATE tmp_scan_state;
	CINE_STATE tmp_cine_state = CINE_STATE_PLAY;
	data_view->get_ScanState(&tmp_scan_state);
	if (m_pUsgCine != NULL)
		m_pUsgCine->get_State(&tmp_cine_state);

	int old_number_of_frames_to_record;
	old_number_of_frames_to_record = number_of_frames_to_record;

	StartRfWrite();
	record_single_frame = true;
	if ((tmp_scan_state != SCAN_STATE_RUN) && (tmp_cine_state != CINE_STATE_PLAY) && (m_pUsgCine != NULL))
	{
		// set the same position to pass frame through callback
		INT64 position;
		m_pUsgCine->get_CurrentPosition(&position);
		m_pUsgCine->put_CurrentPosition(position);
	}

	number_of_frames_to_record = old_number_of_frames_to_record;
}

void CsampleDlg::RecordNFramesUpdateGUI()
{
	CString str = L"";
	str.Format(L"%d", number_of_frames_to_record);
	m_record_n_frames_edit.SetWindowText(str);
}

// record N RF frames button click
void CsampleDlg::OnBnClickedRecordNFramesButton()
{
	if (data_view == NULL)
		return;
	if (record_n_frames)
		return;

	int ret = 0;

	do
	{
		recorded_n_frames = 0;
		CString str = L"";
		m_record_n_frames_edit.GetWindowText(str);
		int val;
		val = _wtoi(str);
		if (val <= 0)
		{
			ret = -1;
			break;
		}

		int tmp_num_frames = val;

		record_single_frame = false;
		check_for_stop = false;
		do_stop = false;
		CheckDlgButton(IDC_RADIO_STOP_RF, BST_CHECKED);

		if (tmp_num_frames <= 0)
		{
			EnableDisableUserInterfaceControls();
			ret = -2;
			break;
		}

		SCAN_STATE tmp_scan_state;
		CINE_STATE tmp_cine_state = CINE_STATE_PLAY;
		data_view->get_ScanState(&tmp_scan_state);
		if (m_pUsgCine != NULL)
			m_pUsgCine->get_State(&tmp_cine_state);

		if (tmp_scan_state == SCAN_STATE_RUN)
		{
			number_of_frames_to_record = tmp_num_frames;
			StartRfWrite();
			record_n_frames = true;
			EnableDisableUserInterfaceControls();
		}
		else
		{
			if ((tmp_cine_state != CINE_STATE_PLAY) && (m_pUsgCine != NULL))
			{
				INT64 position;
				INT64 start;
				INT64 end;
				m_pUsgCine->get_CurrentPosition(&position);
				m_pUsgCine->GetContinuousInterval(position, &start, &end);

				if ((position + tmp_num_frames - 1) <= end)
				{
					number_of_frames_to_record = tmp_num_frames;
					StartRfWrite();
					record_n_frames = true;
					for (int i = 0; i < tmp_num_frames; i++)
						m_pUsgCine->put_CurrentPosition(position + i);
				}
				else
				{
					ret = -3;
					record_n_frames = false;
				}
			}
			UpdateCineFrameIndicator();
			UpdateFrameSliderCurrent();
		}

		EnableDisableUserInterfaceControls();
	} while (false);

	if (ret != 0)
		MessageBox(L"Error. Please check cine and saving frame numbers.", L"Error", MB_OK);

}

// -----------------------------------------------
// begin: HID
// -----------------------------------------------

// find our HID device (with specified VID, PID, Usage Page, Usage) and start using it
void CsampleDlg::SetHidDevice()
{
	StopReading();			// before setting new device, stop using current device
	m_hid_device = NULL;

	CHidDevice* pHidDevice;
	pHidDevice = (CHidDevice*)GetFirstHidDevice();

	// loop through all hid devices and find our device
	while (pHidDevice)
	{
		if ((pHidDevice->m_HidAttributes.VendorID == m_vid)
			&& (pHidDevice->m_HidAttributes.ProductID == m_pid)
			&& (pHidDevice->m_HidCaps.UsagePage == m_usage_page)
			&& (pHidDevice->m_HidCaps.Usage == m_usage)
			&& (pHidDevice->m_pHidPpd != NULL)
			)
		{
			m_hid_device = pHidDevice;
			pHidDevice = NULL;
			StartReading(); // start reading input HID reports from device
		}
		else
		{
			pHidDevice = (CHidDevice*)GetNextHidDevice();
		}
	} // while (pHidDevice)

}

void CsampleDlg::HIDCallbackFunction(CsampleDlg *context, CHidDevice *pHidDevice, UINT uHidCode)
{
	if (g_app_closing != 0)
		return;

	// added HID device
	if (uHidCode == HIDCB_NOTIFY_DEVICE_ADDED)
	{
		if (pHidDevice != NULL)
		{
			int vid, pid;
			vid = pHidDevice->m_HidAttributes.VendorID;
			pid = pHidDevice->m_HidAttributes.ProductID;
			//if (context != NULL)
			//	context->
		}

		if (context != NULL)
			context->SetHidDevice();
	}
	// removed HID device
	else if (uHidCode == HIDCB_NOTIFY_DEVICE_REMOVED)
	{
		if (pHidDevice != NULL)
		{
			int vid, pid;
			vid = pHidDevice->m_HidAttributes.VendorID;
			pid = pHidDevice->m_HidAttributes.ProductID;
			//if (context != NULL)
			//	context->
		}

		if (context != NULL)
			context->SetHidDevice();
	}

}

// freeze/run message handler
LRESULT CsampleDlg::OnHidFreeze(WPARAM wParam, LPARAM lParam)
{
	FreezeRun();
	return 0;
}

// process received input HID report
void CsampleDlg::HandleReceivedData(HWND output_wnd)
{
	if (m_hid_device == NULL)
		return;
	if (m_hid_device->IsOpenForRead() == false)
		return;

	int len;
	len = m_hid_device->m_HidCaps.InputReportByteLength;
	if (len <= 1)
		return;

	//CString str;
	//CString str1;
	//str = "";
	//for (int i1=1; i1<len; i1++)
	//{
	//	str1 = "";
	//	str1.Format(L"0x%02X ", (byte)(m_hid_device->m_pInputReportBuffer[i1]));
	//	str += str1;
	//}
	//str += "\r\n";
	//OutputDebugString(str);

	if ((m_hid_device->m_pInputReportBuffer[1] == 1) && (output_wnd != NULL))
		::PostMessage(output_wnd, WM_MESSAGE_HID_FREEZE, 0, 0);
}

// start thread for reading input reports
void CsampleDlg::StartReading()
{
	if (m_hid_device == NULL)
		return;
	if (m_hid_device->IsOpenForRead() == false)
		return;
	ZeroMemory(&m_overlapped, sizeof(OVERLAPPED));
	m_overlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	m_thread_stop = 0;
	g_thread_finished = 0;

	m_read_thread = AfxBeginThread(&CsampleDlg::ReadThreadProc, this);
	if (m_read_thread != NULL)
		m_read_thread_id = m_read_thread->m_nThreadID;
}

// stop reading input reports and wait for thread completion
void CsampleDlg::StopReading()
{
	InterlockedExchange(&m_thread_stop, 1);
	for (int i1 = 0; i1 < 10; i1++)
	{
		if (g_thread_finished <= 0)
			Sleep(100);
	}

	m_read_thread_id = 0;
	m_read_thread = NULL;

	g_thread_finished = 1;
	if (m_overlapped.hEvent != NULL)
	{
		CloseHandle(m_overlapped.hEvent);
		m_overlapped.hEvent = NULL;
	}
}

// read input report from HID device
// if is returned false, then reading loop (and thread) will be finished
bool CsampleDlg::ReadReport(HWND output_wnd)
{
	if (m_thread_stop > 0)
		return false;
	if (g_app_closing > 0)
		return false;
	if (output_wnd == NULL)
		return false;

	if (m_hid_device == NULL)
		return false;
	if (m_hid_device->IsOpenForRead() == false)
		return false;

	DWORD bytesRead;

	BOOL res;
	// read input report from HID device
	res = ReadFile(m_hid_device->GetDeviceHandle(),
		m_hid_device->m_pInputReportBuffer,
		m_hid_device->m_HidCaps.InputReportByteLength,
		&bytesRead,
		&m_overlapped);
	if (!res)
	{
		if (GetLastError() == ERROR_IO_PENDING)
		{
			DWORD resW = WaitForSingleObject(m_overlapped.hEvent, 100);
			if (resW != WAIT_OBJECT_0) // timeout
			{
				if (m_thread_stop > 0)
				{
					return false;
				}
				else
				{
					CancelIo(m_hid_device->GetDeviceHandle());
					return true; // true, because we do not want to terminate reading
				}
			}

			if (m_hid_device == NULL)
				return false;
			res = GetOverlappedResult(m_hid_device->GetDeviceHandle(), &m_overlapped, &bytesRead, FALSE);
		}

	}
	if (!res)
		return false;

	if (m_hid_device == NULL)
		return false;
	if (bytesRead != m_hid_device->m_HidCaps.InputReportByteLength)
		return false;
	if (m_thread_stop > 0)
		return false;

	HandleReceivedData(output_wnd);

	if (m_thread_stop > 0)
		return false;

	return true;

}

// reading thread procedure
UINT CsampleDlg::ReadThreadProc(LPVOID pParam)
{
	CsampleDlg* dlg = (CsampleDlg*)pParam;

	if (dlg == NULL)
	{
		InterlockedIncrement(&g_thread_finished);
		return 1;
	}

	// read reports
	while (dlg->ReadReport(((CDialog*)dlg)->m_hWnd))
	{
	}

	InterlockedIncrement(&g_thread_finished);

	AfxEndThread(0, 1);

	return 0;

}

// -----------------------------------------------
// end: HID
// -----------------------------------------------

void CsampleDlg::OptionsSave()
{
	CString fn;
	fn = OptionsGetFn();

	CString section = L"Default";
	CString str = L"";

	str.Format(L"%d", rf_data_source_point_id);
	WritePrivateProfileString(section, L"rf_data_source_point_id", str, fn);

	if (rf_iq_autofit)
		str = L"1";
	else
		str = L"0";
	WritePrivateProfileString(section, L"rf_iq_autofit", str, fn);

	str.Format(L"%d", number_of_frames_to_record);
	WritePrivateProfileString(section, L"rf_number_of_frames_to_record", str, fn);

	str.Format(L"%f", rf_data_i_scale2);
	WritePrivateProfileString(section, L"rf_data_i_scale", str, fn);
	str.Format(L"%f", rf_data_q_scale2);
	WritePrivateProfileString(section, L"rf_data_q_scale", str, fn);

	str.Format(L"%d", display_range);
	WritePrivateProfileString(section, L"rf_iq_display_range_bits", str, fn);

	str.Format(L"%d", cine_size_mb);
	WritePrivateProfileString(section, L"cine_size_mb", str, fn);
}

void CsampleDlg::RfDataSourcePointUpdateGui()
{
	CheckDlgButton(IDC_RADIO_AFTER_SUM, BST_UNCHECKED);
	CheckDlgButton(IDC_RADIO_AFTER_TFC, BST_UNCHECKED);
	CheckDlgButton(IDC_RADIO_AFTER_APODIZATION, BST_UNCHECKED);
	CheckDlgButton(IDC_RADIO_AFTER_HILBERT, BST_UNCHECKED);

	if (rf_data_source_point_id == 1)
		CheckDlgButton(IDC_RADIO_AFTER_SUM, BST_CHECKED);
	else if (rf_data_source_point_id == 2)
		CheckDlgButton(IDC_RADIO_AFTER_TFC, BST_CHECKED);
	else if (rf_data_source_point_id == 3)
		CheckDlgButton(IDC_RADIO_AFTER_APODIZATION, BST_CHECKED);
	else if (rf_data_source_point_id == 4)
		CheckDlgButton(IDC_RADIO_AFTER_HILBERT, BST_CHECKED);

	UpdateIScaleGUI();
	UpdateQScaleGUI();
}

void CsampleDlg::OptionsLoad()
{
	CString fn;
	fn = OptionsGetFn();

	const int BUF_SIZE = 64;
	TCHAR buf[BUF_SIZE];
	CString section = L"Default";

	CString str;

	if (GetPrivateProfileString(section, L"rf_data_source_point_id", L"", buf, BUF_SIZE, fn) > 0)
		rf_data_source_point_id = _wtoi(buf);
	else
		rf_data_source_point_id = 4;
	SetRfDataSourceByPointId(rf_data_source_point_id);

	if (GetPrivateProfileString(section, L"rf_iq_autofit", L"", buf, BUF_SIZE, fn) > 0)
	{
		if (_wtoi(buf) == 1)
			rf_iq_autofit = true;
		else
			rf_iq_autofit = false;
	}

	if (GetPrivateProfileString(section, L"rf_number_of_frames_to_record", L"", buf, BUF_SIZE, fn) > 0)
		number_of_frames_to_record = _wtoi(buf);
	else
		number_of_frames_to_record = 100;
	if (number_of_frames_to_record <= 0)
		number_of_frames_to_record = 100;

	if (GetPrivateProfileString(section, L"rf_data_i_scale", L"", buf, BUF_SIZE, fn) > 0)
		rf_data_i_scale2 = _wtof(buf);
	else
		rf_data_i_scale2 = 1;
	if (rf_data_i_scale2 <= 0)
		rf_data_i_scale2 = 1;

	if (GetPrivateProfileString(section, L"rf_data_q_scale", L"", buf, BUF_SIZE, fn) > 0)
		rf_data_q_scale2 = _wtof(buf);
	else
		rf_data_q_scale2 = 1;
	if (rf_data_q_scale2 <= 0)
		rf_data_q_scale2 = 1;

	if (GetPrivateProfileString(section, L"rf_iq_display_range_bits", L"", buf, BUF_SIZE, fn) > 0)
		display_range = _wtoi(buf);
	else
		display_range = 8;
	if (display_range < 2)
		display_range = 8;
	if (display_range > 16)
		display_range = 16;

	if (GetPrivateProfileString(section, L"cine_size_mb", L"", buf, BUF_SIZE, fn) > 0)
		cine_size_mb = _wtoi(buf);
	else
		cine_size_mb = 1;
	if (cine_size_mb < 0)
		cine_size_mb = 1;
}

CString CsampleDlg::OptionsGetFn()
{
	TCHAR path[_MAX_PATH];
	TCHAR fn[_MAX_PATH];
	TCHAR drive[_MAX_DRIVE];
	TCHAR dir[_MAX_DIR];
	TCHAR fname[_MAX_FNAME];
	TCHAR ext[_MAX_EXT];
	GetModuleFileName(NULL, path, sizeof(path));
	_wsplitpath(path, drive, dir, fname, ext);
	_wmakepath(fn, drive, dir, L"options", L"ini");
	return CString(fn);
}

void CsampleDlg::UpdateAutofitGUI()
{
	if (rf_iq_autofit)
		CheckDlgButton(IDC_AUTOFIT_CHECK, BST_CHECKED);
	else
		CheckDlgButton(IDC_AUTOFIT_CHECK, BST_UNCHECKED);

	CString str;
	if (using_regs_line_mode)
		str = L"Lines 1, 2 Autofit";
	else
		str = L"I, Q Autofit";
	SetDlgItemText(IDC_AUTOFIT_CHECK, str);

}

void CsampleDlg::OnBnClickedAutofitCheck()
{
	rf_iq_autofit = !rf_iq_autofit;
	PaintRFData();
	OptionsSave();
	EnableDisableUserInterfaceControls();
}

void CsampleDlg::DisplayRangeUpdateGUI()
{
	CString str;
	if (using_regs_line_mode)
		str.Format(L"Lines 1, 2 Display Range: %d bits", display_range);
	else
		str.Format(L"I, Q Display Range: %d bits", display_range);
	SetDlgItemText(IDC_DISPLAY_RANGE_STATIC, str);
}

void CsampleDlg::OnBnClickedDisplayRangePrevButton()
{
	display_range = display_range - 1;
	if (display_range < 2)
		display_range = 2;
	DisplayRangeUpdateGUI();
	PaintRFData();
	OptionsSave();
}

void CsampleDlg::OnBnClickedDisplayRangeNextButton()
{
	display_range = display_range + 1;
	if (display_range > 16)
		display_range = 16;
	DisplayRangeUpdateGUI();
	PaintRFData();
	OptionsSave();
}

// ----------------------------------------
// begin: line registers
// ----------------------------------------

void CsampleDlg::RegsReset()
{
	regs_line1_idx = -1;
	regs_line2_idx = -1;
	m_edtCtrl1.SetWindowText(L"0");
	m_edtCtrl2.SetWindowText(L"0");

	if (!m_iKsControl)
		return;

	DWORD cbReturned = 0;
	KSPROPERTY_USBARTUSREGS_S prop;
	WORD wDataBuffer[2] = { 0,0 };

	prop.property.Set = KSPROPSETID_UsbArtUsRegisters;
	prop.property.Id = KSPROPERTY_MEMORY_REGS;
	prop.property.Flags = KSPROPERTY_TYPE_SET;
	prop.dwRegistersArray = 0;
	prop.dwRegistersOffset = 69;

	bool done_freeze = false;
	SCAN_STATE state;
	data_view->get_ScanState(&state);
	if (state != SCAN_STATE_FREEZE)
	{
		data_view->put_ScanState(SCAN_STATE_FREEZE);
		done_freeze = true;
	}

	m_iKsControl->KsProperty((PKSPROPERTY)&prop, sizeof(KSPROPERTY_USBARTUSREGS_S), wDataBuffer, sizeof(WORD) * 2, &cbReturned);

	if (done_freeze)
		data_view->put_ScanState(SCAN_STATE_RUN);
}

void CsampleDlg::RegsAttach()
{
	do
	{
		if ((data_view == NULL) || (probe == NULL))
			return;

		CComBSTR device_path = L"";

		IUnknown* tmp_obj = NULL;
		probe->get_Beamformer(&tmp_obj);
		if (tmp_obj != NULL)
		{
			IBeamformer* tmp_beamformer = NULL;
			HRESULT hr = S_OK;
			hr = tmp_obj->QueryInterface(IID_IBeamformer, (void**)&tmp_beamformer);
			tmp_obj->Release();
			if ((hr != S_OK) || (tmp_beamformer == NULL))
				tmp_beamformer = NULL;
			if (tmp_beamformer != NULL)
			{
				tmp_beamformer->get_DisplayName(&device_path);
			}
			SAFE_RELEASE(tmp_beamformer);
		}


		CComPtr<IMoniker> moniker;
		IBaseFilter* pBaseFilter = NULL;

		CComPtr<ICreateDevEnum> devEnum;
		HRESULT hr = devEnum.CoCreateInstance(CLSID_SystemDeviceEnum);
		if (hr != S_OK)
			break;

		CComPtr<IEnumMoniker> enumMoniker;
		hr = devEnum->CreateClassEnumerator(CLSID_UltrasonicBeamformer, &enumMoniker, 0);
		if (hr != S_OK)
			break;

		LPOLESTR pDevicePath = NULL;
		while (enumMoniker->Next(1, &moniker, NULL) == S_OK)
		{
			LPOLESTR pDisplayName = NULL;
			if (moniker->GetDisplayName(NULL, NULL, &pDisplayName) != S_OK)
			{
				moniker.Release();
				continue;
			}

			do
			{
				CComPtr<IPersist> pPersist;

				if (moniker->BindToStorage(0, 0, IID_IPersist, (void **)&pPersist) != S_OK)
					break;

				VARIANT varFilterClsid;
				varFilterClsid.vt = VT_BSTR;
				CLSID clsidFilter;

				// Read CLSID string from property bag   
				hr = pPersist->GetClassID(&clsidFilter);

			} while (false);

			// check if device is supported
			_wcsupr(pDisplayName);
			bool ret = (wcsstr(pDisplayName, L"VID_FFFF&PID_3430") != NULL)
				|| (wcsstr(pDisplayName, L"VID_FFFF&PID_3436") != NULL)
				|| (wcsstr(pDisplayName, L"VID_FFFF&PID_3438") != NULL)
				|| (wcsstr(pDisplayName, L"VID_FFFF&PID_343B") != NULL);

			// check if enumerated device is device for that we created data_view
			bool ret2 = (_wcsicmp(pDisplayName, device_path) == 0);

			CoTaskMemFree(pDisplayName);

			if (!ret || !ret2)
			{
				moniker.Release();
				continue;
			}

			hr = moniker->BindToObject(NULL, NULL, __uuidof(IBaseFilter), (LPVOID*)&pBaseFilter);
			if (hr != S_OK)
				pBaseFilter = NULL;

			moniker.Release();
			if (!pBaseFilter)
				continue;

			m_iKsControl = pBaseFilter;
			pBaseFilter->Release();
			pBaseFilter = NULL;

			if (m_iKsControl)
				break;
		}

		if (!m_iKsControl)
			break;

		// check if direct control property is available
		ULONG cbReturned = 0;
		ULONG uFlags = 0;
		KSPROPERTY KsProperty;

		KsProperty.Set = KSPROPSET_UsbEchoBlasterControl;
		KsProperty.Id = KSPROPERTY_USBRAWCTRL;
		KsProperty.Flags = KSPROPERTY_TYPE_BASICSUPPORT;

		hr = m_iKsControl->KsProperty(&KsProperty, sizeof(KSPROPERTY), &uFlags, sizeof(ULONG), &cbReturned);
		if (FAILED(hr))
		{
			m_iKsControl.Release();
			break;
		}
		if ((cbReturned < sizeof(ULONG)) || !(uFlags &  KSPROPERTY_TYPE_SET) || !(uFlags &  KSPROPERTY_TYPE_GET))
		{
			m_iKsControl.Release();
		}

		cbReturned = 0;
		uFlags = 0;

		KsProperty.Set = KSPROPSETID_UsbArtUsRegisters;
		KsProperty.Id = KSPROPERTY_MEMORY_REGS;
		KsProperty.Flags = KSPROPERTY_TYPE_BASICSUPPORT;

		hr = m_iKsControl->KsProperty(&KsProperty, sizeof(KSPROPERTY), &uFlags, sizeof(ULONG), &cbReturned);
		if (FAILED(hr))
			break;
		if ((cbReturned < sizeof(ULONG)) || !(uFlags &  KSPROPERTY_TYPE_SET) || !(uFlags &  KSPROPERTY_TYPE_GET))
			break;

	} while (false);
}

int CsampleDlg::RegsGetLine1()
{
	CString str = L"";
	m_edtCtrl1.GetWindowText(str);
	int val;
	val = _wtoi(str);
	return val;
}

int CsampleDlg::RegsGetLine2()
{
	CString str = L"";
	m_edtCtrl2.GetWindowText(str);
	int val;
	val = _wtoi(str);
	return val;
}

void CsampleDlg::RegsWrite()
{
	if (!data_view)
		return;

	if (!m_iKsControl)
	{
		using_regs_line_mode = false;
		return;
	}

	WORD wDataBuffer[2];
	wDataBuffer[0] = 0;
	wDataBuffer[1] = 0;

	regs_line1_idx = RegsGetLine1() - 1;
	regs_line2_idx = RegsGetLine2() - 1;

	if (scan_line_properties_b != NULL)
	{
		wDataBuffer[0] = (WORD)regs_line1_idx;
		wDataBuffer[1] = (WORD)regs_line2_idx;

		LONG min_line_idx, max_line_idx;
		scan_line_properties_b->GetScanLinesRange(&min_line_idx, &max_line_idx);
		bool write_zeros = false;
		if ((regs_line1_idx == -1) || (regs_line2_idx == -1))
			write_zeros = true;
		else if ((regs_line1_idx < min_line_idx) || (regs_line1_idx > max_line_idx) || (regs_line2_idx < min_line_idx) || (regs_line2_idx > max_line_idx))
		{
			write_zeros = true;
			CString str = L"";
			str.Format(L"Error. Allowed line numbers are from interval [%d;%d] or 0 to reset.", min_line_idx + 1, max_line_idx + 1);
			MessageBox(str, L"Error", MB_OK);
		}
		if (write_zeros)
		{
			wDataBuffer[0] = 0;
			wDataBuffer[1] = 0;
		}
	}

	DWORD cbReturned = 0;
	KSPROPERTY_USBARTUSREGS_S prop;

	prop.property.Set = KSPROPSETID_UsbArtUsRegisters;
	prop.property.Id = KSPROPERTY_MEMORY_REGS;
	prop.property.Flags = KSPROPERTY_TYPE_SET;
	prop.dwRegistersArray = 0;
	prop.dwRegistersOffset = 69;

	bool done_freeze = false;
	SCAN_STATE state;
	data_view->get_ScanState(&state);
	if (state != SCAN_STATE_FREEZE)
	{
		data_view->put_ScanState(SCAN_STATE_FREEZE);
		done_freeze = true;
	}

	m_iKsControl->KsProperty((PKSPROPERTY)&prop, sizeof(KSPROPERTY_USBARTUSREGS_S), wDataBuffer, sizeof(WORD) * 2, &cbReturned);

	if (done_freeze)
		data_view->put_ScanState(SCAN_STATE_RUN);
}

void CsampleDlg::RegsClose()
{
	regs_line1_idx = -1;
	regs_line2_idx = -1;
	if (m_iKsControl)
		m_iKsControl.Release();
}

// set window width from first to last line
void CsampleDlg::RegsSetWindowWidth()
{
	if (!m_pUsgWindowRF)
		return;

	LONG depth1, depth2;
	LONG line1, line2;

	// get current RF window size
	m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);

	VALUE_STEPPING LinesRange;
	m_pUsgWindowRF->get_LinesRange(&LinesRange);
	line1 = LinesRange.minimum;
	line2 = LinesRange.maximum;

	// set new RF window size
	m_pUsgWindowRF->SetWindow(line1, line2, depth1, depth2);
}

// read the number of lines
void CsampleDlg::RegsUpdateLinesNum()
{
	if (!m_pUsgWindowRF)
		return;
	LONG depth1, depth2;
	LONG line1, line2;
	m_pUsgWindowRF->GetWindow(&line1, &line2, &depth1, &depth2);
	regs_lines_num = line2 - line1 + 1;
}

void CsampleDlg::OnBnClickedRegsWriteButton()
{
	if ((RegsGetLine1() > 0) && (RegsGetLine2() > 0))
	{
		using_regs_line_mode = true;
		if ((rf_data_source_point_id != 1) && (rf_data_source_point_id != 4))
		{
			rf_data_source_point_id = 1;
			RfDataSourcePointUpdateGui();
			SetRfDataSourceByPointId(rf_data_source_point_id);
		}
	}
	else
		using_regs_line_mode = false;

	if (using_regs_line_mode)
		ScanTypeSet(SCAN_TYPE_NORMAL);
	ScanTypeUpdateGUI();

	if (using_regs_line_mode)
	{
		B_LinesDensitySet(LINE_DENSITY_STANDARD);
		RegsSetWindowWidth();
	}
	RegsUpdateLinesNum();
	RegsWrite();

	PostMessage(WM_MESSAGE_UPDATE_BITMAP, 0, 0);
	UpdateIScaleGUI();
	UpdateQScaleGUI();
	DisplayRangeUpdateGUI();
	UpdateAutofitGUI();
	UpdateLineIndicator();
	EnableDisableUserInterfaceControls();
}

// ----------------------------------------
// end: line registers
// ----------------------------------------

void CsampleDlg::OnBnClickedGoToFrameButton()
{
	if (data_view == NULL)
		return;
	if (!m_pUsgCine)
		return;

	CString str = L"";
	go_to_frame_edit_ctrl.GetWindowText(str);
	int val;
	val = _wtoi(str);
	if (val <= 0)
	{
		MessageBox(L"Error. Please check frame number.", L"Error", MB_OK);
		return;
	}

	INT64 position;
	INT64 start;
	INT64 end;
	m_pUsgCine->get_CurrentPosition(&position);
	m_pUsgCine->GetContinuousInterval(position, &start, &end);

	position = start + val - 1;
	if (position > end)
		position = end;

	m_pUsgCine->put_CurrentPosition(position);

	UpdateCineFrameIndicator();
	UpdateFrameSliderCurrent();
}

void CsampleDlg::UpdateFrameSliderRange()
{
	if (data_view == NULL)
		return;
	if (!m_pUsgCine)
		return;

	int val_count = 1;

	SCAN_STATE state;
	data_view->get_ScanState(&state);

	INT64 position;
	INT64 start;
	INT64 end;
	m_pUsgCine->get_CurrentPosition(&position);
	m_pUsgCine->GetContinuousInterval(position, &start, &end);

	val_count = (int)(end - start + 1);

	HWND hslider = NULL;
	hslider = (HWND)(*(GetDlgItem(IDC_FRAME_SLIDER)));
	if (hslider == NULL)
		return;

	int tmp_page;
	tmp_page = (int)::SendMessage(hslider, TBM_GETPAGESIZE, 0L, 0L);
	if (tmp_page != 1)
		::SendMessage(hslider, TBM_SETPAGESIZE, 0, 1);

	int tmp_min, tmp_max;
	tmp_min = (int)::SendMessage(hslider, TBM_GETRANGEMIN, 0L, 0L);
	tmp_max = (int)::SendMessage(hslider, TBM_GETRANGEMAX, 0L, 0L);

	if ((tmp_min != 0) || (tmp_max != val_count - 1))
		::SendMessage(hslider, TBM_SETRANGE, TRUE, (LPARAM)MAKELONG(0, val_count - 1));
}

void CsampleDlg::UpdateFrameSliderCurrent()
{
	if (data_view == NULL)
		return;
	if (!m_pUsgCine)
		return;

	SCAN_STATE state;
	data_view->get_ScanState(&state);

	INT64 position;
	INT64 start;
	INT64 end;
	m_pUsgCine->get_CurrentPosition(&position);
	m_pUsgCine->GetContinuousInterval(position, &start, &end);

	int val_idx;
	val_idx = (int)(position - start);

	HWND hslider = NULL;
	hslider = (HWND)(*(GetDlgItem(IDC_FRAME_SLIDER)));
	if (hslider == NULL)
		return;

	// perform asynchronous trackbar update in order to avoid locking
	if (val_idx >= 0)
	{
		int tmp_pos;
		tmp_pos = (int)::SendMessage(hslider, TBM_GETPOS, 0, 0);
		if (val_idx != tmp_pos)
			::PostMessage(hslider, TBM_SETPOS, 1, val_idx);
	}
}

void CsampleDlg::FrameSliderGoToFrame(int idx)
{
	if (data_view == NULL)
		return;
	if (!m_pUsgCine)
		return;
	if (idx < 0)
		return;

	SCAN_STATE state;
	data_view->get_ScanState(&state);
	if (state != SCAN_STATE_FREEZE)
		return;

	INT64 position;
	INT64 start;
	INT64 end;
	m_pUsgCine->get_CurrentPosition(&position);
	m_pUsgCine->GetContinuousInterval(position, &start, &end);
	LONGLONG val;
	val = start + idx;
	if (val > end)
		val = end;

	m_pUsgCine->put_CurrentPosition(val);

	UpdateCineFrameIndicator();
}

// save RF frames from-to
void CsampleDlg::OnBnClickedSaveButton()
{
	if (data_view == NULL)
		return;
	if (!m_pUsgCine)
		return;

	int ret = 0;
	do
	{
		int save_from = 1;
		int save_to = 1;
		CString str = L"";

		save_from_edit.GetWindowText(str);
		save_from = _wtoi(str);

		save_to_edit.GetWindowText(str);
		save_to = _wtoi(str);

		int tmp_num_frames;
		tmp_num_frames = save_to - save_from + 1;

		if ((save_from <= 0) || (save_to <= 0) || (tmp_num_frames <= 0))
		{
			ret = -1;
			break;
		}

		if (tmp_num_frames <= 0)
		{
			EnableDisableUserInterfaceControls();
			ret = -2;
			break;
		}

		CINE_STATE tmp_cine_state = CINE_STATE_PLAY;
		m_pUsgCine->get_State(&tmp_cine_state);
		if (tmp_cine_state == CINE_STATE_PLAY)
			m_pUsgCine->put_State(CINE_STATE_STOP);

		INT64 position;
		INT64 start;
		INT64 end;
		m_pUsgCine->get_CurrentPosition(&position);
		m_pUsgCine->GetContinuousInterval(position, &start, &end);

		if ((start + save_from - 1) > end)
		{
			ret = -3;
			break;
		}
		if ((start + save_to - 1) > end)
		{
			ret = -4;
			break;
		}

		recorded_n_frames = 0;
		record_single_frame = false;
		check_for_stop = false;
		do_stop = false;
		CheckDlgButton(IDC_RADIO_STOP_RF, BST_CHECKED);

		StartRfWrite();

		int old_number_of_frames_to_record;
		old_number_of_frames_to_record = number_of_frames_to_record;

		number_of_frames_to_record = tmp_num_frames;
		record_n_frames = true;
		for (int i = 0; i < tmp_num_frames; i++)
			m_pUsgCine->put_CurrentPosition(start + save_from - 1 + i);

		number_of_frames_to_record = old_number_of_frames_to_record;

		UpdateCineFrameIndicator();
		UpdateFrameSliderCurrent();
		EnableDisableUserInterfaceControls();
	} while (false);

	if (ret != 0)
		MessageBox(L"Error. Please check cine and saving frame numbers.", L"Error", MB_OK);
}

void CsampleDlg::OnBnClickedOptionsButton()
{
	COptionsDialog dlg;
	dlg.cine_size_mb = cine_size_mb;
	dlg.max_cine_size_mb = MAX_CINE_BUFFER_SIZE_MB;

	if (m_pUsgCineSink != NULL)
	{
		LONG total, used, write_pos;
		m_pUsgCineSink->GetBufferSize(&total, &used, &write_pos);
		dlg.current_cine_size_mb = total / (1024 * 1024);
	}

	INT_PTR nRet = dlg.DoModal();
	if (nRet == IDOK)
	{
		//if (cine_size_mb != dlg.cine_size_mb)
		{
			int tmp_cine_size;
			tmp_cine_size = dlg.cine_size_mb;
			if (tmp_cine_size > MAX_CINE_BUFFER_SIZE_MB)	// memory limit for 32-bit process is 2GB; some memory is required for GUI itself; the rest can be used for cine;
				tmp_cine_size = MAX_CINE_BUFFER_SIZE_MB;

			MEMORYSTATUSEX statex;
			statex.dwLength = sizeof(statex);
			GlobalMemoryStatusEx(&statex);
			// limit cine size if with newly allocated cine will be left less than nnn MB free physical memory
			if ((statex.ullAvailPhys / (1024 * 1024) + dlg.current_cine_size_mb - tmp_cine_size) < 300)
				tmp_cine_size = (int)(statex.ullAvailPhys / (1024 * 1024)) + dlg.current_cine_size_mb - 300;
			if (tmp_cine_size < 0)
				tmp_cine_size = 1;

			cine_size_mb = tmp_cine_size;
			if ((m_pUsgCineSink != NULL) && (cine_size_mb > 0))
				m_pUsgCineSink->SetBufferSize(cine_size_mb * 1024 * 1024);
		}

	}
	else
	{
	}
}

